local _, core = ...;
local _G = _G;
local L = core.L;

local DKP = core.DKP;
local GUI = core.GUI;
local Util = core.Util;
local PDKP = core.PDKP;
local Raid = core.Raid;
local Guild = core.Guild;
local Shroud = core.Shroud;
local Defaults = core.defaults;
local Import = core.Import;
local Comms = core.Comms;

local pdkpTestDatabase = {
        addon_version = '0.9.6',
        full = false,
        guildDB = {
            numOfMembers = 410,
            members = {
                        {
                            ["inRaid"] = false,
                            ["class"] = "Paladin",
                            ["dkpTotal"] = 0,
                            ["online"] = false,
                            ["rankIndex"] = 7,
                            ["formattedName"] = "|cffF58CBAThepink|r",
                            ["bName"] = "pdkp_dkp_entry1",
                            ["name"] = "Thepink",
                            ["canEdit"] = false,
                        }, -- [1]
                        {
                            ["dkpTotal"] = 0,
                            ["online"] = false,
                            ["name"] = "Wazzoo",
                            ["formattedName"] = "|cff40C7EBWazzoo|r",
                            ["canEdit"] = false,
                            ["class"] = "Mage",
                            ["inRaid"] = false,
                            ["rankIndex"] = 7,
                        }, -- [2]
                        {
                            ["dkpTotal"] = 0,
                            ["online"] = false,
                            ["name"] = "Angelyheth",
                            ["formattedName"] = "|cffABD473Angelyheth|r",
                            ["canEdit"] = false,
                            ["class"] = "Hunter",
                            ["inRaid"] = false,
                            ["rankIndex"] = 6,
                        }, -- [3]
                        {
                            ["inRaid"] = false,
                            ["class"] = "Hunter",
                            ["dkpTotal"] = 0,
                            ["online"] = false,
                            ["rankIndex"] = 6,
                            ["formattedName"] = "|cffABD473Fardon|r",
                            ["canEdit"] = false,
                            ["name"] = "Fardon",
                            ["bName"] = "pdkp_dkp_entry13",
                        }, -- [4]
                        {
                            ["inRaid"] = false,
                            ["class"] = "Hunter",
                            ["dkpTotal"] = 0,
                            ["online"] = false,
                            ["rankIndex"] = 6,
                            ["formattedName"] = "|cffABD473Jellytime|r",
                            ["bName"] = "pdkp_dkp_entry5",
                            ["name"] = "Jellytime",
                            ["canEdit"] = false,
                        }, -- [5]
                        {
                            ["inRaid"] = false,
                            ["class"] = "Mage",
                            ["dkpTotal"] = 0,
                            ["online"] = false,
                            ["rankIndex"] = 6,
                            ["formattedName"] = "|cff40C7EBLaird|r",
                            ["bName"] = "pdkp_dkp_entry6",
                            ["name"] = "Laird",
                            ["canEdit"] = false,
                        }, -- [6]
                        {
                            ["dkpTotal"] = 0,
                            ["online"] = false,
                            ["name"] = "Pootootwo",
                            ["formattedName"] = "|cff40C7EBPootootwo|r",
                            ["canEdit"] = false,
                            ["class"] = "Mage",
                            ["inRaid"] = false,
                            ["rankIndex"] = 6,
                        }, -- [7]
                        {
                            ["dkpTotal"] = 0,
                            ["online"] = false,
                            ["name"] = "Scudd",
                            ["formattedName"] = "|cffC79C6EScudd|r",
                            ["canEdit"] = false,
                            ["class"] = "Warrior",
                            ["inRaid"] = false,
                            ["rankIndex"] = 6,
                        }, -- [8]
                        {
                            ["dkpTotal"] = 0,
                            ["online"] = false,
                            ["name"] = "Sretsam",
                            ["formattedName"] = "|cffC79C6ESretsam|r",
                            ["canEdit"] = false,
                            ["class"] = "Warrior",
                            ["inRaid"] = false,
                            ["rankIndex"] = 6,
                        }, -- [9]
                        {
                            ["dkpTotal"] = 0,
                            ["online"] = false,
                            ["name"] = "Ttoken",
                            ["formattedName"] = "|cff40C7EBTtoken|r",
                            ["canEdit"] = false,
                            ["class"] = "Mage",
                            ["inRaid"] = false,
                            ["rankIndex"] = 6,
                        }, -- [10]
                        {
                            ["inRaid"] = false,
                            ["class"] = "Warlock",
                            ["dkpTotal"] = 0,
                            ["online"] = false,
                            ["rankIndex"] = 6,
                            ["formattedName"] = "|cff8787EDWalterpayton|r",
                            ["bName"] = "pdkp_dkp_entry11",
                            ["canEdit"] = false,
                            ["name"] = "Walterpayton",
                        }, -- [11]
                        {
                            ["inRaid"] = false,
                            ["class"] = "Warlock",
                            ["dkpTotal"] = 400,
                            ["online"] = false,
                            ["rankIndex"] = 5,
                            ["formattedName"] = "|cff8787EDEdgyboi|r",
                            ["bName"] = "pdkp_dkp_entry2",
                            ["canEdit"] = false,
                            ["name"] = "Edgyboi",
                        }, -- [12]
                        {
                            ["inRaid"] = false,
                            ["class"] = "Warrior",
                            ["dkpTotal"] = 139,
                            ["online"] = false,
                            ["rankIndex"] = 5,
                            ["formattedName"] = "|cffC79C6EGirlslayer|r",
                            ["bName"] = "pdkp_dkp_entry2",
                            ["canEdit"] = false,
                            ["name"] = "Girlslayer",
                        }, -- [13]
                        {
                            ["inRaid"] = false,
                            ["class"] = "Warrior",
                            ["dkpTotal"] = 400,
                            ["online"] = false,
                            ["rankIndex"] = 5,
                            ["formattedName"] = "|cffC79C6EGoobimus|r",
                            ["bName"] = "pdkp_dkp_entry12",
                            ["canEdit"] = false,
                            ["name"] = "Goobimus",
                        }, -- [14]
                        {
                            ["inRaid"] = false,
                            ["class"] = "Warrior",
                            ["dkpTotal"] = 400,
                            ["online"] = false,
                            ["rankIndex"] = 5,
                            ["formattedName"] = "|cffC79C6EHotdogbroth|r",
                            ["bName"] = "pdkp_dkp_entry5",
                            ["canEdit"] = false,
                            ["name"] = "Hotdogbroth",
                        }, -- [15]
                        {
                            ["inRaid"] = false,
                            ["class"] = "Mage",
                            ["dkpTotal"] = 272,
                            ["online"] = false,
                            ["rankIndex"] = 5,
                            ["formattedName"] = "|cff40C7EBJakemeoff|r",
                            ["bName"] = "pdkp_dkp_entry5",
                            ["canEdit"] = false,
                            ["name"] = "Jakemeoff",
                        }, -- [16]
                        {
                            ["inRaid"] = true,
                            ["class"] = "Warrior",
                            ["dkpTotal"] = 300,
                            ["online"] = false,
                            ["rankIndex"] = 5,
                            ["formattedName"] = "|cffC79C6ELuckerdawg|r",
                            ["bName"] = "pdkp_dkp_entry4",
                            ["canEdit"] = false,
                            ["name"] = "Luckerdawg",
                        }, -- [17]
                        {
                            ["inRaid"] = false,
                            ["class"] = "Rogue",
                            ["dkpTotal"] = 225,
                            ["online"] = false,
                            ["rankIndex"] = 5,
                            ["formattedName"] = "|cffFFF569Mcstaberson|r",
                            ["bName"] = "pdkp_dkp_entry6",
                            ["canEdit"] = false,
                            ["name"] = "Mcstaberson",
                        }, -- [18]
                        {
                            ["inRaid"] = false,
                            ["class"] = "Warlock",
                            ["dkpTotal"] = 200,
                            ["online"] = false,
                            ["rankIndex"] = 5,
                            ["formattedName"] = "|cff8787EDThenight|r",
                            ["bName"] = "pdkp_dkp_entry1",
                            ["canEdit"] = false,
                            ["name"] = "Thenight",
                        }, -- [19]
                        {
                            ["inRaid"] = false,
                            ["class"] = "Druid",
                            ["dkpTotal"] = 232,
                            ["online"] = false,
                            ["rankIndex"] = 4,
                            ["formattedName"] = "|cffFF7D0AEmmyy|r",
                            ["bName"] = "pdkp_dkp_entry1",
                            ["canEdit"] = false,
                            ["name"] = "Emmyy",
                        }, -- [20]
                        {
                            ["inRaid"] = false,
                            ["class"] = "Priest",
                            ["dkpTotal"] = 260,
                            ["online"] = false,
                            ["rankIndex"] = 4,
                            ["formattedName"] = "|cffFFFFFFEvangelina|r",
                            ["bName"] = "pdkp_dkp_entry1",
                            ["canEdit"] = false,
                            ["name"] = "Evangelina",
                        }, -- [21]
                        {
                            ["inRaid"] = false,
                            ["class"] = "Rogue",
                            ["dkpTotal"] = 0,
                            ["online"] = false,
                            ["rankIndex"] = 3,
                            ["formattedName"] = "|cffFFF569Pantheonbank|r",
                            ["bName"] = "pdkp_dkp_entry3",
                            ["canEdit"] = true,
                            ["name"] = "Pantheonbank",
                        }, -- [22]
                        {
                            ["inRaid"] = false,
                            ["class"] = "Warlock",
                            ["dkpTotal"] = 345,
                            ["online"] = false,
                            ["rankIndex"] = 2,
                            ["formattedName"] = "|cff8787EDSparklenips|r",
                            ["bName"] = "pdkp_dkp_entry19",
                            ["canEdit"] = true,
                            ["name"] = "Sparklenips",
                        }, -- [23]
                        {
                            ["inRaid"] = false,
                            ["class"] = "Mage",
                            ["dkpTotal"] = 370,
                            ["online"] = false,
                            ["rankIndex"] = 2,
                            ["formattedName"] = "|cff40C7EBVeriandra|r",
                            ["bName"] = "pdkp_dkp_entry10",
                            ["canEdit"] = true,
                            ["name"] = "Veriandra",
                        }, -- [24]
                        {
                            ["dkpTotal"] = 0,
                            ["online"] = false,
                            ["name"] = "Doormann",
                            ["formattedName"] = "|cffC79C6EDoormann|r",
                            ["canEdit"] = false,
                            ["class"] = "Warrior",
                            ["inRaid"] = false,
                            ["rankIndex"] = 7,
                        }, -- [25]
                        {
                            ["dkpTotal"] = 0,
                            ["online"] = false,
                            ["name"] = "Flard",
                            ["formattedName"] = "|cffABD473Flard|r",
                            ["canEdit"] = false,
                            ["class"] = "Hunter",
                            ["inRaid"] = false,
                            ["rankIndex"] = 7,
                        }, -- [26]
                        {
                            ["dkpTotal"] = 0,
                            ["online"] = false,
                            ["name"] = "Prfx",
                            ["formattedName"] = "|cffFFFFFFPrfx|r",
                            ["canEdit"] = false,
                            ["class"] = "Priest",
                            ["inRaid"] = false,
                            ["rankIndex"] = 7,
                        }, -- [27]
                        {
                            ["dkpTotal"] = 0,
                            ["online"] = false,
                            ["name"] = "Septra",
                            ["formattedName"] = "|cffABD473Septra|r",
                            ["canEdit"] = false,
                            ["class"] = "Hunter",
                            ["inRaid"] = false,
                            ["rankIndex"] = 7,
                        }, -- [28]
                        {
                            ["dkpTotal"] = 0,
                            ["online"] = false,
                            ["name"] = "Protius",
                            ["formattedName"] = "|cffF58CBAProtius|r",
                            ["canEdit"] = false,
                            ["class"] = "Paladin",
                            ["inRaid"] = false,
                            ["rankIndex"] = 7,
                        }, -- [29]
                        {
                            ["dkpTotal"] = 0,
                            ["online"] = false,
                            ["name"] = "Weiniehutjr",
                            ["formattedName"] = "|cffFF7D0AWeiniehutjr|r",
                            ["canEdit"] = false,
                            ["class"] = "Druid",
                            ["inRaid"] = false,
                            ["rankIndex"] = 7,
                        }, -- [30]
                        {
                            ["dkpTotal"] = 0,
                            ["online"] = false,
                            ["name"] = "Mordem",
                            ["formattedName"] = "|cffFF7D0AMordem|r",
                            ["canEdit"] = false,
                            ["class"] = "Druid",
                            ["inRaid"] = false,
                            ["rankIndex"] = 7,
                        }, -- [31]
                        {
                            ["dkpTotal"] = 0,
                            ["online"] = true,
                            ["name"] = "Willich",
                            ["formattedName"] = "|cffF58CBAWillich|r",
                            ["canEdit"] = false,
                            ["class"] = "Paladin",
                            ["inRaid"] = false,
                            ["rankIndex"] = 7,
                        }, -- [32]
                        {
                            ["dkpTotal"] = 0,
                            ["online"] = false,
                            ["name"] = "Darticus",
                            ["formattedName"] = "|cff40C7EBDarticus|r",
                            ["canEdit"] = false,
                            ["class"] = "Mage",
                            ["inRaid"] = false,
                            ["rankIndex"] = 7,
                        }, -- [33]
                        {
                            ["dkpTotal"] = 0,
                            ["online"] = false,
                            ["name"] = "Joreid",
                            ["formattedName"] = "|cffFFFFFFJoreid|r",
                            ["canEdit"] = false,
                            ["class"] = "Priest",
                            ["inRaid"] = false,
                            ["rankIndex"] = 7,
                        }, -- [34]
                        {
                            ["dkpTotal"] = 0,
                            ["online"] = false,
                            ["name"] = "Knittie",
                            ["formattedName"] = "|cffFFF569Knittie|r",
                            ["canEdit"] = false,
                            ["class"] = "Rogue",
                            ["inRaid"] = false,
                            ["rankIndex"] = 7,
                        }, -- [35]
                        {
                            ["dkpTotal"] = 0,
                            ["online"] = false,
                            ["name"] = "Adjora",
                            ["formattedName"] = "|cffFFFFFFAdjora|r",
                            ["canEdit"] = false,
                            ["class"] = "Priest",
                            ["inRaid"] = false,
                            ["rankIndex"] = 6,
                        }, -- [36]
                        {
                            ["dkpTotal"] = 0,
                            ["online"] = false,
                            ["name"] = "Aeropress",
                            ["formattedName"] = "|cff40C7EBAeropress|r",
                            ["canEdit"] = false,
                            ["class"] = "Mage",
                            ["inRaid"] = false,
                            ["rankIndex"] = 6,
                        }, -- [37]
                        {
                            ["dkpTotal"] = 0,
                            ["online"] = false,
                            ["name"] = "Apolyne",
                            ["formattedName"] = "|cffFFFFFFApolyne|r",
                            ["canEdit"] = false,
                            ["class"] = "Priest",
                            ["inRaid"] = false,
                            ["rankIndex"] = 6,
                        }, -- [38]
                        {
                            ["dkpTotal"] = 0,
                            ["online"] = false,
                            ["name"] = "Artorion",
                            ["formattedName"] = "|cffF58CBAArtorion|r",
                            ["canEdit"] = false,
                            ["class"] = "Paladin",
                            ["inRaid"] = false,
                            ["rankIndex"] = 6,
                        }, -- [39]
                        {
                            ["inRaid"] = true,
                            ["class"] = "Druid",
                            ["dkpTotal"] = 82,
                            ["online"] = false,
                            ["rankIndex"] = 6,
                            ["formattedName"] = "|cffFF7D0ABlanebites|r",
                            ["canEdit"] = false,
                            ["name"] = "Blanebites",
                            ["bName"] = "pdkp_dkp_entry1",
                        }, -- [40]
                        {
                            ["dkpTotal"] = 0,
                            ["online"] = false,
                            ["name"] = "Bojack",
                            ["formattedName"] = "|cffFF7D0ABojack|r",
                            ["canEdit"] = false,
                            ["class"] = "Druid",
                            ["inRaid"] = false,
                            ["rankIndex"] = 6,
                        }, -- [41]
                        {
                            ["dkpTotal"] = 0,
                            ["online"] = false,
                            ["name"] = "Bruhtato",
                            ["formattedName"] = "|cffC79C6EBruhtato|r",
                            ["canEdit"] = false,
                            ["class"] = "Warrior",
                            ["inRaid"] = false,
                            ["rankIndex"] = 6,
                        }, -- [42]
                        {
                            ["dkpTotal"] = 0,
                            ["online"] = false,
                            ["name"] = "Calixta",
                            ["formattedName"] = "|cff8787EDCalixta|r",
                            ["canEdit"] = false,
                            ["class"] = "Warlock",
                            ["inRaid"] = false,
                            ["rankIndex"] = 6,
                        }, -- [43]
                        {
                            ["inRaid"] = false,
                            ["class"] = "Paladin",
                            ["dkpTotal"] = 100,
                            ["online"] = false,
                            ["rankIndex"] = 6,
                            ["formattedName"] = "|cffF58CBACaptnutsack|r",
                            ["bName"] = "pdkp_dkp_entry1",
                            ["canEdit"] = false,
                            ["name"] = "Captnutsack",
                        }, -- [44]
                        {
                            ["dkpTotal"] = 0,
                            ["online"] = false,
                            ["name"] = "Celestaes",
                            ["formattedName"] = "|cffF58CBACelestaes|r",
                            ["canEdit"] = false,
                            ["class"] = "Paladin",
                            ["inRaid"] = false,
                            ["rankIndex"] = 6,
                        }, -- [45]
                        {
                            ["dkpTotal"] = 0,
                            ["online"] = false,
                            ["name"] = "Chutter",
                            ["formattedName"] = "|cffFFFFFFChutter|r",
                            ["canEdit"] = false,
                            ["class"] = "Priest",
                            ["inRaid"] = false,
                            ["rankIndex"] = 6,
                        }, -- [46]
                        {
                            ["dkpTotal"] = 0,
                            ["online"] = false,
                            ["name"] = "Claireamy",
                            ["formattedName"] = "|cffFF7D0AClaireamy|r",
                            ["canEdit"] = false,
                            ["class"] = "Druid",
                            ["inRaid"] = false,
                            ["rankIndex"] = 6,
                        }, -- [47]
                        {
                            ["dkpTotal"] = 0,
                            ["online"] = false,
                            ["name"] = "Coldjuice",
                            ["formattedName"] = "|cff40C7EBColdjuice|r",
                            ["canEdit"] = false,
                            ["class"] = "Mage",
                            ["inRaid"] = false,
                            ["rankIndex"] = 6,
                        }, -- [48]
                        {
                            ["dkpTotal"] = 0,
                            ["online"] = false,
                            ["name"] = "Coneofcool",
                            ["formattedName"] = "|cff40C7EBConeofcool|r",
                            ["canEdit"] = false,
                            ["class"] = "Mage",
                            ["inRaid"] = false,
                            ["rankIndex"] = 6,
                        }, -- [49]
                        {
                            ["inRaid"] = false,
                            ["class"] = "Druid",
                            ["dkpTotal"] = 0,
                            ["online"] = false,
                            ["rankIndex"] = 6,
                            ["formattedName"] = "|cffFF7D0ACorseau|r",
                            ["canEdit"] = false,
                            ["name"] = "Corseau",
                            ["bName"] = "pdkp_dkp_entry7",
                        }, -- [50]
                        {
                            ["dkpTotal"] = 0,
                            ["online"] = false,
                            ["name"] = "Cptncold",
                            ["formattedName"] = "|cff40C7EBCptncold|r",
                            ["canEdit"] = false,
                            ["class"] = "Mage",
                            ["inRaid"] = false,
                            ["rankIndex"] = 6,
                        }, -- [51]
                        {
                            ["dkpTotal"] = 0,
                            ["online"] = false,
                            ["name"] = "Crazymarbles",
                            ["formattedName"] = "|cffF58CBACrazymarbles|r",
                            ["canEdit"] = false,
                            ["class"] = "Paladin",
                            ["inRaid"] = false,
                            ["rankIndex"] = 6,
                        }, -- [52]
                        {
                            ["dkpTotal"] = 0,
                            ["online"] = false,
                            ["name"] = "Danatelo",
                            ["formattedName"] = "|cffFF7D0ADanatelo|r",
                            ["canEdit"] = false,
                            ["class"] = "Druid",
                            ["inRaid"] = false,
                            ["rankIndex"] = 6,
                        }, -- [53]
                        {
                            ["dkpTotal"] = 0,
                            ["online"] = false,
                            ["name"] = "Dasmook",
                            ["formattedName"] = "|cff8787EDDasmook|r",
                            ["canEdit"] = false,
                            ["class"] = "Warlock",
                            ["inRaid"] = false,
                            ["rankIndex"] = 6,
                        }, -- [54]
                        {
                            ["dkpTotal"] = 0,
                            ["online"] = false,
                            ["name"] = "Deeprider",
                            ["formattedName"] = "|cffABD473Deeprider|r",
                            ["canEdit"] = false,
                            ["class"] = "Hunter",
                            ["inRaid"] = false,
                            ["rankIndex"] = 6,
                        }, -- [55]
                        {
                            ["dkpTotal"] = 0,
                            ["online"] = false,
                            ["name"] = "Ellara",
                            ["formattedName"] = "|cff40C7EBEllara|r",
                            ["canEdit"] = false,
                            ["class"] = "Mage",
                            ["inRaid"] = false,
                            ["rankIndex"] = 6,
                        }, -- [56]
                        {
                            ["dkpTotal"] = 0,
                            ["online"] = false,
                            ["name"] = "Finrir",
                            ["formattedName"] = "|cffABD473Finrir|r",
                            ["canEdit"] = false,
                            ["class"] = "Hunter",
                            ["inRaid"] = false,
                            ["rankIndex"] = 6,
                        }, -- [57]
                        {
                            ["dkpTotal"] = 0,
                            ["online"] = false,
                            ["name"] = "Finryr",
                            ["formattedName"] = "|cffC79C6EFinryr|r",
                            ["canEdit"] = false,
                            ["class"] = "Warrior",
                            ["inRaid"] = false,
                            ["rankIndex"] = 6,
                        }, -- [58]
                        {
                            ["inRaid"] = false,
                            ["class"] = "Mage",
                            ["dkpTotal"] = 0,
                            ["online"] = false,
                            ["rankIndex"] = 6,
                            ["formattedName"] = "|cff40C7EBFiz|r",
                            ["canEdit"] = false,
                            ["name"] = "Fiz",
                            ["bName"] = "pdkp_dkp_entry12",
                        }, -- [59]
                        {
                            ["inRaid"] = false,
                            ["class"] = "Mage",
                            ["dkpTotal"] = 395,
                            ["online"] = false,
                            ["rankIndex"] = 5,
                            ["formattedName"] = "|cff40C7EBFlatulent|r",
                            ["bName"] = "pdkp_dkp_entry14",
                            ["canEdit"] = false,
                            ["name"] = "Flatulent",
                        }, -- [60]
                        {
                            ["inRaid"] = false,
                            ["class"] = "Paladin",
                            ["dkpTotal"] = 187,
                            ["online"] = false,
                            ["rankIndex"] = 6,
                            ["formattedName"] = "|cffF58CBAForerunner|r",
                            ["bName"] = "pdkp_dkp_entry1",
                            ["canEdit"] = false,
                            ["name"] = "Forerunner",
                        }, -- [61]
                        {
                            ["inRaid"] = false,
                            ["class"] = "Warrior",
                            ["dkpTotal"] = 0,
                            ["online"] = false,
                            ["rankIndex"] = 6,
                            ["formattedName"] = "|cffC79C6EFradge|r",
                            ["bName"] = "pdkp_dkp_entry4",
                            ["canEdit"] = false,
                            ["name"] = "Fradge",
                        }, -- [62]
                        {
                            ["inRaid"] = false,
                            ["class"] = "Rogue",
                            ["dkpTotal"] = 0,
                            ["online"] = false,
                            ["rankIndex"] = 6,
                            ["formattedName"] = "|cffFFF569Getcrit|r",
                            ["bName"] = "pdkp_dkp_entry14",
                            ["canEdit"] = false,
                            ["name"] = "Getcrit",
                        }, -- [63]
                        {
                            ["inRaid"] = false,
                            ["class"] = "Mage",
                            ["dkpTotal"] = 0,
                            ["online"] = false,
                            ["rankIndex"] = 6,
                            ["formattedName"] = "|cff40C7EBGhettonaga|r",
                            ["canEdit"] = false,
                            ["name"] = "Ghettonaga",
                            ["bName"] = "pdkp_dkp_entry10",
                        }, -- [64]
                        {
                            ["inRaid"] = true,
                            ["class"] = "Paladin",
                            ["dkpTotal"] = 300,
                            ["online"] = false,
                            ["rankIndex"] = 6,
                            ["formattedName"] = "|cffF58CBAGneissguy|r",
                            ["canEdit"] = false,
                            ["name"] = "Gneissguy",
                            ["bName"] = "pdkp_dkp_entry3",
                        }, -- [65]
                        {
                            ["dkpTotal"] = 0,
                            ["online"] = false,
                            ["name"] = "Gnomenuts",
                            ["formattedName"] = "|cff40C7EBGnomenuts|r",
                            ["canEdit"] = false,
                            ["class"] = "Mage",
                            ["inRaid"] = false,
                            ["rankIndex"] = 6,
                        }, -- [66]
                        {
                            ["dkpTotal"] = 0,
                            ["online"] = false,
                            ["name"] = "Gorthaurr",
                            ["formattedName"] = "|cff8787EDGorthaurr|r",
                            ["canEdit"] = false,
                            ["class"] = "Warlock",
                            ["inRaid"] = false,
                            ["rankIndex"] = 6,
                        }, -- [67]
                        {
                            ["dkpTotal"] = 0,
                            ["online"] = false,
                            ["name"] = "Grymmlock",
                            ["formattedName"] = "|cff8787EDGrymmlock|r",
                            ["canEdit"] = false,
                            ["class"] = "Warlock",
                            ["inRaid"] = false,
                            ["rankIndex"] = 6,
                        }, -- [68]
                        {
                            ["dkpTotal"] = 0,
                            ["online"] = false,
                            ["name"] = "Hamickle",
                            ["formattedName"] = "|cff8787EDHamickle|r",
                            ["canEdit"] = false,
                            ["class"] = "Warlock",
                            ["inRaid"] = false,
                            ["rankIndex"] = 6,
                        }, -- [69]
                        {
                            ["dkpTotal"] = 0,
                            ["online"] = false,
                            ["name"] = "Hazie",
                            ["formattedName"] = "|cffABD473Hazie|r",
                            ["canEdit"] = false,
                            ["class"] = "Hunter",
                            ["inRaid"] = false,
                            ["rankIndex"] = 6,
                        }, -- [70]
                        {
                            ["dkpTotal"] = 0,
                            ["online"] = false,
                            ["name"] = "Healsonweelz",
                            ["formattedName"] = "|cffFF7D0AHealsonweelz|r",
                            ["canEdit"] = false,
                            ["class"] = "Druid",
                            ["inRaid"] = false,
                            ["rankIndex"] = 6,
                        }, -- [71]
                        {
                            ["inRaid"] = false,
                            ["class"] = "Warrior",
                            ["dkpTotal"] = 0,
                            ["online"] = false,
                            ["rankIndex"] = 6,
                            ["formattedName"] = "|cffC79C6EHew|r",
                            ["canEdit"] = false,
                            ["name"] = "Hew",
                            ["bName"] = "pdkp_dkp_entry18",
                        }, -- [72]
                        {
                            ["dkpTotal"] = 0,
                            ["online"] = false,
                            ["name"] = "Hititnquitit",
                            ["formattedName"] = "|cffFFF569Hititnquitit|r",
                            ["canEdit"] = false,
                            ["class"] = "Rogue",
                            ["inRaid"] = false,
                            ["rankIndex"] = 6,
                        }, -- [73]
                        {
                            ["dkpTotal"] = 0,
                            ["online"] = false,
                            ["name"] = "Holychaos",
                            ["formattedName"] = "|cff40C7EBHolychaos|r",
                            ["canEdit"] = false,
                            ["class"] = "Mage",
                            ["inRaid"] = false,
                            ["rankIndex"] = 6,
                        }, -- [74]
                        {
                            ["dkpTotal"] = 0,
                            ["online"] = false,
                            ["name"] = "Holyfingerer",
                            ["formattedName"] = "|cffFFFFFFHolyfingerer|r",
                            ["canEdit"] = false,
                            ["class"] = "Priest",
                            ["inRaid"] = false,
                            ["rankIndex"] = 6,
                        }, -- [75]
                        {
                            ["inRaid"] = false,
                            ["class"] = "Rogue",
                            ["dkpTotal"] = 50,
                            ["online"] = false,
                            ["rankIndex"] = 5,
                            ["formattedName"] = "|cffFFF569Iszell|r",
                            ["canEdit"] = false,
                            ["name"] = "Iszell",
                            ["bName"] = "pdkp_dkp_entry1",
                        }, -- [76]
                        {
                            ["dkpTotal"] = 0,
                            ["online"] = false,
                            ["name"] = "Itsbritneyb",
                            ["formattedName"] = "|cffFFFFFFItsbritneyb|r",
                            ["canEdit"] = false,
                            ["class"] = "Priest",
                            ["inRaid"] = false,
                            ["rankIndex"] = 6,
                        }, -- [77]
                        {
                            ["dkpTotal"] = 0,
                            ["online"] = false,
                            ["name"] = "Jeabus",
                            ["formattedName"] = "|cffFFFFFFJeabus|r",
                            ["canEdit"] = false,
                            ["class"] = "Priest",
                            ["inRaid"] = false,
                            ["rankIndex"] = 6,
                        }, -- [78]
                        {
                            ["dkpTotal"] = 0,
                            ["online"] = false,
                            ["name"] = "Jumbognome",
                            ["formattedName"] = "|cff40C7EBJumbognome|r",
                            ["canEdit"] = false,
                            ["class"] = "Mage",
                            ["inRaid"] = false,
                            ["rankIndex"] = 6,
                        }, -- [79]
                        {
                            ["inRaid"] = true,
                            ["class"] = "Druid",
                            ["dkpTotal"] = 85,
                            ["online"] = false,
                            ["rankIndex"] = 6,
                            ["formattedName"] = "|cffFF7D0AKalijah|r",
                            ["canEdit"] = false,
                            ["name"] = "Kalijah",
                            ["bName"] = "pdkp_dkp_entry1",
                        }, -- [80]
                        {
                            ["dkpTotal"] = 0,
                            ["online"] = false,
                            ["name"] = "Killionaire",
                            ["formattedName"] = "|cffFFF569Killionaire|r",
                            ["canEdit"] = false,
                            ["class"] = "Rogue",
                            ["inRaid"] = false,
                            ["rankIndex"] = 6,
                        }, -- [81]
                        {
                            ["dkpTotal"] = 0,
                            ["online"] = false,
                            ["name"] = "Kinter",
                            ["formattedName"] = "|cff40C7EBKinter|r",
                            ["canEdit"] = false,
                            ["class"] = "Mage",
                            ["inRaid"] = false,
                            ["rankIndex"] = 6,
                        }, -- [82]
                        {
                            ["dkpTotal"] = 0,
                            ["online"] = false,
                            ["name"] = "Kittysnake",
                            ["formattedName"] = "|cff8787EDKittysnake|r",
                            ["canEdit"] = false,
                            ["class"] = "Warlock",
                            ["inRaid"] = false,
                            ["rankIndex"] = 6,
                        }, -- [83]
                        {
                            ["dkpTotal"] = 0,
                            ["online"] = false,
                            ["name"] = "Knifeyspoony",
                            ["formattedName"] = "|cffFFF569Knifeyspoony|r",
                            ["canEdit"] = false,
                            ["class"] = "Rogue",
                            ["inRaid"] = false,
                            ["rankIndex"] = 6,
                        }, -- [84]
                        {
                            ["dkpTotal"] = 0,
                            ["online"] = false,
                            ["name"] = "Kuckuck",
                            ["formattedName"] = "|cffF58CBAKuckuck|r",
                            ["canEdit"] = false,
                            ["class"] = "Paladin",
                            ["inRaid"] = false,
                            ["rankIndex"] = 6,
                        }, -- [85]
                        {
                            ["dkpTotal"] = 0,
                            ["online"] = false,
                            ["name"] = "Landlubbers",
                            ["formattedName"] = "|cff40C7EBLandlubbers|r",
                            ["canEdit"] = false,
                            ["class"] = "Mage",
                            ["inRaid"] = false,
                            ["rankIndex"] = 6,
                        }, -- [86]
                        {
                            ["inRaid"] = false,
                            ["class"] = "Paladin",
                            ["dkpTotal"] = 240,
                            ["online"] = false,
                            ["rankIndex"] = 6,
                            ["formattedName"] = "|cffF58CBALawduk|r",
                            ["bName"] = "pdkp_dkp_entry1",
                            ["canEdit"] = false,
                            ["name"] = "Lawduk",
                        }, -- [87]
                        {
                            ["inRaid"] = false,
                            ["class"] = "Druid",
                            ["dkpTotal"] = 0,
                            ["online"] = false,
                            ["rankIndex"] = 6,
                            ["formattedName"] = "|cffFF7D0ALemonz|r",
                            ["canEdit"] = false,
                            ["name"] = "Lemonz",
                            ["bName"] = "pdkp_dkp_entry23",
                        }, -- [88]
                        {
                            ["dkpTotal"] = 0,
                            ["online"] = false,
                            ["name"] = "Limeybeard",
                            ["formattedName"] = "|cff40C7EBLimeybeard|r",
                            ["canEdit"] = false,
                            ["class"] = "Mage",
                            ["inRaid"] = false,
                            ["rankIndex"] = 6,
                        }, -- [89]
                        {
                            ["dkpTotal"] = 0,
                            ["online"] = false,
                            ["name"] = "Littleshiv",
                            ["formattedName"] = "|cffFFF569Littleshiv|r",
                            ["canEdit"] = false,
                            ["class"] = "Rogue",
                            ["inRaid"] = false,
                            ["rankIndex"] = 6,
                        }, -- [90]
                        {
                            ["dkpTotal"] = 0,
                            ["online"] = false,
                            ["name"] = "Lixx",
                            ["formattedName"] = "|cffFFF569Lixx|r",
                            ["canEdit"] = false,
                            ["class"] = "Rogue",
                            ["inRaid"] = false,
                            ["rankIndex"] = 6,
                        }, -- [91]
                        {
                            ["inRaid"] = false,
                            ["class"] = "Warrior",
                            ["dkpTotal"] = 50,
                            ["online"] = false,
                            ["rankIndex"] = 6,
                            ["formattedName"] = "|cffC79C6EMadmartagen|r",
                            ["canEdit"] = false,
                            ["name"] = "Madmartagen",
                            ["bName"] = "pdkp_dkp_entry1",
                        }, -- [92]
                        {
                            ["dkpTotal"] = 0,
                            ["online"] = false,
                            ["name"] = "Mallix",
                            ["formattedName"] = "|cffABD473Mallix|r",
                            ["canEdit"] = false,
                            ["class"] = "Hunter",
                            ["inRaid"] = false,
                            ["rankIndex"] = 6,
                        }, -- [93]
                        {
                            ["dkpTotal"] = 0,
                            ["online"] = false,
                            ["name"] = "Maryse",
                            ["formattedName"] = "|cffABD473Maryse|r",
                            ["canEdit"] = false,
                            ["class"] = "Hunter",
                            ["inRaid"] = false,
                            ["rankIndex"] = 6,
                        }, -- [94]
                        {
                            ["dkpTotal"] = 0,
                            ["online"] = false,
                            ["name"] = "Minifridge",
                            ["formattedName"] = "|cff40C7EBMinifridge|r",
                            ["canEdit"] = false,
                            ["class"] = "Mage",
                            ["inRaid"] = false,
                            ["rankIndex"] = 6,
                        }, -- [95]
                        {
                            ["inRaid"] = false,
                            ["class"] = "Rogue",
                            ["dkpTotal"] = 223,
                            ["online"] = false,
                            ["rankIndex"] = 6,
                            ["formattedName"] = "|cffFFF569Mujinn|r",
                            ["bName"] = "pdkp_dkp_entry1",
                            ["canEdit"] = false,
                            ["name"] = "Mujinn",
                        }, -- [96]
                        {
                            ["dkpTotal"] = 0,
                            ["online"] = false,
                            ["name"] = "Natgeo",
                            ["formattedName"] = "|cff40C7EBNatgeo|r",
                            ["canEdit"] = false,
                            ["class"] = "Mage",
                            ["inRaid"] = false,
                            ["rankIndex"] = 6,
                        }, -- [97]
                        {
                            ["dkpTotal"] = 0,
                            ["online"] = false,
                            ["name"] = "Neonlight",
                            ["formattedName"] = "|cffFFFFFFNeonlight|r",
                            ["canEdit"] = false,
                            ["class"] = "Priest",
                            ["inRaid"] = false,
                            ["rankIndex"] = 6,
                        }, -- [98]
                        {
                            ["dkpTotal"] = 0,
                            ["online"] = false,
                            ["name"] = "Neotemplar",
                            ["formattedName"] = "|cffF58CBANeotemplar|r",
                            ["canEdit"] = false,
                            ["class"] = "Paladin",
                            ["inRaid"] = false,
                            ["rankIndex"] = 6,
                        }, -- [99]
                        {
                            ["dkpTotal"] = 0,
                            ["online"] = false,
                            ["name"] = "Nubslayer",
                            ["formattedName"] = "|cff8787EDNubslayer|r",
                            ["canEdit"] = false,
                            ["class"] = "Warlock",
                            ["inRaid"] = false,
                            ["rankIndex"] = 6,
                        }, -- [100]
                        {
                            ["dkpTotal"] = 0,
                            ["online"] = false,
                            ["name"] = "Oldmanmike",
                            ["formattedName"] = "|cffFF7D0AOldmanmike|r",
                            ["canEdit"] = false,
                            ["class"] = "Druid",
                            ["inRaid"] = false,
                            ["rankIndex"] = 6,
                        }, -- [101]
                        {
                            ["inRaid"] = false,
                            ["class"] = "Paladin",
                            ["dkpTotal"] = 400,
                            ["online"] = false,
                            ["rankIndex"] = 6,
                            ["formattedName"] = "|cffF58CBAPapasquach|r",
                            ["bName"] = "pdkp_dkp_entry1",
                            ["canEdit"] = false,
                            ["name"] = "Papasquach",
                        }, -- [102]
                        {
                            ["dkpTotal"] = 0,
                            ["online"] = false,
                            ["name"] = "Partywolf",
                            ["formattedName"] = "|cffC79C6EPartywolf|r",
                            ["canEdit"] = false,
                            ["class"] = "Warrior",
                            ["inRaid"] = false,
                            ["rankIndex"] = 6,
                        }, -- [103]
                        {
                            ["dkpTotal"] = 0,
                            ["online"] = false,
                            ["name"] = "Phasetwoscum",
                            ["formattedName"] = "|cffFFF569Phasetwoscum|r",
                            ["canEdit"] = false,
                            ["class"] = "Rogue",
                            ["inRaid"] = false,
                            ["rankIndex"] = 6,
                        }, -- [104]
                        {
                            ["dkpTotal"] = 0,
                            ["online"] = false,
                            ["name"] = "Puffypoose",
                            ["formattedName"] = "|cffFFF569Puffypoose|r",
                            ["canEdit"] = false,
                            ["class"] = "Rogue",
                            ["inRaid"] = false,
                            ["rankIndex"] = 6,
                        }, -- [105]
                        {
                            ["inRaid"] = false,
                            ["class"] = "Warrior",
                            ["dkpTotal"] = 206,
                            ["online"] = false,
                            ["rankIndex"] = 6,
                            ["formattedName"] = "|cffC79C6EQew|r",
                            ["bName"] = "pdkp_dkp_entry1",
                            ["canEdit"] = false,
                            ["name"] = "Qew",
                        }, -- [106]
                        {
                            ["dkpTotal"] = 0,
                            ["online"] = false,
                            ["name"] = "Regaskogena",
                            ["formattedName"] = "|cffFFFFFFRegaskogena|r",
                            ["canEdit"] = false,
                            ["class"] = "Priest",
                            ["inRaid"] = false,
                            ["rankIndex"] = 6,
                        }, -- [107]
                        {
                            ["dkpTotal"] = 0,
                            ["online"] = false,
                            ["name"] = "Rollnfatties",
                            ["formattedName"] = "|cffF58CBARollnfatties|r",
                            ["canEdit"] = false,
                            ["class"] = "Paladin",
                            ["inRaid"] = false,
                            ["rankIndex"] = 6,
                        }, -- [108]
                        {
                            ["dkpTotal"] = 0,
                            ["online"] = false,
                            ["name"] = "Savos",
                            ["formattedName"] = "|cff40C7EBSavos|r",
                            ["canEdit"] = false,
                            ["class"] = "Mage",
                            ["inRaid"] = false,
                            ["rankIndex"] = 6,
                        }, -- [109]
                        {
                            ["dkpTotal"] = 0,
                            ["online"] = false,
                            ["name"] = "Shixx",
                            ["formattedName"] = "|cffFFFFFFShixx|r",
                            ["canEdit"] = false,
                            ["class"] = "Priest",
                            ["inRaid"] = false,
                            ["rankIndex"] = 6,
                        }, -- [110]
                        {
                            ["inRaid"] = false,
                            ["class"] = "Rogue",
                            ["dkpTotal"] = 390,
                            ["online"] = false,
                            ["rankIndex"] = 5,
                            ["formattedName"] = "|cffFFF569Sicarrio|r",
                            ["bName"] = "pdkp_dkp_entry3",
                            ["canEdit"] = false,
                            ["name"] = "Sicarrio",
                        }, -- [111]
                        {
                            ["dkpTotal"] = 0,
                            ["online"] = false,
                            ["name"] = "Solana",
                            ["formattedName"] = "|cff40C7EBSolana|r",
                            ["canEdit"] = false,
                            ["class"] = "Mage",
                            ["inRaid"] = false,
                            ["rankIndex"] = 6,
                        }, -- [112]
                        {
                            ["dkpTotal"] = 0,
                            ["online"] = false,
                            ["name"] = "Solidsix",
                            ["formattedName"] = "|cffF58CBASolidsix|r",
                            ["canEdit"] = false,
                            ["class"] = "Paladin",
                            ["inRaid"] = false,
                            ["rankIndex"] = 6,
                        }, -- [113]
                        {
                            ["dkpTotal"] = 0,
                            ["online"] = false,
                            ["name"] = "Solten",
                            ["formattedName"] = "|cffFFFFFFSolten|r",
                            ["canEdit"] = false,
                            ["class"] = "Priest",
                            ["inRaid"] = false,
                            ["rankIndex"] = 6,
                        }, -- [114]
                        {
                            ["dkpTotal"] = 0,
                            ["online"] = false,
                            ["name"] = "Sprocket",
                            ["formattedName"] = "|cff40C7EBSprocket|r",
                            ["canEdit"] = false,
                            ["class"] = "Mage",
                            ["inRaid"] = false,
                            ["rankIndex"] = 6,
                        }, -- [115]
                        {
                            ["dkpTotal"] = 0,
                            ["online"] = false,
                            ["name"] = "Squach",
                            ["formattedName"] = "|cffFFF569Squach|r",
                            ["canEdit"] = false,
                            ["class"] = "Rogue",
                            ["inRaid"] = false,
                            ["rankIndex"] = 6,
                        }, -- [116]
                        {
                            ["dkpTotal"] = 0,
                            ["online"] = false,
                            ["name"] = "Squidprophet",
                            ["formattedName"] = "|cffFFFFFFSquidprophet|r",
                            ["canEdit"] = false,
                            ["class"] = "Priest",
                            ["inRaid"] = false,
                            ["rankIndex"] = 6,
                        }, -- [117]
                        {
                            ["inRaid"] = false,
                            ["class"] = "Rogue",
                            ["dkpTotal"] = 0,
                            ["online"] = false,
                            ["rankIndex"] = 6,
                            ["formattedName"] = "|cffFFF569Stikyiki|r",
                            ["bName"] = "pdkp_dkp_entry8",
                            ["canEdit"] = false,
                            ["name"] = "Stikyiki",
                        }, -- [118]
                        {
                            ["dkpTotal"] = 0,
                            ["online"] = false,
                            ["name"] = "Straton",
                            ["formattedName"] = "|cffC79C6EStraton|r",
                            ["canEdit"] = false,
                            ["class"] = "Warrior",
                            ["inRaid"] = false,
                            ["rankIndex"] = 6,
                        }, -- [119]
                        {
                            ["dkpTotal"] = 0,
                            ["online"] = false,
                            ["name"] = "Stridder",
                            ["formattedName"] = "|cffFFF569Stridder|r",
                            ["canEdit"] = false,
                            ["class"] = "Rogue",
                            ["inRaid"] = false,
                            ["rankIndex"] = 6,
                        }, -- [120]
                        {
                            ["inRaid"] = true,
                            ["class"] = "Warlock",
                            ["dkpTotal"] = 80,
                            ["online"] = false,
                            ["rankIndex"] = 6,
                            ["formattedName"] = "|cff8787EDTalisker|r",
                            ["canEdit"] = false,
                            ["bName"] = "pdkp_dkp_entry1",
                            ["name"] = "Talisker",
                        }, -- [121]
                        {
                            ["dkpTotal"] = 0,
                            ["online"] = false,
                            ["name"] = "Toysfordots",
                            ["formattedName"] = "|cff8787EDToysfordots|r",
                            ["canEdit"] = false,
                            ["class"] = "Warlock",
                            ["inRaid"] = false,
                            ["rankIndex"] = 6,
                        }, -- [122]
                        {
                            ["dkpTotal"] = 0,
                            ["online"] = false,
                            ["name"] = "Tuggspeedman",
                            ["formattedName"] = "|cffF58CBATuggspeedman|r",
                            ["canEdit"] = false,
                            ["class"] = "Paladin",
                            ["inRaid"] = false,
                            ["rankIndex"] = 6,
                        }, -- [123]
                        {
                            ["dkpTotal"] = 0,
                            ["online"] = false,
                            ["name"] = "Turbodeeps",
                            ["formattedName"] = "|cff8787EDTurbodeeps|r",
                            ["canEdit"] = false,
                            ["class"] = "Warlock",
                            ["inRaid"] = false,
                            ["rankIndex"] = 6,
                        }, -- [124]
                        {
                            ["dkpTotal"] = 0,
                            ["online"] = true,
                            ["name"] = "Urkh",
                            ["formattedName"] = "|cff40C7EBUrkh|r",
                            ["canEdit"] = false,
                            ["class"] = "Mage",
                            ["inRaid"] = false,
                            ["rankIndex"] = 6,
                        }, -- [125]
                        {
                            ["inRaid"] = true,
                            ["class"] = "Hunter",
                            ["dkpTotal"] = 107,
                            ["online"] = false,
                            ["rankIndex"] = 6,
                            ["formattedName"] = "|cffABD473Wahcha|r",
                            ["bName"] = "pdkp_dkp_entry1",
                            ["canEdit"] = false,
                            ["name"] = "Wahcha",
                        }, -- [126]
                        {
                            ["dkpTotal"] = 0,
                            ["online"] = false,
                            ["name"] = "Webroinacint",
                            ["formattedName"] = "|cffF58CBAWebroinacint|r",
                            ["canEdit"] = false,
                            ["class"] = "Paladin",
                            ["inRaid"] = false,
                            ["rankIndex"] = 6,
                        }, -- [127]
                        {
                            ["dkpTotal"] = 0,
                            ["online"] = false,
                            ["name"] = "Wernstrum",
                            ["formattedName"] = "|cffFFF569Wernstrum|r",
                            ["canEdit"] = false,
                            ["class"] = "Rogue",
                            ["inRaid"] = false,
                            ["rankIndex"] = 6,
                        }, -- [128]
                        {
                            ["dkpTotal"] = 0,
                            ["online"] = false,
                            ["name"] = "Winancy",
                            ["formattedName"] = "|cff8787EDWinancy|r",
                            ["canEdit"] = false,
                            ["class"] = "Warlock",
                            ["inRaid"] = false,
                            ["rankIndex"] = 6,
                        }, -- [129]
                        {
                            ["inRaid"] = false,
                            ["class"] = "Mage",
                            ["dkpTotal"] = 273,
                            ["online"] = false,
                            ["rankIndex"] = 5,
                            ["formattedName"] = "|cff40C7EBXyen|r",
                            ["canEdit"] = false,
                            ["name"] = "Xyen",
                            ["bName"] = "pdkp_dkp_entry3",
                        }, -- [130]
                        {
                            ["dkpTotal"] = 0,
                            ["online"] = false,
                            ["name"] = "Zeeksa",
                            ["formattedName"] = "|cffFFFFFFZeeksa|r",
                            ["canEdit"] = false,
                            ["class"] = "Priest",
                            ["inRaid"] = false,
                            ["rankIndex"] = 6,
                        }, -- [131]
                        {
                            ["dkpTotal"] = 0,
                            ["online"] = false,
                            ["name"] = "Ziggi",
                            ["formattedName"] = "|cffC79C6EZiggi|r",
                            ["canEdit"] = false,
                            ["class"] = "Warrior",
                            ["inRaid"] = false,
                            ["rankIndex"] = 6,
                        }, -- [132]
                        {
                            ["dkpTotal"] = 0,
                            ["online"] = false,
                            ["name"] = "Acehart",
                            ["formattedName"] = "|cffC79C6EAcehart|r",
                            ["canEdit"] = false,
                            ["class"] = "Warrior",
                            ["inRaid"] = false,
                            ["rankIndex"] = 3,
                        }, -- [133]
                        {
                            ["inRaid"] = true,
                            ["class"] = "Rogue",
                            ["dkpTotal"] = 155,
                            ["online"] = false,
                            ["rankIndex"] = 5,
                            ["formattedName"] = "|cffFFF569Aku|r",
                            ["bName"] = "pdkp_dkp_entry1",
                            ["canEdit"] = false,
                            ["name"] = "Aku",
                        }, -- [134]
                        {
                            ["dkpTotal"] = 0,
                            ["online"] = false,
                            ["name"] = "Astlan",
                            ["formattedName"] = "|cffF58CBAAstlan|r",
                            ["canEdit"] = false,
                            ["class"] = "Paladin",
                            ["inRaid"] = false,
                            ["rankIndex"] = 5,
                        }, -- [135]
                        {
                            ["inRaid"] = false,
                            ["class"] = "Priest",
                            ["dkpTotal"] = 170,
                            ["online"] = false,
                            ["rankIndex"] = 5,
                            ["formattedName"] = "|cffFFFFFFBallour|r",
                            ["bName"] = "pdkp_dkp_entry4",
                            ["canEdit"] = false,
                            ["name"] = "Ballour",
                        }, -- [136]
                        {
                            ["inRaid"] = false,
                            ["class"] = "Hunter",
                            ["dkpTotal"] = 260,
                            ["online"] = false,
                            ["rankIndex"] = 5,
                            ["formattedName"] = "|cffABD473Bigbootyhoho|r",
                            ["bName"] = "pdkp_dkp_entry15",
                            ["canEdit"] = false,
                            ["name"] = "Bigbootyhoho",
                        }, -- [137]
                        {
                            ["inRaid"] = false,
                            ["class"] = "Priest",
                            ["dkpTotal"] = 160,
                            ["online"] = false,
                            ["rankIndex"] = 5,
                            ["formattedName"] = "|cffFFFFFFBubblè|r",
                            ["bName"] = "pdkp_dkp_entry16",
                            ["canEdit"] = false,
                            ["name"] = "Bubblè",
                        }, -- [138]
                        {
                            ["inRaid"] = false,
                            ["class"] = "Hunter",
                            ["dkpTotal"] = 290,
                            ["online"] = false,
                            ["rankIndex"] = 5,
                            ["formattedName"] = "|cffABD473Bádtothebow|r",
                            ["bName"] = "pdkp_dkp_entry14",
                            ["canEdit"] = false,
                            ["name"] = "Bádtothebow",
                        }, -- [139]
                        {
                            ["inRaid"] = true,
                            ["class"] = "Mage",
                            ["dkpTotal"] = 242,
                            ["online"] = false,
                            ["rankIndex"] = 5,
                            ["formattedName"] = "|cff40C7EBChipgizmo|r",
                            ["bName"] = "pdkp_dkp_entry15",
                            ["canEdit"] = false,
                            ["name"] = "Chipgizmo",
                        }, -- [140]
                        {
                            ["dkpTotal"] = 0,
                            ["online"] = false,
                            ["name"] = "Cyndr",
                            ["formattedName"] = "|cff40C7EBCyndr|r",
                            ["canEdit"] = false,
                            ["class"] = "Mage",
                            ["inRaid"] = false,
                            ["rankIndex"] = 5,
                        }, -- [141]
                        {
                            ["inRaid"] = false,
                            ["class"] = "Hunter",
                            ["dkpTotal"] = 392,
                            ["online"] = false,
                            ["rankIndex"] = 5,
                            ["formattedName"] = "|cffABD473Cyskul|r",
                            ["bName"] = "pdkp_dkp_entry13",
                            ["canEdit"] = false,
                            ["name"] = "Cyskul",
                        }, -- [142]
                        {
                            ["inRaid"] = true,
                            ["class"] = "Paladin",
                            ["dkpTotal"] = 183,
                            ["online"] = false,
                            ["rankIndex"] = 5,
                            ["formattedName"] = "|cffF58CBADolamroth|r",
                            ["bName"] = "pdkp_dkp_entry1",
                            ["canEdit"] = false,
                            ["name"] = "Dolamroth",
                        }, -- [143]
                        {
                            ["dkpTotal"] = 0,
                            ["online"] = false,
                            ["name"] = "Dwindle",
                            ["formattedName"] = "|cffFFF569Dwindle|r",
                            ["canEdit"] = false,
                            ["class"] = "Rogue",
                            ["inRaid"] = false,
                            ["rankIndex"] = 5,
                        }, -- [144]
                        {
                            ["inRaid"] = true,
                            ["class"] = "Priest",
                            ["dkpTotal"] = 280,
                            ["online"] = false,
                            ["rankIndex"] = 5,
                            ["formattedName"] = "|cffFFFFFFFawntine|r",
                            ["bName"] = "pdkp_dkp_entry3",
                            ["canEdit"] = false,
                            ["name"] = "Fawntine",
                        }, -- [145]
                        {
                            ["inRaid"] = false,
                            ["class"] = "Warlock",
                            ["dkpTotal"] = 200,
                            ["online"] = false,
                            ["rankIndex"] = 5,
                            ["formattedName"] = "|cff8787EDGartog|r",
                            ["bName"] = "pdkp_dkp_entry1",
                            ["canEdit"] = false,
                            ["name"] = "Gartog",
                        }, -- [146]
                        {
                            ["inRaid"] = true,
                            ["class"] = "Priest",
                            ["dkpTotal"] = 285,
                            ["online"] = false,
                            ["rankIndex"] = 5,
                            ["formattedName"] = "|cffFFFFFFHoneypot|r",
                            ["bName"] = "pdkp_dkp_entry1",
                            ["canEdit"] = false,
                            ["name"] = "Honeypot",
                        }, -- [147]
                        {
                            ["inRaid"] = true,
                            ["class"] = "Warlock",
                            ["dkpTotal"] = 233,
                            ["online"] = false,
                            ["rankIndex"] = 5,
                            ["formattedName"] = "|cff8787EDIhurricanel|r",
                            ["bName"] = "pdkp_dkp_entry12",
                            ["canEdit"] = false,
                            ["name"] = "Ihurricanel",
                        }, -- [148]
                        {
                            ["inRaid"] = true,
                            ["class"] = "Druid",
                            ["dkpTotal"] = 300,
                            ["online"] = false,
                            ["rankIndex"] = 5,
                            ["formattedName"] = "|cffFF7D0AInebriated|r",
                            ["canEdit"] = false,
                            ["bName"] = "pdkp_dkp_entry1",
                            ["name"] = "Inebriated",
                        }, -- [149]
                        {
                            ["inRaid"] = true,
                            ["class"] = "Rogue",
                            ["dkpTotal"] = 240,
                            ["online"] = false,
                            ["rankIndex"] = 5,
                            ["formattedName"] = "|cffFFF569Inigma|r",
                            ["bName"] = "pdkp_dkp_entry14",
                            ["canEdit"] = false,
                            ["name"] = "Inigma",
                        }, -- [150]
                        {
                            ["inRaid"] = true,
                            ["class"] = "Druid",
                            ["dkpTotal"] = 267,
                            ["online"] = false,
                            ["rankIndex"] = 5,
                            ["formattedName"] = "|cffFF7D0AInsub|r",
                            ["bName"] = "pdkp_dkp_entry2",
                            ["canEdit"] = false,
                            ["name"] = "Insub",
                        }, -- [151]
                        {
                            ["dkpTotal"] = 200,
                            ["online"] = false,
                            ["name"] = "Jeffry",
                            ["formattedName"] = "|cffC79C6EJeffry|r",
                            ["canEdit"] = false,
                            ["class"] = "Warrior",
                            ["inRaid"] = true,
                            ["rankIndex"] = 5,
                        }, -- [152]
                        {
                            ["inRaid"] = false,
                            ["class"] = "Druid",
                            ["dkpTotal"] = 72,
                            ["online"] = false,
                            ["rankIndex"] = 5,
                            ["formattedName"] = "|cffFF7D0AJunglemain|r",
                            ["bName"] = "pdkp_dkp_entry1",
                            ["canEdit"] = false,
                            ["name"] = "Junglemain",
                        }, -- [153]
                        {
                            ["inRaid"] = false,
                            ["class"] = "Warrior",
                            ["dkpTotal"] = 400,
                            ["online"] = true,
                            ["rankIndex"] = 5,
                            ["formattedName"] = "|cffC79C6EKalita|r",
                            ["bName"] = "pdkp_dkp_entry8",
                            ["canEdit"] = false,
                            ["name"] = "Kalita",
                        }, -- [154]
                        {
                            ["inRaid"] = true,
                            ["class"] = "Warlock",
                            ["dkpTotal"] = 200,
                            ["online"] = false,
                            ["rankIndex"] = 5,
                            ["formattedName"] = "|cff8787EDLittledubs|r",
                            ["canEdit"] = false,
                            ["bName"] = "pdkp_dkp_entry3",
                            ["name"] = "Littledubs",
                        }, -- [155]
                        {
                            ["inRaid"] = false,
                            ["class"] = "Rogue",
                            ["dkpTotal"] = 100,
                            ["online"] = false,
                            ["rankIndex"] = 5,
                            ["formattedName"] = "|cffFFF569Mariku|r",
                            ["bName"] = "pdkp_dkp_entry1",
                            ["canEdit"] = false,
                            ["name"] = "Mariku",
                        }, -- [156]
                        {
                            ["inRaid"] = false,
                            ["class"] = "Paladin",
                            ["dkpTotal"] = 324,
                            ["online"] = false,
                            ["rankIndex"] = 7,
                            ["formattedName"] = "|cffF58CBAMaryjohanna|r",
                            ["bName"] = "pdkp_dkp_entry1",
                            ["canEdit"] = false,
                            ["name"] = "Maryjohanna",
                        }, -- [157]
                        {
                            ["inRaid"] = false,
                            ["class"] = "Warrior",
                            ["dkpTotal"] = 119,
                            ["online"] = false,
                            ["rankIndex"] = 5,
                            ["formattedName"] = "|cffC79C6EMihai|r",
                            ["bName"] = "pdkp_dkp_entry17",
                            ["canEdit"] = false,
                            ["name"] = "Mihai",
                        }, -- [158]
                        {
                            ["inRaid"] = true,
                            ["class"] = "Priest",
                            ["dkpTotal"] = 300,
                            ["online"] = false,
                            ["rankIndex"] = 5,
                            ["formattedName"] = "|cffFFFFFFMongous|r",
                            ["bName"] = "pdkp_dkp_entry1",
                            ["canEdit"] = false,
                            ["name"] = "Mongous",
                        }, -- [159]
                        {
                            ["inRaid"] = false,
                            ["class"] = "Druid",
                            ["dkpTotal"] = -9,
                            ["online"] = false,
                            ["rankIndex"] = 5,
                            ["formattedName"] = "|cffFF7D0AMorphintyme|r",
                            ["bName"] = "pdkp_dkp_entry8",
                            ["canEdit"] = false,
                            ["name"] = "Morphintyme",
                        }, -- [160]
                        {
                            ["inRaid"] = false,
                            ["class"] = "Rogue",
                            ["dkpTotal"] = 186,
                            ["online"] = false,
                            ["rankIndex"] = 5,
                            ["formattedName"] = "|cffFFF569Mozzarella|r",
                            ["bName"] = "pdkp_dkp_entry1",
                            ["canEdit"] = false,
                            ["name"] = "Mozzarella",
                        }, -- [161]
                        {
                            ["inRaid"] = false,
                            ["class"] = "Mage",
                            ["dkpTotal"] = 385,
                            ["online"] = false,
                            ["rankIndex"] = 5,
                            ["formattedName"] = "|cff40C7EBNeckbeardo|r",
                            ["bName"] = "pdkp_dkp_entry1",
                            ["canEdit"] = false,
                            ["name"] = "Neckbeardo",
                        }, -- [162]
                        {
                            ["inRaid"] = true,
                            ["class"] = "Warrior",
                            ["dkpTotal"] = 168,
                            ["online"] = false,
                            ["rankIndex"] = 5,
                            ["formattedName"] = "|cffC79C6ENightshelf|r",
                            ["bName"] = "pdkp_dkp_entry19",
                            ["canEdit"] = false,
                            ["name"] = "Nightshelf",
                        }, -- [163]
                        {
                            ["inRaid"] = false,
                            ["class"] = "Paladin",
                            ["dkpTotal"] = 370,
                            ["online"] = false,
                            ["rankIndex"] = 5,
                            ["formattedName"] = "|cffF58CBAOdin|r",
                            ["bName"] = "pdkp_dkp_entry14",
                            ["canEdit"] = false,
                            ["name"] = "Odin",
                        }, -- [164]
                        {
                            ["inRaid"] = true,
                            ["class"] = "Paladin",
                            ["dkpTotal"] = 236,
                            ["online"] = false,
                            ["rankIndex"] = 5,
                            ["formattedName"] = "|cffF58CBAOnes|r",
                            ["bName"] = "pdkp_dkp_entry2",
                            ["canEdit"] = false,
                            ["name"] = "Ones",
                        }, -- [165]
                        {
                            ["inRaid"] = false,
                            ["class"] = "Warrior",
                            ["dkpTotal"] = 257,
                            ["online"] = false,
                            ["rankIndex"] = 5,
                            ["formattedName"] = "|cffC79C6EOragan|r",
                            ["bName"] = "pdkp_dkp_entry6",
                            ["canEdit"] = false,
                            ["name"] = "Oragan",
                        }, -- [166]
                        {
                            ["inRaid"] = true,
                            ["class"] = "Mage",
                            ["dkpTotal"] = 155,
                            ["online"] = true,
                            ["rankIndex"] = 3,
                            ["formattedName"] = "|cff40C7EBOxford|r",
                            ["bName"] = "pdkp_dkp_entry1",
                            ["canEdit"] = false,
                            ["name"] = "Oxford",
                        }, -- [167]
                        {
                            ["inRaid"] = false,
                            ["class"] = "Priest",
                            ["dkpTotal"] = 298,
                            ["online"] = false,
                            ["rankIndex"] = 5,
                            ["formattedName"] = "|cffFFFFFFPhilonious|r",
                            ["canEdit"] = false,
                            ["name"] = "Philonious",
                            ["bName"] = "pdkp_dkp_entry26",
                        }, -- [168]
                        {
                            ["inRaid"] = false,
                            ["class"] = "Priest",
                            ["dkpTotal"] = 390,
                            ["online"] = false,
                            ["rankIndex"] = 5,
                            ["formattedName"] = "|cffFFFFFFPriesticuffs|r",
                            ["bName"] = "pdkp_dkp_entry10",
                            ["canEdit"] = false,
                            ["name"] = "Priesticuffs",
                        }, -- [169]
                        {
                            ["inRaid"] = true,
                            ["class"] = "Hunter",
                            ["dkpTotal"] = 300,
                            ["online"] = false,
                            ["rankIndex"] = 5,
                            ["formattedName"] = "|cffABD473Primera|r",
                            ["bName"] = "pdkp_dkp_entry18",
                            ["canEdit"] = false,
                            ["name"] = "Primera",
                        }, -- [170]
                        {
                            ["inRaid"] = false,
                            ["class"] = "Priest",
                            ["dkpTotal"] = 100,
                            ["online"] = false,
                            ["rankIndex"] = 5,
                            ["formattedName"] = "|cffFFFFFFPuffhead|r",
                            ["bName"] = "pdkp_dkp_entry1",
                            ["canEdit"] = false,
                            ["name"] = "Puffhead",
                        }, -- [171]
                        {
                            ["inRaid"] = false,
                            ["class"] = "Warlock",
                            ["dkpTotal"] = 350,
                            ["online"] = false,
                            ["rankIndex"] = 5,
                            ["formattedName"] = "|cff8787EDRaspütin|r",
                            ["bName"] = "pdkp_dkp_entry12",
                            ["canEdit"] = false,
                            ["name"] = "Raspütin",
                        }, -- [172]
                        {
                            ["inRaid"] = true,
                            ["class"] = "Warrior",
                            ["dkpTotal"] = 255,
                            ["online"] = false,
                            ["rankIndex"] = 5,
                            ["formattedName"] = "|cffC79C6EReina|r",
                            ["bName"] = "pdkp_dkp_entry2",
                            ["canEdit"] = false,
                            ["name"] = "Reina",
                        }, -- [173]
                        {
                            ["inRaid"] = true,
                            ["class"] = "Priest",
                            ["dkpTotal"] = 280,
                            ["online"] = false,
                            ["rankIndex"] = 5,
                            ["formattedName"] = "|cffFFFFFFRez|r",
                            ["bName"] = "pdkp_dkp_entry1",
                            ["canEdit"] = false,
                            ["name"] = "Rez",
                        }, -- [174]
                        {
                            ["dkpTotal"] = 0,
                            ["online"] = false,
                            ["name"] = "Shadowheals",
                            ["formattedName"] = "|cffFFFFFFShadowheals|r",
                            ["canEdit"] = false,
                            ["class"] = "Priest",
                            ["inRaid"] = false,
                            ["rankIndex"] = 5,
                        }, -- [175]
                        {
                            ["inRaid"] = false,
                            ["class"] = "Warrior",
                            ["dkpTotal"] = 347,
                            ["online"] = false,
                            ["rankIndex"] = 5,
                            ["formattedName"] = "|cffC79C6EShinynickels|r",
                            ["bName"] = "pdkp_dkp_entry22",
                            ["canEdit"] = false,
                            ["name"] = "Shinynickels",
                        }, -- [176]
                        {
                            ["inRaid"] = false,
                            ["class"] = "Rogue",
                            ["dkpTotal"] = 159,
                            ["online"] = false,
                            ["rankIndex"] = 5,
                            ["formattedName"] = "|cffFFF569Slipperyjohn|r",
                            ["bName"] = "pdkp_dkp_entry14",
                            ["canEdit"] = false,
                            ["name"] = "Slipperyjohn",
                        }, -- [177]
                        {
                            ["dkpTotal"] = 0,
                            ["online"] = false,
                            ["name"] = "Sneakattac",
                            ["formattedName"] = "|cffFFF569Sneakattac|r",
                            ["canEdit"] = false,
                            ["class"] = "Rogue",
                            ["inRaid"] = false,
                            ["rankIndex"] = 6,
                        }, -- [178]
                        {
                            ["inRaid"] = true,
                            ["class"] = "Warlock",
                            ["dkpTotal"] = 275,
                            ["online"] = false,
                            ["rankIndex"] = 5,
                            ["formattedName"] = "|cff8787EDSoftfondle|r",
                            ["bName"] = "pdkp_dkp_entry19",
                            ["canEdit"] = false,
                            ["name"] = "Softfondle",
                        }, -- [179]
                        {
                            ["inRaid"] = false,
                            ["class"] = "Mage",
                            ["dkpTotal"] = 100,
                            ["online"] = false,
                            ["rankIndex"] = 5,
                            ["formattedName"] = "|cff40C7EBSpellbender|r",
                            ["bName"] = "pdkp_dkp_entry1",
                            ["canEdit"] = false,
                            ["name"] = "Spellbender",
                        }, -- [180]
                        {
                            ["inRaid"] = true,
                            ["class"] = "Hunter",
                            ["dkpTotal"] = 94,
                            ["online"] = false,
                            ["rankIndex"] = 5,
                            ["formattedName"] = "|cffABD473Stellâ|r",
                            ["bName"] = "pdkp_dkp_entry1",
                            ["canEdit"] = false,
                            ["name"] = "Stellâ",
                        }, -- [181]
                        {
                            ["inRaid"] = false,
                            ["class"] = "Mage",
                            ["dkpTotal"] = 400,
                            ["online"] = false,
                            ["rankIndex"] = 5,
                            ["formattedName"] = "|cff40C7EBStitchess|r",
                            ["bName"] = "pdkp_dkp_entry1",
                            ["canEdit"] = false,
                            ["name"] = "Stitchess",
                        }, -- [182]
                        {
                            ["inRaid"] = true,
                            ["class"] = "Mage",
                            ["dkpTotal"] = 300,
                            ["online"] = false,
                            ["rankIndex"] = 5,
                            ["formattedName"] = "|cff40C7EBSuprarz|r",
                            ["bName"] = "pdkp_dkp_entry2",
                            ["canEdit"] = false,
                            ["name"] = "Suprarz",
                        }, -- [183]
                        {
                            ["dkpTotal"] = 0,
                            ["online"] = false,
                            ["name"] = "Tankdaddy",
                            ["formattedName"] = "|cffC79C6ETankdaddy|r",
                            ["canEdit"] = false,
                            ["class"] = "Warrior",
                            ["inRaid"] = false,
                            ["rankIndex"] = 5,
                        }, -- [184]
                        {
                            ["inRaid"] = true,
                            ["class"] = "Warlock",
                            ["dkpTotal"] = 232,
                            ["online"] = false,
                            ["rankIndex"] = 5,
                            ["formattedName"] = "|cff8787EDThepurple|r",
                            ["bName"] = "pdkp_dkp_entry24",
                            ["canEdit"] = false,
                            ["name"] = "Thepurple",
                        }, -- [185]
                        {
                            ["inRaid"] = true,
                            ["class"] = "Warrior",
                            ["dkpTotal"] = 300,
                            ["online"] = false,
                            ["rankIndex"] = 5,
                            ["formattedName"] = "|cffC79C6ETokentoken|r",
                            ["bName"] = "pdkp_dkp_entry1",
                            ["canEdit"] = false,
                            ["name"] = "Tokentoken",
                        }, -- [186]
                        {
                            ["dkpTotal"] = 100,
                            ["online"] = false,
                            ["name"] = "Trickster",
                            ["formattedName"] = "|cffFFF569Trickster|r",
                            ["canEdit"] = false,
                            ["class"] = "Rogue",
                            ["inRaid"] = true,
                            ["rankIndex"] = 6,
                        }, -- [187]
                        {
                            ["inRaid"] = true,
                            ["class"] = "Rogue",
                            ["dkpTotal"] = 250,
                            ["online"] = false,
                            ["rankIndex"] = 5,
                            ["formattedName"] = "|cffFFF569Tuggernuts|r",
                            ["bName"] = "pdkp_dkp_entry11",
                            ["canEdit"] = false,
                            ["name"] = "Tuggernuts",
                        }, -- [188]
                        {
                            ["inRaid"] = false,
                            ["class"] = "Priest",
                            ["dkpTotal"] = 159,
                            ["online"] = false,
                            ["rankIndex"] = 5,
                            ["formattedName"] = "|cffFFFFFFTurbohealz|r",
                            ["bName"] = "pdkp_dkp_entry26",
                            ["canEdit"] = false,
                            ["name"] = "Turbohealz",
                        }, -- [189]
                        {
                            ["inRaid"] = true,
                            ["class"] = "Hunter",
                            ["dkpTotal"] = 300,
                            ["online"] = false,
                            ["rankIndex"] = 5,
                            ["formattedName"] = "|cffABD473Ugro|r",
                            ["bName"] = "pdkp_dkp_entry1",
                            ["canEdit"] = false,
                            ["name"] = "Ugro",
                        }, -- [190]
                        {
                            ["inRaid"] = true,
                            ["class"] = "Warlock",
                            ["dkpTotal"] = 291,
                            ["online"] = false,
                            ["rankIndex"] = 5,
                            ["formattedName"] = "|cff8787EDVarix|r",
                            ["bName"] = "pdkp_dkp_entry1",
                            ["canEdit"] = false,
                            ["name"] = "Varix",
                        }, -- [191]
                        {
                            ["inRaid"] = true,
                            ["class"] = "Mage",
                            ["dkpTotal"] = 264,
                            ["online"] = false,
                            ["rankIndex"] = 5,
                            ["formattedName"] = "|cff40C7EBZukohere|r",
                            ["bName"] = "pdkp_dkp_entry24",
                            ["canEdit"] = false,
                            ["name"] = "Zukohere",
                        }, -- [192]
                        {
                            ["inRaid"] = false,
                            ["class"] = "Paladin",
                            ["dkpTotal"] = 305,
                            ["online"] = false,
                            ["rankIndex"] = 4,
                            ["formattedName"] = "|cffF58CBAAdvanty|r",
                            ["bName"] = "pdkp_dkp_entry9",
                            ["canEdit"] = false,
                            ["name"] = "Advanty",
                        }, -- [193]
                        {
                            ["inRaid"] = true,
                            ["class"] = "Mage",
                            ["dkpTotal"] = 300,
                            ["online"] = false,
                            ["rankIndex"] = 4,
                            ["formattedName"] = "|cff40C7EBBugaboom|r",
                            ["bName"] = "pdkp_dkp_entry21",
                            ["canEdit"] = false,
                            ["name"] = "Bugaboom",
                        }, -- [194]
                        {
                            ["inRaid"] = false,
                            ["class"] = "Hunter",
                            ["dkpTotal"] = 83,
                            ["online"] = false,
                            ["rankIndex"] = 4,
                            ["formattedName"] = "|cffABD473Ithgar|r",
                            ["bName"] = "pdkp_dkp_entry3",
                            ["canEdit"] = false,
                            ["name"] = "Ithgar",
                        }, -- [195]
                        {
                            ["inRaid"] = false,
                            ["class"] = "Warlock",
                            ["dkpTotal"] = 362,
                            ["online"] = false,
                            ["rankIndex"] = 4,
                            ["formattedName"] = "|cff8787EDLindo|r",
                            ["bName"] = "pdkp_dkp_entry16",
                            ["canEdit"] = false,
                            ["name"] = "Lindo",
                        }, -- [196]
                        {
                            ["inRaid"] = true,
                            ["class"] = "Rogue",
                            ["dkpTotal"] = 270,
                            ["online"] = false,
                            ["rankIndex"] = 4,
                            ["formattedName"] = "|cffFFF569Mystile|r",
                            ["bName"] = "pdkp_dkp_entry3",
                            ["canEdit"] = false,
                            ["name"] = "Mystile",
                        }, -- [197]
                        {
                            ["inRaid"] = true,
                            ["class"] = "Warrior",
                            ["dkpTotal"] = 300,
                            ["online"] = false,
                            ["rankIndex"] = 4,
                            ["formattedName"] = "|cffC79C6ERetkin|r",
                            ["bName"] = "pdkp_dkp_entry24",
                            ["canEdit"] = false,
                            ["name"] = "Retkin",
                        }, -- [198]
                        {
                            ["inRaid"] = false,
                            ["class"] = "Hunter",
                            ["dkpTotal"] = 400,
                            ["online"] = false,
                            ["rankIndex"] = 3,
                            ["formattedName"] = "|cffABD473Athico|r",
                            ["bName"] = "pdkp_dkp_entry9",
                            ["canEdit"] = true,
                            ["name"] = "Athico",
                        }, -- [199]
                        {
                            ["inRaid"] = true,
                            ["class"] = "Warrior",
                            ["dkpTotal"] = 300,
                            ["online"] = true,
                            ["rankIndex"] = 3,
                            ["formattedName"] = "|cffC79C6ESnaildaddy|r",
                            ["bName"] = "pdkp_dkp_entry1",
                            ["canEdit"] = true,
                            ["name"] = "Snaildaddy",
                        }, -- [200]
                        {
                            ["inRaid"] = false,
                            ["class"] = "Warlock",
                            ["dkpTotal"] = 275,
                            ["online"] = false,
                            ["rankIndex"] = 2,
                            ["formattedName"] = "|cff8787EDDalia|r",
                            ["bName"] = "pdkp_dkp_entry5",
                            ["canEdit"] = true,
                            ["name"] = "Dalia",
                        }, -- [201]
                        {
                            ["inRaid"] = false,
                            ["class"] = "Mage",
                            ["dkpTotal"] = 107,
                            ["online"] = false,
                            ["rankIndex"] = 2,
                            ["formattedName"] = "|cff40C7EBEolith|r",
                            ["bName"] = "pdkp_dkp_entry2",
                            ["canEdit"] = true,
                            ["name"] = "Eolith",
                        }, -- [202]
                        {
                            ["inRaid"] = true,
                            ["class"] = "Warrior",
                            ["dkpTotal"] = 200,
                            ["online"] = false,
                            ["rankIndex"] = 1,
                            ["formattedName"] = "|cffC79C6EGalagus|r",
                            ["bName"] = "pdkp_dkp_entry4",
                            ["canEdit"] = true,
                            ["name"] = "Galagus",
                        }, -- [203]
                        {
                            ["inRaid"] = false,
                            ["class"] = "Druid",
                            ["dkpTotal"] = 400,
                            ["online"] = false,
                            ["rankIndex"] = 0,
                            ["formattedName"] = "|cffFF7D0ANeekio|r",
                            ["bName"] = "pdkp_dkp_entry3",
                            ["canEdit"] = true,
                            ["name"] = "Neekio",
                        }, -- [204]
                        {
                            ["dkpTotal"] = 0,
                            ["online"] = false,
                            ["name"] = "Rickjamesbch",
                            ["formattedName"] = "|cff40C7EBRickjamesbch|r",
                            ["canEdit"] = false,
                            ["class"] = "Mage",
                            ["rankIndex"] = 7,
                            ["inRaid"] = false,
                        }, -- [205]
                        {
                            ["dkpTotal"] = 0,
                            ["online"] = false,
                            ["name"] = "Eesha",
                            ["formattedName"] = "|cff8787EDEesha|r",
                            ["canEdit"] = false,
                            ["class"] = "Warlock",
                            ["rankIndex"] = 7,
                            ["inRaid"] = false,
                        }, -- [206]
                        {
                            ["dkpTotal"] = 0,
                            ["online"] = false,
                            ["name"] = "Malorthroot",
                            ["formattedName"] = "|cffABD473Malorthroot|r",
                            ["canEdit"] = false,
                            ["class"] = "Hunter",
                            ["rankIndex"] = 7,
                            ["inRaid"] = false,
                        }, -- [207]
                        {
                            ["dkpTotal"] = 0,
                            ["online"] = false,
                            ["name"] = "Dessiola",
                            ["formattedName"] = "|cffF58CBADessiola|r",
                            ["canEdit"] = false,
                            ["class"] = "Paladin",
                            ["rankIndex"] = 7,
                            ["inRaid"] = false,
                        }, -- [208]
                        {
                            ["dkpTotal"] = 0,
                            ["online"] = false,
                            ["name"] = "Maroz",
                            ["formattedName"] = "|cff40C7EBMaroz|r",
                            ["canEdit"] = false,
                            ["class"] = "Mage",
                            ["rankIndex"] = 7,
                            ["inRaid"] = false,
                        }, -- [209]
                        {
                            ["dkpTotal"] = 0,
                            ["online"] = false,
                            ["name"] = "Terek",
                            ["formattedName"] = "|cffFFF569Terek|r",
                            ["canEdit"] = false,
                            ["class"] = "Rogue",
                            ["rankIndex"] = 7,
                            ["inRaid"] = false,
                        }, -- [210]
                        {
                            ["dkpTotal"] = 0,
                            ["online"] = false,
                            ["name"] = "Randomossity",
                            ["formattedName"] = "|cffF58CBARandomossity|r",
                            ["canEdit"] = false,
                            ["class"] = "Paladin",
                            ["rankIndex"] = 7,
                            ["inRaid"] = false,
                        }, -- [211]
                        {
                            ["dkpTotal"] = 0,
                            ["online"] = false,
                            ["name"] = "Turboboom",
                            ["formattedName"] = "|cffFF7D0ATurboboom|r",
                            ["canEdit"] = false,
                            ["class"] = "Druid",
                            ["rankIndex"] = 7,
                            ["inRaid"] = false,
                        }, -- [212]
                        {
                            ["inRaid"] = false,
                            ["class"] = "Priest",
                            ["dkpTotal"] = 0,
                            ["online"] = false,
                            ["rankIndex"] = 3,
                            ["formattedName"] = "|cffFFFFFFVeltrix|r",
                            ["bName"] = "pdkp_dkp_entry2",
                            ["canEdit"] = true,
                            ["name"] = "Veltrix",
                        }, -- [213]
                    },
            },
        dkpDB = {
            lastEdit=1585807486,
            history = {
                deleted = {
                    1585173018, -- [1]
                    1585191793, -- [2]
                    1585777354, -- [3]
                    1585777354, -- [4]
                    1585689243, -- [5]
                    1585692977, -- [6]
                    1585779636, -- [7]
                    1585790445, -- [8]
                    1585790425, -- [9]
                    1585790300, -- [10]
                    1585790267, -- [11]
                    1585790209, -- [12]
                    1585790181, -- [13]
                    1585790126, -- [14]
                    1585790116, -- [15]
                    1585790092, -- [16]
                    1585789943, -- [17]
                    1585789896, -- [18]
                    1585789840, -- [19]
                    1585789592, -- [20]
                    1585789586, -- [21]
                    1585789068, -- [22]
                    1585779744, -- [23]
                    1585779734, -- [24]
                },
                all = {
                    {
                        ["reason"] = "Boss Kill",
                        ["id"] = 1585801611,
                        ["text"] = "|cff22bb33Blackwing Lair - Nefarian|r",
                        ["bossKill"] = "Nefarian",
                        ["names"] = "|cff8787EDLittledubs|r",
                        ["datetime"] = 1585801613,
                        ["dkpChangeText"] = "|cff22bb3320 DKP|r",
                        ["time"] = "09:26:53 PM",
                        ["date"] = "04/01/20",
                        ["dkpChange"] = 20,
                        ["raid"] = "Blackwing Lair",
                        ["serverTime"] = 1585801611,
                        ["officer"] = "|cffFFFFFFVeltrix|r",
                    }, -- [1]
                    [1585196021] = {
                        ["names"] = "|cff40C7EBEolith|r",
                        ["id"] = 1585196021,
                        ["text"] = "|cffE71D36Item Win - |rNone",
                        ["dkpChangeText"] = "|cffE71D36-65 DKP|r",
                        ["reason"] = "Item Win",
                        ["datetime"] = 1585196023,
                        ["item"] = "None",
                        ["time"] = "09:13:43 PM",
                        ["date"] = "03/25/20",
                        ["dkpChange"] = -65,
                        ["raid"] = "Blackwing Lair",
                        ["serverTime"] = 1585196021,
                        ["officer"] = "|cff8787EDSparklenips|r",
                    },
                    [1585199340] = {
                        ["names"] = "|cff8787EDSparklenips|r",
                        ["id"] = 1585199340,
                        ["text"] = "|cffE71D36Item Win - |rNone",
                        ["dkpChangeText"] = "|cffE71D36-75 DKP|r",
                        ["reason"] = "Item Win",
                        ["datetime"] = 1585199342,
                        ["item"] = "None",
                        ["time"] = "10:09:02 PM",
                        ["date"] = "03/25/20",
                        ["dkpChange"] = -75,
                        ["raid"] = "Blackwing Lair",
                        ["serverTime"] = 1585199340,
                        ["officer"] = "|cff8787EDSparklenips|r",
                    },
                    [1585189152] = {
                        ["reason"] = "Item Win",
                        ["id"] = 1585189152,
                        ["text"] = "|cffE71D36Item Win - |r|cffa335ee|Hitem:19437::::::::60:::::::|h[Boots of Pure Thought]|h|r",
                        ["item"] = "|cffa335ee|Hitem:19437::::::::60:::::::|h[Boots of Pure Thought]|h|r",
                        ["names"] = "|cffF58CBAMaryjohanna|r",
                        ["datetime"] = 1585189153,
                        ["dkpChangeText"] = "|cffE71D36-65 DKP|r",
                        ["time"] = "10:19:13 PM",
                        ["date"] = "03/25/20",
                        ["dkpChange"] = -65,
                        ["raid"] = "Blackwing Lair",
                        ["serverTime"] = 1585189152,
                        ["officer"] = "|cffFF7D0ANeekio|r",
                    },
                    [1585173091] = {
                        ["id"] = 1585173091,
                        ["text"] = "|cff22bb33Blackwing Lair - Completion Bonus|r",
                        ["names"] = "|cffFFFFFFMongous|r, |cffFFFFFFHoneypot|r",
                        ["reason"] = "Completion Bonus",
                        ["datetime"] = 1585173092,
                        ["dkpChangeText"] = "|cff22bb33100 DKP|r",
                        ["time"] = "02:51:32 PM",
                        ["date"] = "03/25/20",
                        ["dkpChange"] = 100,
                        ["raid"] = "Blackwing Lair",
                        ["serverTime"] = 1585173091,
                        ["officer"] = "|cff8787EDSparklenips|r",
                    },
                    [1585806519] = {
                        ["names"] = "|cff40C7EBOxford|r",
                        ["id"] = 1585806519,
                        ["text"] = "|cffE71D36Item Win - |rNone",
                        ["time"] = "10:48:39 PM",
                        ["reason"] = "Item Win",
                        ["datetime"] = 1585806519,
                        ["dkpChangeText"] = "|cffE71D36-145 DKP|r",
                        ["item"] = "None",
                        ["date"] = "04/01/20",
                        ["dkpChange"] = -145,
                        ["raid"] = "Blackwing Lair",
                        ["serverTime"] = 1585806519,
                        ["officer"] = "|cffC79C6ESnaildaddy|r",
                    },
                    [1585173617] = {
                        ["id"] = 1585173617,
                        ["text"] = "|cff22bb33Blackwing Lair - Completion Bonus|r",
                        ["names"] = "|cff40C7EBSpellbender|r",
                        ["reason"] = "Completion Bonus",
                        ["datetime"] = 1585173618,
                        ["dkpChangeText"] = "|cff22bb33100 DKP|r",
                        ["time"] = "03:00:18 PM",
                        ["date"] = "03/25/20",
                        ["dkpChange"] = 100,
                        ["raid"] = "Blackwing Lair",
                        ["serverTime"] = 1585173617,
                        ["officer"] = "|cff8787EDSparklenips|r",
                    },
                    [1585193515] = {
                        ["names"] = "|cff8787EDLindo|r",
                        ["id"] = 1585193515,
                        ["text"] = "|cffE71D36Item Win - |r|cffa335ee|Hitem:16928::::::::60:::::::|h[Nemesis Gloves]|h|r",
                        ["dkpChangeText"] = "|cffE71D36-13 DKP|r",
                        ["reason"] = "Item Win",
                        ["datetime"] = 1585193516,
                        ["time"] = "11:31:56 PM",
                        ["item"] = "|cffa335ee|Hitem:16928::::::::60:::::::|h[Nemesis Gloves]|h|r",
                        ["date"] = "03/25/20",
                        ["dkpChange"] = -13,
                        ["raid"] = "Blackwing Lair",
                        ["serverTime"] = 1585193515,
                        ["officer"] = "|cffFF7D0ANeekio|r",
                    },
                    [1585200659] = {
                        ["names"] = "|cffC79C6ENightshelf|r",
                        ["id"] = 1585200659,
                        ["text"] = "|cffE71D36Item Win - |rNone",
                        ["dkpChangeText"] = "|cffE71D36-67 DKP|r",
                        ["reason"] = "Item Win",
                        ["datetime"] = 1585200661,
                        ["item"] = "None",
                        ["time"] = "10:31:01 PM",
                        ["date"] = "03/25/20",
                        ["dkpChange"] = -67,
                        ["raid"] = "Blackwing Lair",
                        ["serverTime"] = 1585200659,
                        ["officer"] = "|cff8787EDSparklenips|r",
                    },
                    [1585173382] = {
                        ["id"] = 1585173382,
                        ["text"] = "|cff22bb33Blackwing Lair - Completion Bonus|r",
                        ["names"] = "|cffFF7D0AEmmyy|r",
                        ["reason"] = "Completion Bonus",
                        ["datetime"] = 1585173383,
                        ["dkpChangeText"] = "|cff22bb3340 DKP|r",
                        ["time"] = "02:56:23 PM",
                        ["date"] = "03/25/20",
                        ["dkpChange"] = 40,
                        ["raid"] = "Blackwing Lair",
                        ["serverTime"] = 1585173382,
                        ["officer"] = "|cff8787EDSparklenips|r",
                    },
                    [1585199149] = {
                        ["names"] = "|cffFFF569Inigma|r",
                        ["id"] = 1585199149,
                        ["text"] = "|cffE71D36Item Win - |rNone",
                        ["dkpChangeText"] = "|cffE71D36-50 DKP|r",
                        ["reason"] = "Item Win",
                        ["datetime"] = 1585199151,
                        ["item"] = "None",
                        ["time"] = "10:05:51 PM",
                        ["date"] = "03/25/20",
                        ["dkpChange"] = -50,
                        ["raid"] = "Blackwing Lair",
                        ["serverTime"] = 1585199149,
                        ["officer"] = "|cff8787EDSparklenips|r",
                    },
                    [1585173402] = {
                        ["id"] = 1585173402,
                        ["text"] = "|cff22bb33Blackwing Lair - Completion Bonus|r",
                        ["names"] = "|cffABD473Ithgar|r",
                        ["reason"] = "Completion Bonus",
                        ["datetime"] = 1585173403,
                        ["dkpChangeText"] = "|cff22bb3337 DKP|r",
                        ["time"] = "02:56:43 PM",
                        ["date"] = "03/25/20",
                        ["dkpChange"] = 37,
                        ["raid"] = "Blackwing Lair",
                        ["serverTime"] = 1585173402,
                        ["officer"] = "|cff8787EDSparklenips|r",
                    },
                    [1585798152] = {
                        ["id"] = 1585798152,
                        ["text"] = "|cff22bb33Blackwing Lair - On Time Bonus|r",
                        ["reason"] = "On Time Bonus",
                        ["names"] = "|cffFF7D0ABlanebites|r, |cffFF7D0AInebriated|r, |cffFF7D0AKalijah|r, |cffFF7D0AInsub|r, |cffABD473Primera|r, |cffABD473Ugro|r, |cffABD473Wahcha|r, |cffABD473Stellâ|r, |cff40C7EBZukohere|r, |cff40C7EBSuprarz|r, |cff40C7EBBugaboom|r, |cff40C7EBOxford|r, |cff40C7EBChipgizmo|r, |cffF58CBALawduk|r, |cffF58CBAGneissguy|r, |cffF58CBAOnes|r, |cffFFFFFFMongous|r, |cffFFFFFFEvangelina|r, |cffFFFFFFHoneypot|r, |cffFFFFFFFawntine|r, |cffFFFFFFRez|r, |cffFFF569Trickster|r, |cffFFF569Inigma|r, |cffFFF569Aku|r, |cffFFF569Mystile|r, |cffFFF569Tuggernuts|r, |cff8787EDIhurricanel|r, |cff8787EDTalisker|r, |cff8787EDThepurple|r, |cff8787EDLittledubs|r, |cff8787EDSoftfondle|r, |cff8787EDVarix|r, |cffC79C6EGalagus|r, |cffC79C6EReina|r, |cffC79C6EJeffry|r, |cffC79C6ENightshelf|r, |cffC79C6ETokentoken|r, |cffC79C6ELuckerdawg|r, |cffC79C6ESnaildaddy|r, |cffC79C6ERetkin|r",
                        ["datetime"] = 1585798152,
                        ["dkpChangeText"] = "|cff22bb3310 DKP|r",
                        ["time"] = "08:29:12 PM",
                        ["date"] = "04/01/20",
                        ["dkpChange"] = 10,
                        ["raid"] = "Blackwing Lair",
                        ["serverTime"] = 1585798152,
                        ["officer"] = "|cffC79C6ESnaildaddy|r",
                    },
                    [1585173151] = {
                        ["id"] = 1585173151,
                        ["text"] = "|cff22bb33Blackwing Lair - Completion Bonus|r",
                        ["names"] = "|cff40C7EBOxford|r, |cff40C7EBSuprarz|r, |cff40C7EBJakemeoff|r, |cff40C7EBBugaboom|r",
                        ["reason"] = "Completion Bonus",
                        ["datetime"] = 1585173152,
                        ["dkpChangeText"] = "|cff22bb33100 DKP|r",
                        ["time"] = "02:52:32 PM",
                        ["date"] = "03/25/20",
                        ["dkpChange"] = 100,
                        ["raid"] = "Blackwing Lair",
                        ["serverTime"] = 1585173151,
                        ["officer"] = "|cff8787EDSparklenips|r",
                    },
                    [1585792295] = {
                        ["names"] = "|cffC79C6EQew|r",
                        ["id"] = 1585792295,
                        ["text"] = "|cffE71D36Item Win - |rNone",
                        ["item"] = "None",
                        ["reason"] = "Item Win",
                        ["datetime"] = 1585792297,
                        ["dkpChangeText"] = "|cffE71D36-10 DKP|r",
                        ["time"] = "06:51:37 PM",
                        ["date"] = "04/01/20",
                        ["dkpChange"] = -10,
                        ["raid"] = "Blackwing Lair",
                        ["serverTime"] = 1585792295,
                        ["officer"] = "|cff8787EDSparklenips|r",
                    },
                    [1585173159] = {
                        ["id"] = 1585173159,
                        ["text"] = "|cff22bb33Blackwing Lair - Completion Bonus|r",
                        ["names"] = "|cff40C7EBZukohere|r",
                        ["reason"] = "Completion Bonus",
                        ["datetime"] = 1585173160,
                        ["dkpChangeText"] = "|cff22bb3380 DKP|r",
                        ["time"] = "02:52:40 PM",
                        ["date"] = "03/25/20",
                        ["dkpChange"] = 80,
                        ["raid"] = "Blackwing Lair",
                        ["serverTime"] = 1585173159,
                        ["officer"] = "|cff8787EDSparklenips|r",
                    },
                    [1585800212] = {
                        ["names"] = "|cffFF7D0AMorphintyme|r, |cffFF7D0ANeekio|r, |cffFF7D0AEmmyy|r, |cffABD473Athico|r, |cffABD473Bigbootyhoho|r, |cffABD473Bádtothebow|r, |cffABD473Cyskul|r, |cffABD473Ithgar|r, |cff40C7EBVeriandra|r, |cff40C7EBNeckbeardo|r, |cff40C7EBStitchess|r, |cff40C7EBEolith|r, |cff40C7EBXyen|r, |cff40C7EBFlatulent|r, |cff40C7EBJakemeoff|r, |cffF58CBAOdin|r, |cffF58CBAAdvanty|r, |cffF58CBAMaryjohanna|r, |cffF58CBAPapasquach|r, |cffFFFFFFPuffhead|r, |cffFFFFFFBallour|r, |cffFFFFFFTurbohealz|r, |cffFFFFFFPriesticuffs|r, |cffFFFFFFPhilonious|r, |cffFFF569Mcstaberson|r, |cffFFF569Sicarrio|r, |cffFFF569Slipperyjohn|r, |cffFFF569Mariku|r, |cff8787EDLindo|r, |cff8787EDEdgyboi|r, |cff8787EDRaspütin|r, |cff8787EDSparklenips|r, |cff8787EDDalia|r, |cffC79C6EKalita|r, |cffC79C6EQew|r, |cffC79C6EHotdogbroth|r, |cffC79C6EOragan|r, |cffC79C6EShinynickels|r, |cffC79C6EGoobimus|r, |cffC79C6EGirlslayer|r",
                        ["id"] = 1585800212,
                        ["text"] = "|cff22bb33Blackwing Lair - Nefarian|r",
                        ["bossKill"] = "Nefarian",
                        ["reason"] = "Boss Kill",
                        ["datetime"] = 1585800214,
                        ["time"] = "09:03:34 PM",
                        ["dkpChangeText"] = "|cff22bb3320 DKP|r",
                        ["date"] = "04/01/20",
                        ["dkpChange"] = 20,
                        ["raid"] = "Blackwing Lair",
                        ["serverTime"] = 1585800212,
                        ["officer"] = "|cff8787EDSparklenips|r",
                    },
                    [1585173685] = {
                        ["id"] = 1585173685,
                        ["text"] = "|cff22bb33Blackwing Lair - Completion Bonus|r",
                        ["names"] = "|cffFFF569Mcstaberson|r, |cffFFF569Mozzarella|r, |cffFFF569Mariku|r",
                        ["reason"] = "Completion Bonus",
                        ["datetime"] = 1585173686,
                        ["dkpChangeText"] = "|cff22bb33100 DKP|r",
                        ["time"] = "03:01:26 PM",
                        ["date"] = "03/25/20",
                        ["dkpChange"] = 100,
                        ["raid"] = "Blackwing Lair",
                        ["serverTime"] = 1585173685,
                        ["officer"] = "|cff8787EDSparklenips|r",
                    },
                    [1585187208] = {
                        ["reason"] = "Item Win",
                        ["id"] = 1585187208,
                        ["text"] = "|cffE71D36Item Win - |r|cffa335ee|Hitem:16926::::::::60:::::::|h[Bindings of Transcendence]|h|r",
                        ["item"] = "|cffa335ee|Hitem:16926::::::::60:::::::|h[Bindings of Transcendence]|h|r",
                        ["names"] = "|cffFFFFFFPhilonious|r",
                        ["datetime"] = 1585187209,
                        ["dkpChangeText"] = "|cffE71D36-2 DKP|r",
                        ["time"] = "09:46:49 PM",
                        ["date"] = "03/25/20",
                        ["dkpChange"] = -2,
                        ["raid"] = "Blackwing Lair",
                        ["serverTime"] = 1585187208,
                        ["officer"] = "|cffFF7D0ANeekio|r",
                    },
                    [1585195125] = {
                        ["names"] = "|cffC79C6EMihai|r",
                        ["id"] = 1585195125,
                        ["text"] = "|cffE71D36Item Win - |r|cffa335ee|Hitem:19394::::::::60:::::::|h[Drake Talon Pauldrons]|h|r",
                        ["dkpChangeText"] = "|cffE71D36-79 DKP|r",
                        ["reason"] = "Item Win",
                        ["datetime"] = 1585195126,
                        ["time"] = "11:58:46 PM",
                        ["item"] = "|cffa335ee|Hitem:19394::::::::60:::::::|h[Drake Talon Pauldrons]|h|r",
                        ["date"] = "03/25/20",
                        ["dkpChange"] = -79,
                        ["raid"] = "Blackwing Lair",
                        ["serverTime"] = 1585195125,
                        ["officer"] = "|cffFF7D0ANeekio|r",
                    },
                    [1585800499] = {
                        ["names"] = "|cffFF7D0ABlanebites|r, |cffFF7D0AInebriated|r, |cffFF7D0AKalijah|r, |cffFF7D0AInsub|r, |cffABD473Primera|r, |cffABD473Ugro|r, |cffABD473Wahcha|r, |cffABD473Stellâ|r, |cff40C7EBZukohere|r, |cff40C7EBSuprarz|r, |cff40C7EBBugaboom|r, |cff40C7EBOxford|r, |cff40C7EBChipgizmo|r, |cffF58CBALawduk|r, |cffF58CBAOnes|r, |cffF58CBAGneissguy|r, |cffFFFFFFMongous|r, |cffFFFFFFRez|r, |cffFFFFFFFawntine|r, |cffFFFFFFHoneypot|r, |cffFFFFFFEvangelina|r, |cffFFF569Mystile|r, |cffFFF569Inigma|r, |cffFFF569Trickster|r, |cffFFF569Tuggernuts|r, |cffFFF569Aku|r, |cff8787EDIhurricanel|r, |cff8787EDSoftfondle|r, |cff8787EDThepurple|r, |cff8787EDLittledubs|r, |cff8787EDVarix|r, |cff8787EDTalisker|r, |cffC79C6ETokentoken|r, |cffC79C6ERetkin|r, |cffC79C6EReina|r, |cffC79C6ESnaildaddy|r, |cffC79C6EGalagus|r, |cffC79C6EJeffry|r, |cffC79C6ENightshelf|r, |cffC79C6ELuckerdawg|r",
                        ["id"] = 1585800499,
                        ["text"] = "|cff22bb33Blackwing Lair - Broodlord Lashlayer|r",
                        ["bossKill"] = "Broodlord Lashlayer",
                        ["reason"] = "Boss Kill",
                        ["datetime"] = 1585800499,
                        ["time"] = "09:08:19 PM",
                        ["dkpChangeText"] = "|cff22bb3310 DKP|r",
                        ["date"] = "04/01/20",
                        ["dkpChange"] = 10,
                        ["raid"] = "Blackwing Lair",
                        ["serverTime"] = 1585800499,
                        ["officer"] = "|cffC79C6ESnaildaddy|r",
                    },
                    [1585770672] = {
                        ["names"] = "|cff40C7EBUrkh|r",
                        ["id"] = 1585770672,
                        ["text"] = "|cffE71D36Item Win - |rNone",
                        ["time"] = "12:51:13 PM",
                        ["reason"] = "Item Win",
                        ["datetime"] = 1585770673,
                        ["dkpChangeText"] = "|cffE71D36-10 DKP|r",
                        ["item"] = "None",
                        ["date"] = "04/01/20",
                        ["dkpChange"] = -10,
                        ["raid"] = "Blackwing Lair",
                        ["serverTime"] = 1585770672,
                        ["officer"] = "|cff8787EDSparklenips|r",
                    },
                    [1585197436] = {
                        ["reason"] = "Item Win",
                        ["id"] = 1585197436,
                        ["text"] = "|cffE71D36Item Win - |r|cffa335ee|Hitem:16964::::::::60:::::::|h[Gauntlets of Wrath]|h|r",
                        ["time"] = "12:37:17 AM",
                        ["names"] = "|cffC79C6EOragan|r",
                        ["datetime"] = 1585197437,
                        ["item"] = "|cffa335ee|Hitem:16964::::::::60:::::::|h[Gauntlets of Wrath]|h|r",
                        ["dkpChangeText"] = "|cffE71D36-6 DKP|r",
                        ["date"] = "03/26/20",
                        ["dkpChange"] = -6,
                        ["raid"] = "Blackwing Lair",
                        ["serverTime"] = 1585197436,
                        ["officer"] = "|cffFF7D0ANeekio|r",
                    },
                    [1585800013] = {
                        ["names"] = "|cffFF7D0ABlanebites|r",
                        ["id"] = 1585800013,
                        ["text"] = "|cffE71D36Item Win - |rNone",
                        ["time"] = "09:00:13 PM",
                        ["reason"] = "Item Win",
                        ["datetime"] = 1585800013,
                        ["dkpChangeText"] = "|cffE71D36-8 DKP|r",
                        ["item"] = "None",
                        ["date"] = "04/01/20",
                        ["dkpChange"] = -8,
                        ["raid"] = "Blackwing Lair",
                        ["serverTime"] = 1585800013,
                        ["officer"] = "|cffC79C6ESnaildaddy|r",
                    },
                    [1585197193] = {
                        ["names"] = "|cffFF7D0AInsub|r, |cffFF7D0AInebriated|r, |cffABD473Primera|r, |cffABD473Wahcha|r, |cffABD473Stellâ|r, |cffABD473Ugro|r, |cff40C7EBEolith|r, |cff40C7EBSuprarz|r, |cff40C7EBOxford|r, |cff40C7EBZukohere|r, |cff40C7EBBugaboom|r, |cff40C7EBChipgizmo|r, |cffF58CBALawduk|r, |cffF58CBAPapasquach|r, |cffF58CBAGneissguy|r, |cffF58CBAOnes|r, |cffFFFFFFBubblè|r, |cffFFFFFFMongous|r, |cffFFFFFFEvangelina|r, |cffFFFFFFHoneypot|r, |cffFFFFFFFawntine|r, |cffFFFFFFRez|r, |cffFFF569Inigma|r, |cffFFF569Tuggernuts|r, |cffFFF569Mystile|r, |cffFFF569Mujinn|r, |cff8787EDLittledubs|r, |cff8787EDThepurple|r, |cff8787EDSoftfondle|r, |cff8787EDVarix|r, |cff8787EDSparklenips|r, |cffC79C6ESnaildaddy|r, |cffC79C6ERetkin|r, |cffC79C6ETokentoken|r, |cffC79C6ELuckerdawg|r, |cffC79C6EShinynickels|r, |cffC79C6EGalagus|r, |cffC79C6ENightshelf|r, |cffC79C6EJeffry|r, |cffC79C6EReina|r",
                        ["id"] = 1585197193,
                        ["text"] = "|cff22bb33Blackwing Lair - Broodlord Lashlayer|r",
                        ["bossKill"] = "Broodlord Lashlayer",
                        ["reason"] = "Boss Kill",
                        ["datetime"] = 1585197194,
                        ["time"] = "09:33:14 PM",
                        ["dkpChangeText"] = "|cff22bb3310 DKP|r",
                        ["date"] = "03/25/20",
                        ["dkpChange"] = 10,
                        ["raid"] = "Blackwing Lair",
                        ["serverTime"] = 1585197193,
                        ["officer"] = "|cff8787EDSparklenips|r",
                    },
                    [1585173486] = {
                        ["id"] = 1585173486,
                        ["text"] = "|cff22bb33Blackwing Lair - Completion Bonus|r",
                        ["names"] = "|cffC79C6EKalita|r, |cffC79C6ELuckerdawg|r, |cffC79C6EHotdogbroth|r",
                        ["reason"] = "Completion Bonus",
                        ["datetime"] = 1585173487,
                        ["dkpChangeText"] = "|cff22bb33100 DKP|r",
                        ["time"] = "02:58:07 PM",
                        ["date"] = "03/25/20",
                        ["dkpChange"] = 100,
                        ["raid"] = "Blackwing Lair",
                        ["serverTime"] = 1585173486,
                        ["officer"] = "|cff8787EDSparklenips|r",
                    },
                    [1585184965] = {
                        ["id"] = 1585184965,
                        ["text"] = "|cff22bb33Blackwing Lair - Benched|r",
                        ["reason"] = "Benched",
                        ["names"] = "|cffFF7D0ABlanebites|r, |cff40C7EBFlatulent|r, |cffF58CBACaptnutsack|r, |cffF58CBADolamroth|r, |cffFFF569Iszell|r, |cffFFF569Sicarrio|r, |cff8787EDIhurricanel|r, |cffC79C6EMadmartagen|r",
                        ["datetime"] = 1585184966,
                        ["time"] = "09:09:26 PM",
                        ["dkpChangeText"] = "|cff22bb3350 DKP|r",
                        ["date"] = "03/25/20",
                        ["dkpChange"] = 50,
                        ["raid"] = "Blackwing Lair",
                        ["serverTime"] = 1585184965,
                        ["officer"] = "|cffFF7D0ANeekio|r",
                    },
                    [1585689622] = {
                        ["id"] = 1585689622,
                        ["text"] = "|cff22bb33Molten Core - On Time Bonus|r",
                        ["reason"] = "On Time Bonus",
                        ["names"] = "|cff40C7EBUrkh|r",
                        ["datetime"] = 1585689623,
                        ["dkpChangeText"] = "|cff22bb3310 DKP|r",
                        ["time"] = "02:20:23 PM",
                        ["date"] = "03/31/20",
                        ["dkpChange"] = 10,
                        ["raid"] = "Molten Core",
                        ["serverTime"] = 1585689622,
                        ["officer"] = "|cff8787EDSparklenips|r",
                    },
                    [1585188810] = {
                        ["reason"] = "Item Win",
                        ["id"] = 1585188810,
                        ["text"] = "|cffE71D36Item Win - |r|cffa335ee|Hitem:19372::::::::60:::::::|h[Helm of Endless Rage]|h|r",
                        ["item"] = "|cffa335ee|Hitem:19372::::::::60:::::::|h[Helm of Endless Rage]|h|r",
                        ["names"] = "|cffC79C6EOragan|r",
                        ["datetime"] = 1585188811,
                        ["dkpChangeText"] = "|cffE71D36-48 DKP|r",
                        ["time"] = "10:13:31 PM",
                        ["date"] = "03/25/20",
                        ["dkpChange"] = -48,
                        ["raid"] = "Blackwing Lair",
                        ["serverTime"] = 1585188810,
                        ["officer"] = "|cffFF7D0ANeekio|r",
                    },
                    [1585202580] = {
                        ["names"] = "|cffC79C6EGalagus|r",
                        ["id"] = 1585202580,
                        ["text"] = "|cffE71D36Item Win - |rNone",
                        ["dkpChangeText"] = "|cffE71D36-90 DKP|r",
                        ["reason"] = "Item Win",
                        ["datetime"] = 1585202582,
                        ["item"] = "None",
                        ["time"] = "11:03:02 PM",
                        ["date"] = "03/25/20",
                        ["dkpChange"] = -90,
                        ["raid"] = "Blackwing Lair",
                        ["serverTime"] = 1585202580,
                        ["officer"] = "|cff8787EDSparklenips|r",
                    },
                    [1585173518] = {
                        ["id"] = 1585173518,
                        ["text"] = "|cff22bb33Blackwing Lair - Completion Bonus|r",
                        ["names"] = "|cffC79C6EOragan|r",
                        ["reason"] = "Completion Bonus",
                        ["datetime"] = 1585173519,
                        ["dkpChangeText"] = "|cff22bb3375 DKP|r",
                        ["time"] = "02:58:39 PM",
                        ["date"] = "03/25/20",
                        ["dkpChange"] = 75,
                        ["raid"] = "Blackwing Lair",
                        ["serverTime"] = 1585173518,
                        ["officer"] = "|cff8787EDSparklenips|r",
                    },
                    [1585195966] = {
                        ["names"] = "|cffC79C6ENightshelf|r",
                        ["id"] = 1585195966,
                        ["text"] = "|cffE71D36Item Win - |rNone",
                        ["dkpChangeText"] = "|cffE71D36-12 DKP|r",
                        ["reason"] = "Item Win",
                        ["datetime"] = 1585195967,
                        ["item"] = "None",
                        ["time"] = "09:12:47 PM",
                        ["date"] = "03/25/20",
                        ["dkpChange"] = -12,
                        ["raid"] = "Blackwing Lair",
                        ["serverTime"] = 1585195966,
                        ["officer"] = "|cff8787EDSparklenips|r",
                    },
                    [1585196484] = {
                        ["reason"] = "Item Win",
                        ["id"] = 1585196484,
                        ["text"] = "|cffE71D36Item Win - |r|cffa335ee|Hitem:19347::::::::60:::::::|h[Claw of Chromaggus]|h|r",
                        ["time"] = "12:21:25 AM",
                        ["names"] = "|cff40C7EBJakemeoff|r",
                        ["datetime"] = 1585196485,
                        ["item"] = "|cffa335ee|Hitem:19347::::::::60:::::::|h[Claw of Chromaggus]|h|r",
                        ["dkpChangeText"] = "|cffE71D36-90 DKP|r",
                        ["date"] = "03/26/20",
                        ["dkpChange"] = -90,
                        ["raid"] = "Blackwing Lair",
                        ["serverTime"] = 1585196484,
                        ["officer"] = "|cffFF7D0ANeekio|r",
                    },
                    [1585202608] = {
                        ["names"] = "|cff40C7EBZukohere|r",
                        ["id"] = 1585202608,
                        ["text"] = "|cffE71D36Item Win - |rNone",
                        ["dkpChangeText"] = "|cffE71D36-16 DKP|r",
                        ["reason"] = "Item Win",
                        ["datetime"] = 1585202610,
                        ["item"] = "None",
                        ["time"] = "11:03:30 PM",
                        ["date"] = "03/25/20",
                        ["dkpChange"] = -16,
                        ["raid"] = "Blackwing Lair",
                        ["serverTime"] = 1585202608,
                        ["officer"] = "|cff8787EDSparklenips|r",
                    },
                    [1585793706] = {
                        ["names"] = "|cffFFF569Slipperyjohn|r",
                        ["id"] = 1585793706,
                        ["text"] = "|cffE71D36Item Win - |rNone",
                        ["item"] = "None",
                        ["reason"] = "Item Win",
                        ["datetime"] = 1585793708,
                        ["dkpChangeText"] = "|cffE71D36-98 DKP|r",
                        ["time"] = "07:15:08 PM",
                        ["date"] = "04/01/20",
                        ["dkpChange"] = -98,
                        ["raid"] = "Blackwing Lair",
                        ["serverTime"] = 1585793706,
                        ["officer"] = "|cff8787EDSparklenips|r",
                    },
                    [1585191655] = {
                        ["names"] = "|cffFFF569Mozzarella|r",
                        ["id"] = 1585191655,
                        ["text"] = "|cffE71D36Item Win - |r|cffa335ee|Hitem:16906::::::::60:::::::|h[Bloodfang Boots]|h|r",
                        ["dkpChangeText"] = "|cffE71D36-14 DKP|r",
                        ["reason"] = "Item Win",
                        ["datetime"] = 1585191656,
                        ["time"] = "11:00:56 PM",
                        ["item"] = "|cffa335ee|Hitem:16906::::::::60:::::::|h[Bloodfang Boots]|h|r",
                        ["date"] = "03/25/20",
                        ["dkpChange"] = -14,
                        ["raid"] = "Blackwing Lair",
                        ["serverTime"] = 1585191655,
                        ["officer"] = "|cffFF7D0ANeekio|r",
                    },
                    [1585800603] = {
                        ["names"] = "|cff8787EDTalisker|r",
                        ["id"] = 1585800603,
                        ["text"] = "|cffE71D36Item Win - |rNone",
                        ["time"] = "09:10:03 PM",
                        ["reason"] = "Item Win",
                        ["datetime"] = 1585800603,
                        ["dkpChangeText"] = "|cffE71D36-20 DKP|r",
                        ["item"] = "None",
                        ["date"] = "04/01/20",
                        ["dkpChange"] = -20,
                        ["raid"] = "Blackwing Lair",
                        ["serverTime"] = 1585800603,
                        ["officer"] = "|cffC79C6ESnaildaddy|r",
                    },
                    [1585798822] = {
                        ["reason"] = "Item Win",
                        ["id"] = 1585798822,
                        ["text"] = "|cffE71D36Item Win - |rNone",
                        ["dkpChangeText"] = "|cffE71D36-17 DKP|r",
                        ["names"] = "|cff8787EDIhurricanel|r",
                        ["datetime"] = 1585798822,
                        ["item"] = "None",
                        ["time"] = "08:40:22 PM",
                        ["date"] = "04/01/20",
                        ["dkpChange"] = -17,
                        ["raid"] = "Blackwing Lair",
                        ["serverTime"] = 1585798822,
                        ["officer"] = "|cffC79C6ESnaildaddy|r",
                    },
                    [1585800356] = {
                        ["names"] = "|cffC79C6EQew|r",
                        ["id"] = 1585800356,
                        ["text"] = "|cffE71D36Item Win - |rNone",
                        ["item"] = "None",
                        ["reason"] = "Item Win",
                        ["datetime"] = 1585800358,
                        ["dkpChangeText"] = "|cffE71D36-16 DKP|r",
                        ["time"] = "09:05:58 PM",
                        ["date"] = "04/01/20",
                        ["dkpChange"] = -16,
                        ["raid"] = "Blackwing Lair",
                        ["serverTime"] = 1585800356,
                        ["officer"] = "|cff8787EDSparklenips|r",
                    },
                    [1585173339] = {
                        ["id"] = 1585173339,
                        ["text"] = "|cff22bb33Blackwing Lair - Completion Bonus|r",
                        ["names"] = "|cff8787EDRaspütin|r",
                        ["reason"] = "Completion Bonus",
                        ["datetime"] = 1585173340,
                        ["dkpChangeText"] = "|cff22bb3350 DKP|r",
                        ["time"] = "02:55:40 PM",
                        ["date"] = "03/25/20",
                        ["dkpChange"] = 50,
                        ["raid"] = "Blackwing Lair",
                        ["serverTime"] = 1585173339,
                        ["officer"] = "|cff8787EDSparklenips|r",
                    },
                    [1585173108] = {
                        ["id"] = 1585173108,
                        ["text"] = "|cff22bb33Blackwing Lair - Completion Bonus|r",
                        ["names"] = "|cffFFFFFFBubblè|r",
                        ["reason"] = "Completion Bonus",
                        ["datetime"] = 1585173109,
                        ["dkpChangeText"] = "|cff22bb3380 DKP|r",
                        ["time"] = "02:51:49 PM",
                        ["date"] = "03/25/20",
                        ["dkpChange"] = 80,
                        ["raid"] = "Blackwing Lair",
                        ["serverTime"] = 1585173108,
                        ["officer"] = "|cff8787EDSparklenips|r",
                    },
                    [1585173622] = {
                        ["id"] = 1585173622,
                        ["text"] = "|cff22bb33Blackwing Lair - Completion Bonus|r",
                        ["names"] = "|cff40C7EBStitchess|r",
                        ["reason"] = "Completion Bonus",
                        ["datetime"] = 1585173623,
                        ["dkpChangeText"] = "|cff22bb33100 DKP|r",
                        ["time"] = "03:00:23 PM",
                        ["date"] = "03/25/20",
                        ["dkpChange"] = 100,
                        ["raid"] = "Blackwing Lair",
                        ["serverTime"] = 1585173622,
                        ["officer"] = "|cff8787EDSparklenips|r",
                    },
                    [1585173116] = {
                        ["id"] = 1585173116,
                        ["text"] = "|cff22bb33Blackwing Lair - Completion Bonus|r",
                        ["names"] = "|cffFFFFFFFawntine|r",
                        ["reason"] = "Completion Bonus",
                        ["datetime"] = 1585173117,
                        ["dkpChangeText"] = "|cff22bb3390 DKP|r",
                        ["time"] = "02:51:57 PM",
                        ["date"] = "03/25/20",
                        ["dkpChange"] = 90,
                        ["raid"] = "Blackwing Lair",
                        ["serverTime"] = 1585173116,
                        ["officer"] = "|cff8787EDSparklenips|r",
                    },
                    [1585798894] = {
                        ["reason"] = "Item Win",
                        ["id"] = 1585798894,
                        ["text"] = "|cffE71D36Item Win - |rNone",
                        ["dkpChangeText"] = "|cffE71D36-2 DKP|r",
                        ["names"] = "|cffFF7D0AKalijah|r",
                        ["datetime"] = 1585798894,
                        ["item"] = "None",
                        ["time"] = "08:41:34 PM",
                        ["date"] = "04/01/20",
                        ["dkpChange"] = -2,
                        ["raid"] = "Blackwing Lair",
                        ["serverTime"] = 1585798894,
                        ["officer"] = "|cffC79C6ESnaildaddy|r",
                    },
                    [1585201688] = {
                        ["names"] = "|cffFFF569Mujinn|r",
                        ["id"] = 1585201688,
                        ["text"] = "|cffE71D36Item Win - |rNone",
                        ["dkpChangeText"] = "|cffE71D36-17 DKP|r",
                        ["reason"] = "Item Win",
                        ["datetime"] = 1585201690,
                        ["item"] = "None",
                        ["time"] = "10:48:10 PM",
                        ["date"] = "03/25/20",
                        ["dkpChange"] = -17,
                        ["raid"] = "Blackwing Lair",
                        ["serverTime"] = 1585201688,
                        ["officer"] = "|cff8787EDSparklenips|r",
                    },
                    [1585173391] = {
                        ["id"] = 1585173391,
                        ["text"] = "|cff22bb33Blackwing Lair - Completion Bonus|r",
                        ["names"] = "|cffFF7D0AMorphintyme|r",
                        ["reason"] = "Completion Bonus",
                        ["datetime"] = 1585173392,
                        ["dkpChangeText"] = "|cff22bb3385 DKP|r",
                        ["time"] = "02:56:32 PM",
                        ["date"] = "03/25/20",
                        ["dkpChange"] = 85,
                        ["raid"] = "Blackwing Lair",
                        ["serverTime"] = 1585173391,
                        ["officer"] = "|cff8787EDSparklenips|r",
                    },
                    [1585200935] = {
                        ["names"] = "|cff40C7EBSuprarz|r",
                        ["id"] = 1585200935,
                        ["text"] = "|cffE71D36Item Win - |rNone",
                        ["dkpChangeText"] = "|cffE71D36-10 DKP|r",
                        ["reason"] = "Item Win",
                        ["datetime"] = 1585200937,
                        ["item"] = "None",
                        ["time"] = "10:35:37 PM",
                        ["date"] = "03/25/20",
                        ["dkpChange"] = -10,
                        ["raid"] = "Blackwing Lair",
                        ["serverTime"] = 1585200935,
                        ["officer"] = "|cff8787EDSparklenips|r",
                    },
                    [1585797639] = {
                        ["id"] = 1585797639,
                        ["text"] = "|cffE71D36Other - Correction|r",
                        ["reason"] = "Other",
                        ["names"] = "|cffF58CBAOnes|r, |cffFFFFFFRez|r, |cffFFFFFFFawntine|r, |cffFFF569Mystile|r",
                        ["datetime"] = 1585797640,
                        ["time"] = "11:20:40 PM",
                        ["dkpChangeText"] = "|cffE71D36-20 DKP|r",
                        ["date"] = "04/01/20",
                        ["dkpChange"] = -20,
                        ["raid"] = "Blackwing Lair",
                        ["serverTime"] = 1585797639,
                        ["officer"] = "|cffFF7D0ANeekio|r",
                    },
                    [1585173411] = {
                        ["id"] = 1585173411,
                        ["text"] = "|cff22bb33Blackwing Lair - Completion Bonus|r",
                        ["names"] = "|cffABD473Athico|r",
                        ["reason"] = "Completion Bonus",
                        ["datetime"] = 1585173412,
                        ["dkpChangeText"] = "|cff22bb33100 DKP|r",
                        ["time"] = "02:56:52 PM",
                        ["date"] = "03/25/20",
                        ["dkpChange"] = 100,
                        ["raid"] = "Blackwing Lair",
                        ["serverTime"] = 1585173411,
                        ["officer"] = "|cff8787EDSparklenips|r",
                    },
                    [1585195596] = {
                        ["names"] = "|cff8787EDVarix|r",
                        ["id"] = 1585195596,
                        ["text"] = "|cffE71D36Item Win - |rNone",
                        ["dkpChangeText"] = "|cffE71D36-12 DKP|r",
                        ["reason"] = "Item Win",
                        ["datetime"] = 1585195597,
                        ["item"] = "None",
                        ["time"] = "09:06:37 PM",
                        ["date"] = "03/25/20",
                        ["dkpChange"] = -12,
                        ["raid"] = "Blackwing Lair",
                        ["serverTime"] = 1585195596,
                        ["officer"] = "|cff8787EDSparklenips|r",
                    },
                    [1585806588] = {
                        ["id"] = 1585806588,
                        ["text"] = "|cff22bb33Blackwing Lair - Completion Bonus|r",
                        ["names"] = "|cffFF7D0AInsub|r, |cffFF7D0ABlanebites|r, |cffFF7D0AKalijah|r, |cffFF7D0AInebriated|r, |cffABD473Ugro|r, |cffABD473Primera|r, |cffABD473Stellâ|r, |cffABD473Wahcha|r, |cff40C7EBZukohere|r, |cff40C7EBSuprarz|r, |cff40C7EBBugaboom|r, |cff40C7EBOxford|r, |cff40C7EBChipgizmo|r, |cffF58CBAOnes|r, |cffF58CBALawduk|r, |cffF58CBADolamroth|r, |cffF58CBAGneissguy|r, |cffFFFFFFMongous|r, |cffFFFFFFHoneypot|r, |cffFFFFFFRez|r, |cffFFFFFFFawntine|r, |cffFFF569Trickster|r, |cffFFF569Aku|r, |cffFFF569Mystile|r, |cffFFF569Tuggernuts|r, |cffFFF569Inigma|r, |cff8787EDTalisker|r, |cff8787EDSoftfondle|r, |cff8787EDVarix|r, |cff8787EDIhurricanel|r, |cff8787EDLittledubs|r, |cff8787EDThepurple|r, |cffC79C6EGalagus|r, |cffC79C6EReina|r, |cffC79C6ETokentoken|r, |cffC79C6ENightshelf|r, |cffC79C6ESnaildaddy|r, |cffC79C6ELuckerdawg|r, |cffC79C6EJeffry|r, |cffC79C6ERetkin|r",
                        ["reason"] = "Completion Bonus",
                        ["datetime"] = 1585806588,
                        ["time"] = "10:49:48 PM",
                        ["dkpChangeText"] = "|cff22bb3310 DKP|r",
                        ["date"] = "04/01/20",
                        ["dkpChange"] = 10,
                        ["raid"] = "Blackwing Lair",
                        ["serverTime"] = 1585806588,
                        ["officer"] = "|cffC79C6ESnaildaddy|r",
                    },
                    [1585806086] = {
                        ["names"] = "|cffFF7D0ABlanebites|r",
                        ["id"] = 1585806086,
                        ["text"] = "|cffE71D36Item Win - |rNone",
                        ["time"] = "10:41:26 PM",
                        ["reason"] = "Item Win",
                        ["datetime"] = 1585806086,
                        ["dkpChangeText"] = "|cffE71D36-9 DKP|r",
                        ["item"] = "None",
                        ["date"] = "04/01/20",
                        ["dkpChange"] = -9,
                        ["raid"] = "Blackwing Lair",
                        ["serverTime"] = 1585806086,
                        ["officer"] = "|cffC79C6ESnaildaddy|r",
                    },
                    [1585689567] = {
                        ["id"] = 1585689567,
                        ["text"] = "|cff22bb33Molten Core - On Time Bonus|r",
                        ["reason"] = "On Time Bonus",
                        ["names"] = "|cff40C7EBUrkh|r",
                        ["datetime"] = 1585689568,
                        ["dkpChangeText"] = "|cff22bb3310 DKP|r",
                        ["time"] = "02:19:28 PM",
                        ["date"] = "03/31/20",
                        ["dkpChange"] = 10,
                        ["raid"] = "Molten Core",
                        ["serverTime"] = 1585689567,
                        ["officer"] = "|cff8787EDSparklenips|r",
                    },
                    [1585800237] = {
                        ["names"] = "|cffFFF569Mariku|r",
                        ["id"] = 1585800237,
                        ["text"] = "|cffE71D36Item Win - |rNone",
                        ["item"] = "None",
                        ["reason"] = "Item Win",
                        ["datetime"] = 1585800239,
                        ["dkpChangeText"] = "|cffE71D36-116 DKP|r",
                        ["time"] = "09:03:59 PM",
                        ["date"] = "04/01/20",
                        ["dkpChange"] = -116,
                        ["raid"] = "Blackwing Lair",
                        ["serverTime"] = 1585800237,
                        ["officer"] = "|cff8787EDSparklenips|r",
                    },
                    [1585173459] = {
                        ["id"] = 1585173459,
                        ["text"] = "|cff22bb33Blackwing Lair - Completion Bonus|r",
                        ["names"] = "|cffFFFFFFBallour|r",
                        ["reason"] = "Completion Bonus",
                        ["datetime"] = 1585173460,
                        ["dkpChangeText"] = "|cff22bb3398 DKP|r",
                        ["time"] = "02:57:40 PM",
                        ["date"] = "03/25/20",
                        ["dkpChange"] = 98,
                        ["raid"] = "Blackwing Lair",
                        ["serverTime"] = 1585173459,
                        ["officer"] = "|cff8787EDSparklenips|r",
                    },
                    [1585188763] = {
                        ["reason"] = "Item Win",
                        ["id"] = 1585188763,
                        ["text"] = "|cffE71D36Item Win - |r|cffa335ee|Hitem:16960::::::::60:::::::|h[Waistband of Wrath]|h|r",
                        ["item"] = "|cffa335ee|Hitem:16960::::::::60:::::::|h[Waistband of Wrath]|h|r",
                        ["names"] = "|cffC79C6EOragan|r",
                        ["datetime"] = 1585188764,
                        ["dkpChangeText"] = "|cffE71D36-10 DKP|r",
                        ["time"] = "10:12:44 PM",
                        ["date"] = "03/25/20",
                        ["dkpChange"] = -10,
                        ["raid"] = "Blackwing Lair",
                        ["serverTime"] = 1585188763,
                        ["officer"] = "|cffFF7D0ANeekio|r",
                    },
                    [1585804843] = {
                        ["names"] = "|cffFF7D0AKalijah|r, |cffFF7D0AInebriated|r, |cffFF7D0ABlanebites|r, |cffFF7D0AInsub|r, |cffABD473Primera|r, |cffABD473Ugro|r, |cffABD473Wahcha|r, |cffABD473Stellâ|r, |cff40C7EBZukohere|r, |cff40C7EBSuprarz|r, |cff40C7EBBugaboom|r, |cff40C7EBOxford|r, |cff40C7EBChipgizmo|r, |cffF58CBAGneissguy|r, |cffF58CBADolamroth|r, |cffF58CBALawduk|r, |cffF58CBAOnes|r, |cffFFFFFFRez|r, |cffFFFFFFFawntine|r, |cffFFFFFFMongous|r, |cffFFFFFFHoneypot|r, |cffFFF569Aku|r, |cffFFF569Mystile|r, |cffFFF569Trickster|r, |cffFFF569Tuggernuts|r, |cffFFF569Inigma|r, |cff8787EDSoftfondle|r, |cff8787EDThepurple|r, |cff8787EDTalisker|r, |cff8787EDLittledubs|r, |cff8787EDIhurricanel|r, |cff8787EDVarix|r, |cffC79C6ELuckerdawg|r, |cffC79C6EGalagus|r, |cffC79C6EReina|r, |cffC79C6ESnaildaddy|r, |cffC79C6ENightshelf|r, |cffC79C6ETokentoken|r, |cffC79C6ERetkin|r, |cffC79C6EJeffry|r",
                        ["id"] = 1585804843,
                        ["text"] = "|cff22bb33Blackwing Lair - Chromaggus|r",
                        ["bossKill"] = "Chromaggus",
                        ["reason"] = "Boss Kill",
                        ["datetime"] = 1585804843,
                        ["time"] = "10:20:43 PM",
                        ["dkpChangeText"] = "|cff22bb3310 DKP|r",
                        ["date"] = "04/01/20",
                        ["dkpChange"] = 10,
                        ["raid"] = "Blackwing Lair",
                        ["serverTime"] = 1585804843,
                        ["officer"] = "|cffC79C6ESnaildaddy|r",
                    },
                    [1585173212] = {
                        ["id"] = 1585173212,
                        ["text"] = "|cff22bb33Blackwing Lair - Completion Bonus|r",
                        ["names"] = "|cffF58CBADolamroth|r",
                        ["reason"] = "Completion Bonus",
                        ["datetime"] = 1585173213,
                        ["dkpChangeText"] = "|cff22bb3383 DKP|r",
                        ["time"] = "02:53:33 PM",
                        ["date"] = "03/25/20",
                        ["dkpChange"] = 83,
                        ["raid"] = "Blackwing Lair",
                        ["serverTime"] = 1585173212,
                        ["officer"] = "|cff8787EDSparklenips|r",
                    },
                    [1585202541] = {
                        ["names"] = "|cffC79C6EGalagus|r",
                        ["id"] = 1585202541,
                        ["text"] = "|cff22bb33Blackwing Lair - Nefarian|r",
                        ["bossKill"] = "Nefarian",
                        ["reason"] = "Boss Kill",
                        ["datetime"] = 1585202543,
                        ["time"] = "11:02:23 PM",
                        ["dkpChangeText"] = "|cff22bb3310 DKP|r",
                        ["date"] = "03/25/20",
                        ["dkpChange"] = 10,
                        ["raid"] = "Blackwing Lair",
                        ["serverTime"] = 1585202541,
                        ["officer"] = "|cff8787EDSparklenips|r",
                    },
                    [1585196425] = {
                        ["reason"] = "Item Win",
                        ["id"] = 1585196425,
                        ["text"] = "|cffE71D36Item Win - |r|cffa335ee|Hitem:16953::::::::60:::::::|h[Judgement Spaulders]|h|r",
                        ["time"] = "12:20:26 AM",
                        ["names"] = "|cffF58CBAForerunner|r",
                        ["datetime"] = 1585196426,
                        ["item"] = "|cffa335ee|Hitem:16953::::::::60:::::::|h[Judgement Spaulders]|h|r",
                        ["dkpChangeText"] = "|cffE71D36-13 DKP|r",
                        ["date"] = "03/26/20",
                        ["dkpChange"] = -13,
                        ["raid"] = "Blackwing Lair",
                        ["serverTime"] = 1585196425,
                        ["officer"] = "|cffFF7D0ANeekio|r",
                    },
                    [1585173224] = {
                        ["id"] = 1585173224,
                        ["text"] = "|cff22bb33Blackwing Lair - Completion Bonus|r",
                        ["names"] = "|cffFFF569Mystile|r",
                        ["reason"] = "Completion Bonus",
                        ["datetime"] = 1585173225,
                        ["dkpChangeText"] = "|cff22bb3385 DKP|r",
                        ["time"] = "02:53:45 PM",
                        ["date"] = "03/25/20",
                        ["dkpChange"] = 85,
                        ["raid"] = "Blackwing Lair",
                        ["serverTime"] = 1585173224,
                        ["officer"] = "|cff8787EDSparklenips|r",
                    },
                    [1585799508] = {
                        ["names"] = "|cff40C7EBJakemeoff|r",
                        ["id"] = 1585799508,
                        ["text"] = "|cffE71D36Item Win - |rNone",
                        ["item"] = "None",
                        ["reason"] = "Item Win",
                        ["datetime"] = 1585799510,
                        ["dkpChangeText"] = "|cffE71D36-19 DKP|r",
                        ["time"] = "08:51:50 PM",
                        ["date"] = "04/01/20",
                        ["dkpChange"] = -19,
                        ["raid"] = "Blackwing Lair",
                        ["serverTime"] = 1585799508,
                        ["officer"] = "|cff8787EDSparklenips|r",
                    },
                    [1585173240] = {
                        ["id"] = 1585173240,
                        ["text"] = "|cff22bb33Blackwing Lair - Completion Bonus|r",
                        ["names"] = "|cffFFF569Aku|r, |cffFFF569Mujinn|r, |cffFFF569Tuggernuts|r",
                        ["reason"] = "Completion Bonus",
                        ["datetime"] = 1585173241,
                        ["dkpChangeText"] = "|cff22bb33100 DKP|r",
                        ["time"] = "02:54:01 PM",
                        ["date"] = "03/25/20",
                        ["dkpChange"] = 100,
                        ["raid"] = "Blackwing Lair",
                        ["serverTime"] = 1585173240,
                        ["officer"] = "|cff8787EDSparklenips|r",
                    },
                    [1585792129] = {
                        ["names"] = "|cffFF7D0ANeekio|r, |cffFF7D0AEmmyy|r, |cffFF7D0AMorphintyme|r, |cffABD473Athico|r, |cffABD473Bádtothebow|r, |cffABD473Ithgar|r, |cffABD473Bigbootyhoho|r, |cffABD473Cyskul|r, |cff40C7EBEolith|r, |cff40C7EBJakemeoff|r, |cff40C7EBVeriandra|r, |cff40C7EBFlatulent|r, |cff40C7EBStitchess|r, |cff40C7EBNeckbeardo|r, |cff40C7EBXyen|r, |cffF58CBAAdvanty|r, |cffF58CBAMaryjohanna|r, |cffF58CBAOdin|r, |cffF58CBAPapasquach|r, |cffFFFFFFBallour|r, |cffFFFFFFPuffhead|r, |cffFFFFFFPriesticuffs|r, |cffFFFFFFPhilonious|r, |cffFFFFFFTurbohealz|r, |cffFFF569Mcstaberson|r, |cffFFF569Mariku|r, |cffFFF569Slipperyjohn|r, |cffFFF569Sicarrio|r, |cff8787EDSparklenips|r, |cff8787EDLindo|r, |cff8787EDDalia|r, |cff8787EDRaspütin|r, |cff8787EDEdgyboi|r, |cffC79C6EKalita|r, |cffC79C6EQew|r, |cffC79C6EGirlslayer|r, |cffC79C6EGoobimus|r, |cffC79C6EOragan|r, |cffC79C6EShinynickels|r, |cffC79C6EHotdogbroth|r",
                        ["id"] = 1585792129,
                        ["text"] = "|cff22bb33Blackwing Lair - Vaelastrasz the Corrupt|r",
                        ["bossKill"] = "Vaelastrasz the Corrupt",
                        ["reason"] = "Boss Kill",
                        ["datetime"] = 1585792131,
                        ["time"] = "06:48:51 PM",
                        ["dkpChangeText"] = "|cff22bb3310 DKP|r",
                        ["date"] = "04/01/20",
                        ["dkpChange"] = 10,
                        ["raid"] = "Blackwing Lair",
                        ["serverTime"] = 1585792129,
                        ["officer"] = "|cff8787EDSparklenips|r",
                    },
                    [1585198493] = {
                        ["names"] = "|cffABD473Stellâ|r",
                        ["id"] = 1585198493,
                        ["text"] = "|cffE71D36Item Win - |rNone",
                        ["dkpChangeText"] = "|cffE71D36-90 DKP|r",
                        ["reason"] = "Item Win",
                        ["datetime"] = 1585198495,
                        ["item"] = "None",
                        ["time"] = "09:54:55 PM",
                        ["date"] = "03/25/20",
                        ["dkpChange"] = -90,
                        ["raid"] = "Blackwing Lair",
                        ["serverTime"] = 1585198493,
                        ["officer"] = "|cff8787EDSparklenips|r",
                    },
                    [1585198246] = {
                        ["names"] = "|cffFF7D0AInebriated|r",
                        ["id"] = 1585198246,
                        ["text"] = "|cffE71D36Item Win - |rNone",
                        ["dkpChangeText"] = "|cffE71D36-14 DKP|r",
                        ["reason"] = "Item Win",
                        ["datetime"] = 1585198247,
                        ["item"] = "None",
                        ["time"] = "09:50:47 PM",
                        ["date"] = "03/25/20",
                        ["dkpChange"] = -14,
                        ["raid"] = "Blackwing Lair",
                        ["serverTime"] = 1585198246,
                        ["officer"] = "|cff8787EDSparklenips|r",
                    },
                    [1585173264] = {
                        ["id"] = 1585173264,
                        ["text"] = "|cff22bb33Blackwing Lair - Completion Bonus|r",
                        ["names"] = "|cffFFF569Inigma|r",
                        ["reason"] = "Completion Bonus",
                        ["datetime"] = 1585173265,
                        ["dkpChangeText"] = "|cff22bb3350 DKP|r",
                        ["time"] = "02:54:25 PM",
                        ["date"] = "03/25/20",
                        ["dkpChange"] = 50,
                        ["raid"] = "Blackwing Lair",
                        ["serverTime"] = 1585173264,
                        ["officer"] = "|cff8787EDSparklenips|r",
                    },
                    [1585799552] = {
                        ["names"] = "|cffFF7D0AEmmyy|r",
                        ["id"] = 1585799552,
                        ["text"] = "|cffE71D36Item Win - |rNone",
                        ["item"] = "None",
                        ["reason"] = "Item Win",
                        ["datetime"] = 1585799554,
                        ["dkpChangeText"] = "|cffE71D36-20 DKP|r",
                        ["time"] = "08:52:34 PM",
                        ["date"] = "04/01/20",
                        ["dkpChange"] = -20,
                        ["raid"] = "Blackwing Lair",
                        ["serverTime"] = 1585799552,
                        ["officer"] = "|cff8787EDSparklenips|r",
                    },
                    [1585198521] = {
                        ["names"] = "|cff8787EDThepurple|r",
                        ["id"] = 1585198521,
                        ["text"] = "|cffE71D36Item Win - |rNone",
                        ["dkpChangeText"] = "|cffE71D36-53 DKP|r",
                        ["reason"] = "Item Win",
                        ["datetime"] = 1585198523,
                        ["item"] = "None",
                        ["time"] = "09:55:23 PM",
                        ["date"] = "03/25/20",
                        ["dkpChange"] = -53,
                        ["raid"] = "Blackwing Lair",
                        ["serverTime"] = 1585198521,
                        ["officer"] = "|cff8787EDSparklenips|r",
                    },
                    [1585173045] = {
                        ["id"] = 1585173045,
                        ["text"] = "|cff22bb33Blackwing Lair - Completion Bonus|r",
                        ["names"] = "|cffC79C6EReina|r",
                        ["reason"] = "Completion Bonus",
                        ["datetime"] = 1585173046,
                        ["dkpChangeText"] = "|cff22bb3365 DKP|r",
                        ["time"] = "02:50:46 PM",
                        ["date"] = "03/25/20",
                        ["dkpChange"] = 65,
                        ["raid"] = "Blackwing Lair",
                        ["serverTime"] = 1585173045,
                        ["officer"] = "|cff8787EDSparklenips|r",
                    },
                    [1585689679] = {
                        ["id"] = 1585689679,
                        ["text"] = "|cff22bb33Molten Core - On Time Bonus|r",
                        ["reason"] = "On Time Bonus",
                        ["names"] = "|cff40C7EBUrkh|r",
                        ["datetime"] = 1585689680,
                        ["dkpChangeText"] = "|cff22bb3310 DKP|r",
                        ["time"] = "02:21:20 PM",
                        ["date"] = "03/31/20",
                        ["dkpChange"] = 10,
                        ["raid"] = "Molten Core",
                        ["serverTime"] = 1585689679,
                        ["officer"] = "|cff8787EDSparklenips|r",
                    },
                    [1585197537] = {
                        ["reason"] = "Item Win",
                        ["id"] = 1585197537,
                        ["text"] = "|cffE71D36Item Win - |r|cffa335ee|Hitem:19375::::::::60:::::::|h[Mish'undare, Circlet of the Mind Flayer]|h|r",
                        ["time"] = "12:38:58 AM",
                        ["names"] = "|cffF58CBAAdvanty|r",
                        ["datetime"] = 1585197538,
                        ["item"] = "|cffa335ee|Hitem:19375::::::::60:::::::|h[Mish'undare, Circlet of the Mind Flayer]|h|r",
                        ["dkpChangeText"] = "|cffE71D36-95 DKP|r",
                        ["date"] = "03/26/20",
                        ["dkpChange"] = -95,
                        ["raid"] = "Blackwing Lair",
                        ["serverTime"] = 1585197537,
                        ["officer"] = "|cffFF7D0ANeekio|r",
                    },
                    [1585195505] = {
                        ["names"] = "|cffF58CBALawduk|r",
                        ["id"] = 1585195505,
                        ["text"] = "|cffE71D36Item Win - |rNone",
                        ["dkpChangeText"] = "|cffE71D36-12 DKP|r",
                        ["reason"] = "Item Win",
                        ["datetime"] = 1585195507,
                        ["item"] = "None",
                        ["time"] = "09:05:07 PM",
                        ["date"] = "03/25/20",
                        ["dkpChange"] = -12,
                        ["raid"] = "Blackwing Lair",
                        ["serverTime"] = 1585195505,
                        ["officer"] = "|cff8787EDSparklenips|r",
                    },
                    [1585689448] = {
                        ["id"] = 1585689448,
                        ["text"] = "|cff22bb33Molten Core - On Time Bonus|r",
                        ["reason"] = "On Time Bonus",
                        ["names"] = "|cff40C7EBUrkh|r",
                        ["datetime"] = 1585689449,
                        ["dkpChangeText"] = "|cff22bb3310 DKP|r",
                        ["time"] = "02:17:29 PM",
                        ["date"] = "03/31/20",
                        ["dkpChange"] = 10,
                        ["raid"] = "Molten Core",
                        ["serverTime"] = 1585689448,
                        ["officer"] = "|cff8787EDSparklenips|r",
                    },
                    [1585173332] = {
                        ["id"] = 1585173332,
                        ["text"] = "|cff22bb33Blackwing Lair - Completion Bonus|r",
                        ["names"] = "|cff8787EDLindo|r",
                        ["reason"] = "Completion Bonus",
                        ["datetime"] = 1585173333,
                        ["dkpChangeText"] = "|cff22bb3375 DKP|r",
                        ["time"] = "02:55:33 PM",
                        ["date"] = "03/25/20",
                        ["dkpChange"] = 75,
                        ["raid"] = "Blackwing Lair",
                        ["serverTime"] = 1585173332,
                        ["officer"] = "|cff8787EDSparklenips|r",
                    },
                    [1585173077] = {
                        ["id"] = 1585173077,
                        ["text"] = "|cff22bb33Blackwing Lair - Completion Bonus|r",
                        ["names"] = "|cffC79C6ENightshelf|r",
                        ["reason"] = "Completion Bonus",
                        ["datetime"] = 1585173078,
                        ["dkpChangeText"] = "|cff22bb3385 DKP|r",
                        ["time"] = "02:51:18 PM",
                        ["date"] = "03/25/20",
                        ["dkpChange"] = 85,
                        ["raid"] = "Blackwing Lair",
                        ["serverTime"] = 1585173077,
                        ["officer"] = "|cff8787EDSparklenips|r",
                    },
                    [1585173299] = {
                        ["id"] = 1585173299,
                        ["text"] = "|cff22bb33Blackwing Lair - Completion Bonus|r",
                        ["names"] = "|cffABD473Wahcha|r",
                        ["reason"] = "Completion Bonus",
                        ["datetime"] = 1585173300,
                        ["dkpChangeText"] = "|cff22bb33100 DKP|r",
                        ["time"] = "02:55:00 PM",
                        ["date"] = "03/25/20",
                        ["dkpChange"] = 100,
                        ["raid"] = "Blackwing Lair",
                        ["serverTime"] = 1585173299,
                        ["officer"] = "|cff8787EDSparklenips|r",
                    },
                    [1585173348] = {
                        ["id"] = 1585173348,
                        ["text"] = "|cff22bb33Blackwing Lair - Completion Bonus|r",
                        ["names"] = "|cff8787EDSoftfondle|r",
                        ["reason"] = "Completion Bonus",
                        ["datetime"] = 1585173349,
                        ["dkpChangeText"] = "|cff22bb3355 DKP|r",
                        ["time"] = "02:55:49 PM",
                        ["date"] = "03/25/20",
                        ["dkpChange"] = 55,
                        ["raid"] = "Blackwing Lair",
                        ["serverTime"] = 1585173348,
                        ["officer"] = "|cff8787EDSparklenips|r",
                    },
                    [1585806038] = {
                        ["names"] = "|cffFF7D0AInsub|r, |cffFF7D0AKalijah|r, |cffFF7D0ABlanebites|r, |cffFF7D0AInebriated|r, |cffABD473Wahcha|r, |cffABD473Stellâ|r, |cffABD473Primera|r, |cffABD473Ugro|r, |cff40C7EBZukohere|r, |cff40C7EBOxford|r, |cff40C7EBSuprarz|r, |cff40C7EBBugaboom|r, |cff40C7EBChipgizmo|r, |cffF58CBAOnes|r, |cffF58CBADolamroth|r, |cffF58CBAGneissguy|r, |cffF58CBALawduk|r, |cffFFFFFFRez|r, |cffFFFFFFMongous|r, |cffFFFFFFHoneypot|r, |cffFFFFFFFawntine|r, |cffFFF569Aku|r, |cffFFF569Trickster|r, |cffFFF569Mystile|r, |cffFFF569Tuggernuts|r, |cffFFF569Inigma|r, |cff8787EDVarix|r, |cff8787EDIhurricanel|r, |cff8787EDSoftfondle|r, |cff8787EDTalisker|r, |cff8787EDThepurple|r, |cff8787EDLittledubs|r, |cffC79C6ESnaildaddy|r, |cffC79C6ETokentoken|r, |cffC79C6EGalagus|r, |cffC79C6ERetkin|r, |cffC79C6ELuckerdawg|r, |cffC79C6ENightshelf|r, |cffC79C6EJeffry|r, |cffC79C6EReina|r",
                        ["id"] = 1585806038,
                        ["text"] = "|cff22bb33Blackwing Lair - Nefarian|r",
                        ["bossKill"] = "Nefarian",
                        ["reason"] = "Boss Kill",
                        ["datetime"] = 1585806038,
                        ["time"] = "10:40:38 PM",
                        ["dkpChangeText"] = "|cff22bb3310 DKP|r",
                        ["date"] = "04/01/20",
                        ["dkpChange"] = 10,
                        ["raid"] = "Blackwing Lair",
                        ["serverTime"] = 1585806038,
                        ["officer"] = "|cffC79C6ESnaildaddy|r",
                    },
                    [1585805082] = {
                        ["names"] = "|cffFF7D0AKalijah|r",
                        ["id"] = 1585805082,
                        ["text"] = "|cffE71D36Item Win - |rNone",
                        ["time"] = "10:24:42 PM",
                        ["reason"] = "Item Win",
                        ["datetime"] = 1585805082,
                        ["dkpChangeText"] = "|cffE71D36-8 DKP|r",
                        ["item"] = "None",
                        ["date"] = "04/01/20",
                        ["dkpChange"] = -8,
                        ["raid"] = "Blackwing Lair",
                        ["serverTime"] = 1585805082,
                        ["officer"] = "|cffC79C6ESnaildaddy|r",
                    },
                    [1585795765] = {
                        ["names"] = "|cffFFF569Mcstaberson|r",
                        ["id"] = 1585795765,
                        ["text"] = "|cffE71D36Item Win - |rNone",
                        ["item"] = "None",
                        ["reason"] = "Item Win",
                        ["datetime"] = 1585795767,
                        ["dkpChangeText"] = "|cffE71D36-75 DKP|r",
                        ["time"] = "07:49:27 PM",
                        ["date"] = "04/01/20",
                        ["dkpChange"] = -75,
                        ["raid"] = "Blackwing Lair",
                        ["serverTime"] = 1585795765,
                        ["officer"] = "|cff8787EDSparklenips|r",
                    },
                    [1585196310] = {
                        ["names"] = "|cffFF7D0ANeekio|r, |cffFF7D0AEmmyy|r, |cffFF7D0AMorphintyme|r, |cffABD473Ithgar|r, |cffABD473Cyskul|r, |cffABD473Athico|r, |cff40C7EBVeriandra|r, |cff40C7EBJakemeoff|r, |cff40C7EBNeckbeardo|r, |cff40C7EBFlatulent|r, |cff40C7EBStitchess|r, |cff40C7EBXyen|r, |cffF58CBAOdin|r, |cffF58CBAAdvanty|r, |cffF58CBAMaryjohanna|r, |cffF58CBAForerunner|r, |cffFFFFFFPhilonious|r, |cffFFFFFFPuffhead|r, |cffFFFFFFBallour|r, |cffFFFFFFPriesticuffs|r, |cffFFFFFFTurbohealz|r, |cffFFF569Slipperyjohn|r, |cffFFF569Sicarrio|r, |cffFFF569Mozzarella|r, |cffFFF569Aku|r, |cffFFF569Mcstaberson|r, |cffFFF569Mariku|r, |cff8787EDLindo|r, |cff8787EDThenight|r, |cff8787EDRaspütin|r, |cff8787EDDalia|r, |cff8787EDGartog|r, |cff8787EDEdgyboi|r, |cffC79C6EKalita|r, |cffC79C6EGoobimus|r, |cffC79C6EGirlslayer|r, |cffC79C6EHotdogbroth|r, |cffC79C6EMihai|r, |cffC79C6EOragan|r, |cffC79C6EQew|r",
                        ["id"] = 1585196310,
                        ["text"] = "|cff22bb33Blackwing Lair - Chromaggus|r",
                        ["bossKill"] = "Chromaggus",
                        ["reason"] = "Boss Kill",
                        ["datetime"] = 1585196311,
                        ["time"] = "12:18:31 AM",
                        ["dkpChangeText"] = "|cff22bb3310 DKP|r",
                        ["date"] = "03/26/20",
                        ["dkpChange"] = 10,
                        ["raid"] = "Blackwing Lair",
                        ["serverTime"] = 1585196310,
                        ["officer"] = "|cffFF7D0ANeekio|r",
                    },
                    [1585792759] = {
                        ["names"] = "|cffFF7D0AMorphintyme|r",
                        ["id"] = 1585792759,
                        ["text"] = "|cffE71D36Item Win - |rNone",
                        ["item"] = "None",
                        ["reason"] = "Item Win",
                        ["datetime"] = 1585792761,
                        ["dkpChangeText"] = "|cffE71D36-108 DKP|r",
                        ["time"] = "06:59:21 PM",
                        ["date"] = "04/01/20",
                        ["dkpChange"] = -108,
                        ["raid"] = "Blackwing Lair",
                        ["serverTime"] = 1585792759,
                        ["officer"] = "|cff8787EDSparklenips|r",
                    },
                    [1585199123] = {
                        ["names"] = "|cffFFFFFFHoneypot|r",
                        ["id"] = 1585199123,
                        ["text"] = "|cffE71D36Item Win - |rNone",
                        ["dkpChangeText"] = "|cffE71D36-75 DKP|r",
                        ["reason"] = "Item Win",
                        ["datetime"] = 1585199125,
                        ["item"] = "None",
                        ["time"] = "10:05:25 PM",
                        ["date"] = "03/25/20",
                        ["dkpChange"] = -75,
                        ["raid"] = "Blackwing Lair",
                        ["serverTime"] = 1585199123,
                        ["officer"] = "|cff8787EDSparklenips|r",
                    },
                    [1585804246] = {
                        ["names"] = "|cffF58CBAOnes|r",
                        ["id"] = 1585804246,
                        ["text"] = "|cffE71D36Item Win - |rNone",
                        ["time"] = "10:10:46 PM",
                        ["reason"] = "Item Win",
                        ["datetime"] = 1585804246,
                        ["dkpChangeText"] = "|cffE71D36-22 DKP|r",
                        ["item"] = "None",
                        ["date"] = "04/01/20",
                        ["dkpChange"] = -22,
                        ["raid"] = "Blackwing Lair",
                        ["serverTime"] = 1585804246,
                        ["officer"] = "|cffC79C6ESnaildaddy|r",
                    },
                    [1585173631] = {
                        ["id"] = 1585173631,
                        ["text"] = "|cff22bb33Blackwing Lair - Completion Bonus|r",
                        ["names"] = "|cff40C7EBVeriandra|r",
                        ["reason"] = "Completion Bonus",
                        ["datetime"] = 1585173632,
                        ["dkpChangeText"] = "|cff22bb3370 DKP|r",
                        ["time"] = "03:00:32 PM",
                        ["date"] = "03/25/20",
                        ["dkpChange"] = 70,
                        ["raid"] = "Blackwing Lair",
                        ["serverTime"] = 1585173631,
                        ["officer"] = "|cff8787EDSparklenips|r",
                    },
                    [1585195055] = {
                        ["names"] = "|cffFF7D0ANeekio|r, |cffFF7D0AEmmyy|r, |cffFF7D0AMorphintyme|r, |cffABD473Ithgar|r, |cffABD473Athico|r, |cffABD473Cyskul|r, |cff40C7EBJakemeoff|r, |cff40C7EBNeckbeardo|r, |cff40C7EBFlatulent|r, |cff40C7EBVeriandra|r, |cff40C7EBXyen|r, |cff40C7EBStitchess|r, |cffF58CBAOdin|r, |cffF58CBAForerunner|r, |cffF58CBAMaryjohanna|r, |cffF58CBAAdvanty|r, |cffFFFFFFPriesticuffs|r, |cffFFFFFFTurbohealz|r, |cffFFFFFFPuffhead|r, |cffFFFFFFPhilonious|r, |cffFFFFFFBallour|r, |cffFFF569Mcstaberson|r, |cffFFF569Slipperyjohn|r, |cffFFF569Mozzarella|r, |cffFFF569Aku|r, |cffFFF569Mariku|r, |cffFFF569Sicarrio|r, |cff8787EDRaspütin|r, |cff8787EDLindo|r, |cff8787EDEdgyboi|r, |cff8787EDThenight|r, |cff8787EDDalia|r, |cff8787EDGartog|r, |cffC79C6EKalita|r, |cffC79C6EHotdogbroth|r, |cffC79C6EGoobimus|r, |cffC79C6EMihai|r, |cffC79C6EQew|r, |cffC79C6EOragan|r, |cffC79C6EGirlslayer|r",
                        ["id"] = 1585195055,
                        ["text"] = "|cff22bb33Blackwing Lair - Ebonroc|r",
                        ["bossKill"] = "Ebonroc",
                        ["reason"] = "Boss Kill",
                        ["datetime"] = 1585195056,
                        ["time"] = "11:57:36 PM",
                        ["dkpChangeText"] = "|cff22bb3310 DKP|r",
                        ["date"] = "03/25/20",
                        ["dkpChange"] = 10,
                        ["raid"] = "Blackwing Lair",
                        ["serverTime"] = 1585195055,
                        ["officer"] = "|cffFF7D0ANeekio|r",
                    },
                    [1585172949] = {
                        ["id"] = 1585172949,
                        ["text"] = "|cff22bb33Blackwing Lair - Completion Bonus|r",
                        ["names"] = "|cff8787EDVarix|r",
                        ["reason"] = "Completion Bonus",
                        ["datetime"] = 1585172950,
                        ["dkpChangeText"] = "|cff22bb33100 DKP|r",
                        ["time"] = "02:49:10 PM",
                        ["date"] = "03/25/20",
                        ["dkpChange"] = 100,
                        ["raid"] = "Blackwing Lair",
                        ["serverTime"] = 1585172949,
                        ["officer"] = "|cff8787EDSparklenips|r",
                    },
                    [1585796424] = {
                        ["names"] = "|cff8787EDDalia|r",
                        ["id"] = 1585796424,
                        ["text"] = "|cffE71D36Item Win - |rNone",
                        ["item"] = "None",
                        ["reason"] = "Item Win",
                        ["datetime"] = 1585796426,
                        ["dkpChangeText"] = "|cffE71D36-125 DKP|r",
                        ["time"] = "08:00:26 PM",
                        ["date"] = "04/01/20",
                        ["dkpChange"] = -125,
                        ["raid"] = "Blackwing Lair",
                        ["serverTime"] = 1585796424,
                        ["officer"] = "|cff8787EDSparklenips|r",
                    },
                    [1585197663] = {
                        ["id"] = 1585197663,
                        ["text"] = "|cff22bb33Blackwing Lair - Completion Bonus|r",
                        ["reason"] = "Completion Bonus",
                        ["names"] = "|cffFF7D0ANeekio|r, |cffFF7D0AEmmyy|r, |cffFF7D0AMorphintyme|r, |cffABD473Cyskul|r, |cffABD473Athico|r, |cffABD473Ithgar|r, |cff40C7EBVeriandra|r, |cff40C7EBJakemeoff|r, |cff40C7EBFlatulent|r, |cff40C7EBNeckbeardo|r, |cff40C7EBXyen|r, |cff40C7EBStitchess|r, |cffF58CBAAdvanty|r, |cffF58CBAOdin|r, |cffF58CBAForerunner|r, |cffF58CBAMaryjohanna|r, |cffFFFFFFTurbohealz|r, |cffFFFFFFPriesticuffs|r, |cffFFFFFFPhilonious|r, |cffFFFFFFPuffhead|r, |cffFFFFFFBallour|r, |cffFFF569Mcstaberson|r, |cffFFF569Mozzarella|r, |cffFFF569Slipperyjohn|r, |cffFFF569Sicarrio|r, |cffFFF569Mariku|r, |cffFFF569Aku|r, |cff8787EDGartog|r, |cff8787EDRaspütin|r, |cff8787EDLindo|r, |cff8787EDThenight|r, |cff8787EDEdgyboi|r, |cff8787EDDalia|r, |cffC79C6EKalita|r, |cffC79C6EOragan|r, |cffC79C6EGirlslayer|r, |cffC79C6EGoobimus|r, |cffC79C6EMihai|r, |cffC79C6EHotdogbroth|r, |cffC79C6EQew|r",
                        ["datetime"] = 1585197664,
                        ["time"] = "12:41:04 AM",
                        ["dkpChangeText"] = "|cff22bb3310 DKP|r",
                        ["date"] = "03/26/20",
                        ["dkpChange"] = 10,
                        ["raid"] = "Blackwing Lair",
                        ["serverTime"] = 1585197663,
                        ["officer"] = "|cffFF7D0ANeekio|r",
                    },
                    [1585192780] = {
                        ["names"] = "|cffC79C6EQew|r",
                        ["id"] = 1585192780,
                        ["text"] = "|cffE71D36Item Win - |r|cffa335ee|Hitem:19341::::::::60:::::::|h[Lifegiving Gem]|h|r",
                        ["dkpChangeText"] = "|cffE71D36-20 DKP|r",
                        ["reason"] = "Item Win",
                        ["datetime"] = 1585192781,
                        ["time"] = "11:19:41 PM",
                        ["item"] = "|cffa335ee|Hitem:19341::::::::60:::::::|h[Lifegiving Gem]|h|r",
                        ["date"] = "03/25/20",
                        ["dkpChange"] = -20,
                        ["raid"] = "Blackwing Lair",
                        ["serverTime"] = 1585192780,
                        ["officer"] = "|cffFF7D0ANeekio|r",
                    },
                    [1585173655] = {
                        ["id"] = 1585173655,
                        ["text"] = "|cff22bb33Blackwing Lair - Completion Bonus|r",
                        ["names"] = "|cffF58CBAMaryjohanna|r, |cffF58CBAAdvanty|r, |cffF58CBAPapasquach|r",
                        ["reason"] = "Completion Bonus",
                        ["datetime"] = 1585173656,
                        ["dkpChangeText"] = "|cff22bb33100 DKP|r",
                        ["time"] = "03:00:56 PM",
                        ["date"] = "03/25/20",
                        ["dkpChange"] = 100,
                        ["raid"] = "Blackwing Lair",
                        ["serverTime"] = 1585173655,
                        ["officer"] = "|cff8787EDSparklenips|r",
                    },
                    [1585201454] = {
                        ["names"] = "|cffFF7D0AInebriated|r, |cffFF7D0AInsub|r, |cffABD473Stellâ|r, |cffABD473Ugro|r, |cffABD473Primera|r, |cffABD473Wahcha|r, |cff40C7EBEolith|r, |cff40C7EBSuprarz|r, |cff40C7EBOxford|r, |cff40C7EBZukohere|r, |cff40C7EBBugaboom|r, |cff40C7EBChipgizmo|r, |cffF58CBAPapasquach|r, |cffF58CBAOnes|r, |cffF58CBAGneissguy|r, |cffF58CBALawduk|r, |cffFFFFFFMongous|r, |cffFFFFFFBubblè|r, |cffFFFFFFEvangelina|r, |cffFFFFFFHoneypot|r, |cffFFFFFFRez|r, |cffFFFFFFFawntine|r, |cffFFF569Mystile|r, |cffFFF569Inigma|r, |cffFFF569Mujinn|r, |cffFFF569Tuggernuts|r, |cff8787EDSparklenips|r, |cff8787EDThepurple|r, |cff8787EDVarix|r, |cff8787EDLittledubs|r, |cff8787EDSoftfondle|r, |cffC79C6ELuckerdawg|r, |cffC79C6EReina|r, |cffC79C6EGalagus|r, |cffC79C6ENightshelf|r, |cffC79C6ERetkin|r, |cffC79C6EJeffry|r, |cffC79C6EShinynickels|r, |cffC79C6ETokentoken|r, |cffC79C6ESnaildaddy|r",
                        ["id"] = 1585201454,
                        ["text"] = "|cff22bb33Blackwing Lair - Chromaggus|r",
                        ["bossKill"] = "Chromaggus",
                        ["reason"] = "Boss Kill",
                        ["datetime"] = 1585201456,
                        ["time"] = "10:44:16 PM",
                        ["dkpChangeText"] = "|cff22bb3310 DKP|r",
                        ["date"] = "03/25/20",
                        ["dkpChange"] = 10,
                        ["raid"] = "Blackwing Lair",
                        ["serverTime"] = 1585201454,
                        ["officer"] = "|cff8787EDSparklenips|r",
                    },
                    [1585799433] = {
                        ["names"] = "|cff40C7EBEolith|r",
                        ["id"] = 1585799433,
                        ["text"] = "|cffE71D36Item Win - |rNone",
                        ["item"] = "None",
                        ["reason"] = "Item Win",
                        ["datetime"] = 1585799435,
                        ["dkpChangeText"] = "|cffE71D36-11 DKP|r",
                        ["time"] = "08:50:35 PM",
                        ["date"] = "04/01/20",
                        ["dkpChange"] = -11,
                        ["raid"] = "Blackwing Lair",
                        ["serverTime"] = 1585799433,
                        ["officer"] = "|cff8787EDSparklenips|r",
                    },
                    [1585173661] = {
                        ["id"] = 1585173661,
                        ["text"] = "|cff22bb33Blackwing Lair - Completion Bonus|r",
                        ["names"] = "|cffF58CBAOdin|r",
                        ["reason"] = "Completion Bonus",
                        ["datetime"] = 1585173662,
                        ["dkpChangeText"] = "|cff22bb3370 DKP|r",
                        ["time"] = "03:01:02 PM",
                        ["date"] = "03/25/20",
                        ["dkpChange"] = 70,
                        ["raid"] = "Blackwing Lair",
                        ["serverTime"] = 1585173661,
                        ["officer"] = "|cff8787EDSparklenips|r",
                    },
                    [1585799265] = {
                        ["names"] = "|cffFF7D0AMorphintyme|r, |cffFF7D0ANeekio|r, |cffFF7D0AEmmyy|r, |cffABD473Cyskul|r, |cffABD473Ithgar|r, |cffABD473Athico|r, |cffABD473Bádtothebow|r, |cffABD473Bigbootyhoho|r, |cff40C7EBJakemeoff|r, |cff40C7EBStitchess|r, |cff40C7EBNeckbeardo|r, |cff40C7EBXyen|r, |cff40C7EBVeriandra|r, |cff40C7EBFlatulent|r, |cff40C7EBEolith|r, |cffF58CBAAdvanty|r, |cffF58CBAOdin|r, |cffF58CBAMaryjohanna|r, |cffF58CBAPapasquach|r, |cffFFFFFFPuffhead|r, |cffFFFFFFBallour|r, |cffFFFFFFTurbohealz|r, |cffFFFFFFPriesticuffs|r, |cffFFFFFFPhilonious|r, |cffFFF569Mcstaberson|r, |cffFFF569Sicarrio|r, |cffFFF569Slipperyjohn|r, |cffFFF569Mariku|r, |cff8787EDRaspütin|r, |cff8787EDEdgyboi|r, |cff8787EDLindo|r, |cff8787EDSparklenips|r, |cff8787EDDalia|r, |cffC79C6EHotdogbroth|r, |cffC79C6EKalita|r, |cffC79C6EShinynickels|r, |cffC79C6EOragan|r, |cffC79C6EGoobimus|r, |cffC79C6EQew|r, |cffC79C6EGirlslayer|r",
                        ["id"] = 1585799265,
                        ["text"] = "|cff22bb33Blackwing Lair - Chromaggus|r",
                        ["bossKill"] = "Chromaggus",
                        ["reason"] = "Boss Kill",
                        ["datetime"] = 1585799267,
                        ["time"] = "08:47:47 PM",
                        ["dkpChangeText"] = "|cff22bb3310 DKP|r",
                        ["date"] = "04/01/20",
                        ["dkpChange"] = 10,
                        ["raid"] = "Blackwing Lair",
                        ["serverTime"] = 1585799265,
                        ["officer"] = "|cff8787EDSparklenips|r",
                    },
                    [1585173165] = {
                        ["id"] = 1585173165,
                        ["text"] = "|cff22bb33Blackwing Lair - Completion Bonus|r",
                        ["names"] = "|cff40C7EBChipgizmo|r",
                        ["reason"] = "Completion Bonus",
                        ["datetime"] = 1585173166,
                        ["dkpChangeText"] = "|cff22bb3360 DKP|r",
                        ["time"] = "02:52:46 PM",
                        ["date"] = "03/25/20",
                        ["dkpChange"] = 60,
                        ["raid"] = "Blackwing Lair",
                        ["serverTime"] = 1585173165,
                        ["officer"] = "|cff8787EDSparklenips|r",
                    },
                    [1585173290] = {
                        ["id"] = 1585173290,
                        ["text"] = "|cff22bb33Blackwing Lair - Completion Bonus|r",
                        ["names"] = "|cffF58CBACaptnutsack|r",
                        ["reason"] = "Completion Bonus",
                        ["datetime"] = 1585173291,
                        ["dkpChangeText"] = "|cff22bb3350 DKP|r",
                        ["time"] = "02:54:51 PM",
                        ["date"] = "03/25/20",
                        ["dkpChange"] = 50,
                        ["raid"] = "Blackwing Lair",
                        ["serverTime"] = 1585173290,
                        ["officer"] = "|cff8787EDSparklenips|r",
                    },
                    [1585797547] = {
                        ["names"] = "|cffFF7D0AEmmyy|r",
                        ["id"] = 1585797547,
                        ["text"] = "|cffE71D36Item Win - |rNone",
                        ["item"] = "None",
                        ["reason"] = "Item Win",
                        ["datetime"] = 1585797549,
                        ["dkpChangeText"] = "|cffE71D36-19 DKP|r",
                        ["time"] = "08:19:09 PM",
                        ["date"] = "04/01/20",
                        ["dkpChange"] = -19,
                        ["raid"] = "Blackwing Lair",
                        ["serverTime"] = 1585797547,
                        ["officer"] = "|cff8787EDSparklenips|r",
                    },
                    [1585797495] = {
                        ["names"] = "|cffFF7D0AMorphintyme|r, |cffFF7D0ANeekio|r, |cffFF7D0AEmmyy|r, |cffABD473Athico|r, |cffABD473Cyskul|r, |cffABD473Bigbootyhoho|r, |cffABD473Bádtothebow|r, |cffABD473Ithgar|r, |cff40C7EBVeriandra|r, |cff40C7EBEolith|r, |cff40C7EBStitchess|r, |cff40C7EBXyen|r, |cff40C7EBNeckbeardo|r, |cff40C7EBFlatulent|r, |cff40C7EBJakemeoff|r, |cffF58CBAPapasquach|r, |cffF58CBAAdvanty|r, |cffF58CBAMaryjohanna|r, |cffF58CBAOdin|r, |cffFFFFFFPuffhead|r, |cffFFFFFFBallour|r, |cffFFFFFFTurbohealz|r, |cffFFFFFFPriesticuffs|r, |cffFFFFFFPhilonious|r, |cffFFF569Mcstaberson|r, |cffFFF569Sicarrio|r, |cffFFF569Slipperyjohn|r, |cffFFF569Mariku|r, |cff8787EDRaspütin|r, |cff8787EDEdgyboi|r, |cff8787EDLindo|r, |cff8787EDSparklenips|r, |cff8787EDDalia|r, |cffC79C6EHotdogbroth|r, |cffC79C6EKalita|r, |cffC79C6EOragan|r, |cffC79C6EGoobimus|r, |cffC79C6EShinynickels|r, |cffC79C6EQew|r, |cffC79C6EGirlslayer|r",
                        ["id"] = 1585797495,
                        ["text"] = "|cff22bb33Blackwing Lair - Ebonroc|r",
                        ["bossKill"] = "Ebonroc",
                        ["reason"] = "Boss Kill",
                        ["datetime"] = 1585797497,
                        ["time"] = "08:18:17 PM",
                        ["dkpChangeText"] = "|cff22bb3310 DKP|r",
                        ["date"] = "04/01/20",
                        ["dkpChange"] = 10,
                        ["raid"] = "Blackwing Lair",
                        ["serverTime"] = 1585797495,
                        ["officer"] = "|cff8787EDSparklenips|r",
                    },
                    [1585791833] = {
                        ["names"] = "|cffABD473Ithgar|r",
                        ["id"] = 1585791833,
                        ["text"] = "|cffE71D36Item Win - |rNone",
                        ["item"] = "None",
                        ["reason"] = "Item Win",
                        ["datetime"] = 1585791835,
                        ["dkpChangeText"] = "|cffE71D36-17 DKP|r",
                        ["time"] = "06:43:55 PM",
                        ["date"] = "04/01/20",
                        ["dkpChange"] = -17,
                        ["raid"] = "Blackwing Lair",
                        ["serverTime"] = 1585791833,
                        ["officer"] = "|cff8787EDSparklenips|r",
                    },
                    [1585791645] = {
                        ["id"] = 1585791645,
                        ["text"] = "|cff22bb33Blackwing Lair - On Time Bonus|r",
                        ["names"] = "|cffFF7D0AEmmyy|r, |cffFF7D0ANeekio|r, |cffFF7D0AMorphintyme|r, |cffABD473Bádtothebow|r, |cffABD473Athico|r, |cffABD473Ithgar|r, |cffABD473Cyskul|r, |cffABD473Bigbootyhoho|r, |cff40C7EBEolith|r, |cff40C7EBJakemeoff|r, |cff40C7EBVeriandra|r, |cff40C7EBFlatulent|r, |cff40C7EBStitchess|r, |cff40C7EBNeckbeardo|r, |cff40C7EBXyen|r, |cffF58CBAPapasquach|r, |cffF58CBAAdvanty|r, |cffF58CBAOdin|r, |cffF58CBAMaryjohanna|r, |cffFFFFFFBallour|r, |cffFFFFFFPuffhead|r, |cffFFFFFFPriesticuffs|r, |cffFFFFFFTurbohealz|r, |cffFFFFFFPhilonious|r, |cffFFF569Slipperyjohn|r, |cffFFF569Mcstaberson|r, |cffFFF569Sicarrio|r, |cffFFF569Mariku|r, |cff8787EDEdgyboi|r, |cff8787EDLindo|r, |cff8787EDRaspütin|r, |cff8787EDSparklenips|r, |cff8787EDDalia|r, |cffC79C6EGoobimus|r, |cffC79C6EKalita|r, |cffC79C6EGirlslayer|r, |cffC79C6EOragan|r, |cffC79C6EQew|r, |cffC79C6EHotdogbroth|r, |cffC79C6EShinynickels|r",
                        ["reason"] = "On Time Bonus",
                        ["datetime"] = 1585791647,
                        ["dkpChangeText"] = "|cff22bb3310 DKP|r",
                        ["time"] = "06:40:47 PM",
                        ["date"] = "04/01/20",
                        ["dkpChange"] = 10,
                        ["raid"] = "Blackwing Lair",
                        ["serverTime"] = 1585791645,
                        ["officer"] = "|cff8787EDSparklenips|r",
                    },
                    [1585172934] = {
                        ["id"] = 1585172934,
                        ["text"] = "|cff22bb33Blackwing Lair - Completion Bonus|r",
                        ["names"] = "|cff8787EDSparklenips|r",
                        ["reason"] = "Completion Bonus",
                        ["datetime"] = 1585172935,
                        ["dkpChangeText"] = "|cff22bb33100 DKP|r",
                        ["time"] = "02:48:55 PM",
                        ["date"] = "03/25/20",
                        ["dkpChange"] = 100,
                        ["raid"] = "Blackwing Lair",
                        ["serverTime"] = 1585172934,
                        ["officer"] = "|cff8787EDSparklenips|r",
                    },
                    [1585173452] = {
                        ["id"] = 1585173452,
                        ["text"] = "|cff22bb33Blackwing Lair - Completion Bonus|r",
                        ["names"] = "|cffFFFFFFPuffhead|r, |cffFFFFFFPriesticuffs|r, |cffFFFFFFEvangelina|r",
                        ["reason"] = "Completion Bonus",
                        ["datetime"] = 1585173453,
                        ["dkpChangeText"] = "|cff22bb33100 DKP|r",
                        ["time"] = "02:57:33 PM",
                        ["date"] = "03/25/20",
                        ["dkpChange"] = 100,
                        ["raid"] = "Blackwing Lair",
                        ["serverTime"] = 1585173452,
                        ["officer"] = "|cff8787EDSparklenips|r",
                    },
                    [1585202434] = {
                        ["names"] = "|cff8787EDVarix|r",
                        ["id"] = 1585202434,
                        ["text"] = "|cffE71D36Item Win - |rNone",
                        ["dkpChangeText"] = "|cffE71D36-17 DKP|r",
                        ["reason"] = "Item Win",
                        ["datetime"] = 1585202436,
                        ["item"] = "None",
                        ["time"] = "11:00:36 PM",
                        ["date"] = "03/25/20",
                        ["dkpChange"] = -17,
                        ["raid"] = "Blackwing Lair",
                        ["serverTime"] = 1585202434,
                        ["officer"] = "|cff8787EDSparklenips|r",
                    },
                    [1585800373] = {
                        ["names"] = "|cffFF7D0AMorphintyme|r",
                        ["id"] = 1585800373,
                        ["text"] = "|cffE71D36Item Win - |rNone",
                        ["item"] = "None",
                        ["reason"] = "Item Win",
                        ["datetime"] = 1585800375,
                        ["dkpChangeText"] = "|cffE71D36-89 DKP|r",
                        ["time"] = "09:06:15 PM",
                        ["date"] = "04/01/20",
                        ["dkpChange"] = -89,
                        ["raid"] = "Blackwing Lair",
                        ["serverTime"] = 1585800373,
                        ["officer"] = "|cff8787EDSparklenips|r",
                    },
                    [1585173003] = {
                        ["id"] = 1585173003,
                        ["text"] = "|cff22bb33Blackwing Lair - Completion Bonus|r",
                        ["names"] = "|cffABD473Primera|r, |cffABD473Ugro|r",
                        ["reason"] = "Completion Bonus",
                        ["datetime"] = 1585173004,
                        ["dkpChangeText"] = "|cff22bb33100 DKP|r",
                        ["time"] = "02:50:04 PM",
                        ["date"] = "03/25/20",
                        ["dkpChange"] = 100,
                        ["raid"] = "Blackwing Lair",
                        ["serverTime"] = 1585173003,
                        ["officer"] = "|cff8787EDSparklenips|r",
                    },
                    [1585793628] = {
                        ["names"] = "|cffFF7D0AMorphintyme|r, |cffFF7D0ANeekio|r, |cffFF7D0AEmmyy|r, |cffABD473Athico|r, |cffABD473Bigbootyhoho|r, |cffABD473Bádtothebow|r, |cffABD473Cyskul|r, |cffABD473Ithgar|r, |cff40C7EBVeriandra|r, |cff40C7EBNeckbeardo|r, |cff40C7EBStitchess|r, |cff40C7EBXyen|r, |cff40C7EBEolith|r, |cff40C7EBFlatulent|r, |cff40C7EBJakemeoff|r, |cffF58CBAOdin|r, |cffF58CBAAdvanty|r, |cffF58CBAMaryjohanna|r, |cffF58CBAPapasquach|r, |cffFFFFFFPuffhead|r, |cffFFFFFFBallour|r, |cffFFFFFFTurbohealz|r, |cffFFFFFFPriesticuffs|r, |cffFFFFFFPhilonious|r, |cffFFF569Slipperyjohn|r, |cffFFF569Sicarrio|r, |cffFFF569Mcstaberson|r, |cffFFF569Mariku|r, |cff8787EDRaspütin|r, |cff8787EDLindo|r, |cff8787EDSparklenips|r, |cff8787EDDalia|r, |cff8787EDEdgyboi|r, |cffC79C6EShinynickels|r, |cffC79C6EKalita|r, |cffC79C6EHotdogbroth|r, |cffC79C6EGoobimus|r, |cffC79C6EQew|r, |cffC79C6EOragan|r, |cffC79C6EGirlslayer|r",
                        ["id"] = 1585793628,
                        ["text"] = "|cff22bb33Blackwing Lair - Broodlord Lashlayer|r",
                        ["bossKill"] = "Broodlord Lashlayer",
                        ["reason"] = "Boss Kill",
                        ["datetime"] = 1585793630,
                        ["time"] = "07:13:50 PM",
                        ["dkpChangeText"] = "|cff22bb3310 DKP|r",
                        ["date"] = "04/01/20",
                        ["dkpChange"] = 10,
                        ["raid"] = "Blackwing Lair",
                        ["serverTime"] = 1585793628,
                        ["officer"] = "|cff8787EDSparklenips|r",
                    },
                    [1585800513] = {
                        ["names"] = "|cffFF7D0AInsub|r",
                        ["id"] = 1585800513,
                        ["text"] = "|cffE71D36Item Win - |rNone",
                        ["time"] = "09:08:33 PM",
                        ["reason"] = "Item Win",
                        ["datetime"] = 1585800513,
                        ["dkpChangeText"] = "|cffE71D36-23 DKP|r",
                        ["item"] = "None",
                        ["date"] = "04/01/20",
                        ["dkpChange"] = -23,
                        ["raid"] = "Blackwing Lair",
                        ["serverTime"] = 1585800513,
                        ["officer"] = "|cffC79C6ESnaildaddy|r",
                    },
                    [1585188776] = {
                        ["reason"] = "Boss Kill",
                        ["id"] = 1585188776,
                        ["text"] = "|cff22bb33Blackwing Lair - Vaelastrasz the Corrupt|r",
                        ["bossKill"] = "Vaelastrasz the Corrupt",
                        ["names"] = "|cffFF7D0ANeekio|r, |cffFF7D0AMorphintyme|r, |cffFF7D0AEmmyy|r, |cffABD473Athico|r, |cffABD473Ithgar|r, |cffABD473Cyskul|r, |cff40C7EBJakemeoff|r, |cff40C7EBVeriandra|r, |cff40C7EBFlatulent|r, |cff40C7EBNeckbeardo|r, |cff40C7EBXyen|r, |cff40C7EBStitchess|r, |cffF58CBAOdin|r, |cffF58CBAMaryjohanna|r, |cffF58CBAForerunner|r, |cffF58CBAAdvanty|r, |cffFFFFFFPriesticuffs|r, |cffFFFFFFPuffhead|r, |cffFFFFFFPhilonious|r, |cffFFFFFFBallour|r, |cffFFFFFFTurbohealz|r, |cffFFF569Slipperyjohn|r, |cffFFF569Mozzarella|r, |cffFFF569Mcstaberson|r, |cffFFF569Aku|r, |cffFFF569Sicarrio|r, |cffFFF569Mariku|r, |cff8787EDEdgyboi|r, |cff8787EDRaspütin|r, |cff8787EDDalia|r, |cff8787EDLindo|r, |cff8787EDThenight|r, |cff8787EDGartog|r, |cffC79C6EKalita|r, |cffC79C6EHotdogbroth|r, |cffC79C6EGoobimus|r, |cffC79C6EMihai|r, |cffC79C6EOragan|r, |cffC79C6EQew|r, |cffC79C6EGirlslayer|r",
                        ["datetime"] = 1585188777,
                        ["dkpChangeText"] = "|cff22bb3310 DKP|r",
                        ["time"] = "10:12:57 PM",
                        ["date"] = "03/25/20",
                        ["dkpChange"] = 10,
                        ["raid"] = "Blackwing Lair",
                        ["serverTime"] = 1585188776,
                        ["officer"] = "|cffFF7D0ANeekio|r",
                    },
                    [1585804346] = {
                        ["names"] = "|cffFF7D0AKalijah|r, |cffFF7D0AInebriated|r, |cffFF7D0ABlanebites|r, |cffFF7D0AInsub|r, |cffABD473Primera|r, |cffABD473Ugro|r, |cffABD473Wahcha|r, |cffABD473Stellâ|r, |cff40C7EBZukohere|r, |cff40C7EBSuprarz|r, |cff40C7EBBugaboom|r, |cff40C7EBOxford|r, |cff40C7EBChipgizmo|r, |cffF58CBAGneissguy|r, |cffF58CBADolamroth|r, |cffF58CBALawduk|r, |cffF58CBAOnes|r, |cffFFFFFFRez|r, |cffFFFFFFFawntine|r, |cffFFFFFFMongous|r, |cffFFFFFFHoneypot|r, |cffFFF569Aku|r, |cffFFF569Mystile|r, |cffFFF569Trickster|r, |cffFFF569Tuggernuts|r, |cffFFF569Inigma|r, |cff8787EDSoftfondle|r, |cff8787EDThepurple|r, |cff8787EDTalisker|r, |cff8787EDLittledubs|r, |cff8787EDIhurricanel|r, |cff8787EDVarix|r, |cffC79C6ELuckerdawg|r, |cffC79C6EGalagus|r, |cffC79C6EReina|r, |cffC79C6ESnaildaddy|r, |cffC79C6ENightshelf|r, |cffC79C6ETokentoken|r, |cffC79C6ERetkin|r, |cffC79C6EJeffry|r",
                        ["id"] = 1585804346,
                        ["text"] = "|cff22bb33Blackwing Lair - Flamegor|r",
                        ["bossKill"] = "Flamegor",
                        ["reason"] = "Boss Kill",
                        ["datetime"] = 1585804346,
                        ["time"] = "10:12:26 PM",
                        ["dkpChangeText"] = "|cff22bb3310 DKP|r",
                        ["date"] = "04/01/20",
                        ["dkpChange"] = 10,
                        ["raid"] = "Blackwing Lair",
                        ["serverTime"] = 1585804346,
                        ["officer"] = "|cffC79C6ESnaildaddy|r",
                    },
                    [1585173500] = {
                        ["id"] = 1585173500,
                        ["text"] = "|cff22bb33Blackwing Lair - Completion Bonus|r",
                        ["names"] = "|cffC79C6EShinynickels|r",
                        ["reason"] = "Completion Bonus",
                        ["datetime"] = 1585173501,
                        ["dkpChangeText"] = "|cff22bb3355 DKP|r",
                        ["time"] = "02:58:21 PM",
                        ["date"] = "03/25/20",
                        ["dkpChange"] = 55,
                        ["raid"] = "Blackwing Lair",
                        ["serverTime"] = 1585173500,
                        ["officer"] = "|cff8787EDSparklenips|r",
                    },
                    [1585800546] = {
                        ["names"] = "|cffF58CBAOnes|r",
                        ["id"] = 1585800546,
                        ["text"] = "|cffE71D36Item Win - |rNone",
                        ["time"] = "09:09:06 PM",
                        ["reason"] = "Item Win",
                        ["datetime"] = 1585800546,
                        ["dkpChangeText"] = "|cffE71D36-22 DKP|r",
                        ["item"] = "None",
                        ["date"] = "04/01/20",
                        ["dkpChange"] = -22,
                        ["raid"] = "Blackwing Lair",
                        ["serverTime"] = 1585800546,
                        ["officer"] = "|cffC79C6ESnaildaddy|r",
                    },
                    [1585798748] = {
                        ["reason"] = "Boss Kill",
                        ["id"] = 1585798748,
                        ["text"] = "|cff22bb33Blackwing Lair - Razorgore the Untamed|r",
                        ["bossKill"] = "Razorgore the Untamed",
                        ["names"] = "|cffFF7D0ABlanebites|r, |cffFF7D0AInebriated|r, |cffFF7D0AKalijah|r, |cffFF7D0AInsub|r, |cffABD473Primera|r, |cffABD473Ugro|r, |cffABD473Wahcha|r, |cffABD473Stellâ|r, |cff40C7EBZukohere|r, |cff40C7EBSuprarz|r, |cff40C7EBBugaboom|r, |cff40C7EBChipgizmo|r, |cff40C7EBOxford|r, |cffF58CBAOnes|r, |cffF58CBALawduk|r, |cffF58CBAGneissguy|r, |cffFFFFFFEvangelina|r, |cffFFFFFFHoneypot|r, |cffFFFFFFFawntine|r, |cffFFFFFFMongous|r, |cffFFFFFFRez|r, |cffFFF569Tuggernuts|r, |cffFFF569Mystile|r, |cffFFF569Trickster|r, |cffFFF569Inigma|r, |cffFFF569Aku|r, |cff8787EDIhurricanel|r, |cff8787EDSoftfondle|r, |cff8787EDTalisker|r, |cff8787EDThepurple|r, |cff8787EDLittledubs|r, |cff8787EDVarix|r, |cffC79C6ERetkin|r, |cffC79C6EGalagus|r, |cffC79C6EReina|r, |cffC79C6ENightshelf|r, |cffC79C6EJeffry|r, |cffC79C6ESnaildaddy|r, |cffC79C6ETokentoken|r, |cffC79C6ELuckerdawg|r",
                        ["datetime"] = 1585798748,
                        ["dkpChangeText"] = "|cff22bb3310 DKP|r",
                        ["time"] = "08:39:08 PM",
                        ["date"] = "04/01/20",
                        ["dkpChange"] = 10,
                        ["raid"] = "Blackwing Lair",
                        ["serverTime"] = 1585798748,
                        ["officer"] = "|cffC79C6ESnaildaddy|r",
                    },
                    [1585202562] = {
                        ["names"] = "|cffABD473Wahcha|r",
                        ["id"] = 1585202562,
                        ["text"] = "|cffE71D36Item Win - |rNone",
                        ["dkpChangeText"] = "|cffE71D36-55 DKP|r",
                        ["reason"] = "Item Win",
                        ["datetime"] = 1585202564,
                        ["item"] = "None",
                        ["time"] = "11:02:44 PM",
                        ["date"] = "03/25/20",
                        ["dkpChange"] = -55,
                        ["raid"] = "Blackwing Lair",
                        ["serverTime"] = 1585202562,
                        ["officer"] = "|cff8787EDSparklenips|r",
                    },
                    [1585193386] = {
                        ["names"] = "|cffFF7D0ANeekio|r, |cffFF7D0AMorphintyme|r, |cffFF7D0AEmmyy|r, |cffABD473Cyskul|r, |cffABD473Ithgar|r, |cffABD473Athico|r, |cff40C7EBVeriandra|r, |cff40C7EBNeckbeardo|r, |cff40C7EBJakemeoff|r, |cff40C7EBXyen|r, |cff40C7EBStitchess|r, |cff40C7EBFlatulent|r, |cffF58CBAMaryjohanna|r, |cffF58CBAOdin|r, |cffF58CBAAdvanty|r, |cffF58CBAForerunner|r, |cffFFFFFFPuffhead|r, |cffFFFFFFPhilonious|r, |cffFFFFFFBallour|r, |cffFFFFFFPriesticuffs|r, |cffFFFFFFTurbohealz|r, |cffFFF569Mcstaberson|r, |cffFFF569Sicarrio|r, |cffFFF569Slipperyjohn|r, |cffFFF569Mariku|r, |cffFFF569Aku|r, |cffFFF569Mozzarella|r, |cff8787EDEdgyboi|r, |cff8787EDThenight|r, |cff8787EDLindo|r, |cff8787EDDalia|r, |cff8787EDRaspütin|r, |cff8787EDGartog|r, |cffC79C6EKalita|r, |cffC79C6EOragan|r, |cffC79C6EHotdogbroth|r, |cffC79C6EMihai|r, |cffC79C6EQew|r, |cffC79C6EGoobimus|r, |cffC79C6EGirlslayer|r",
                        ["id"] = 1585193386,
                        ["text"] = "|cff22bb33Blackwing Lair - Firemaw|r",
                        ["bossKill"] = "Firemaw",
                        ["reason"] = "Boss Kill",
                        ["datetime"] = 1585193387,
                        ["time"] = "11:29:47 PM",
                        ["dkpChangeText"] = "|cff22bb3310 DKP|r",
                        ["date"] = "03/25/20",
                        ["dkpChange"] = 10,
                        ["raid"] = "Blackwing Lair",
                        ["serverTime"] = 1585193386,
                        ["officer"] = "|cffFF7D0ANeekio|r",
                    },
                    [1585797800] = {
                        ["names"] = "|cffFF7D0AMorphintyme|r, |cffFF7D0ANeekio|r, |cffFF7D0AEmmyy|r, |cffABD473Ithgar|r, |cffABD473Bigbootyhoho|r, |cffABD473Athico|r, |cffABD473Cyskul|r, |cffABD473Bádtothebow|r, |cff40C7EBStitchess|r, |cff40C7EBJakemeoff|r, |cff40C7EBEolith|r, |cff40C7EBNeckbeardo|r, |cff40C7EBXyen|r, |cff40C7EBVeriandra|r, |cff40C7EBFlatulent|r, |cffF58CBAPapasquach|r, |cffF58CBAMaryjohanna|r, |cffF58CBAOdin|r, |cffF58CBAAdvanty|r, |cffFFFFFFPuffhead|r, |cffFFFFFFBallour|r, |cffFFFFFFTurbohealz|r, |cffFFFFFFPriesticuffs|r, |cffFFFFFFPhilonious|r, |cffFFF569Slipperyjohn|r, |cffFFF569Sicarrio|r, |cffFFF569Mcstaberson|r, |cffFFF569Mariku|r, |cff8787EDLindo|r, |cff8787EDSparklenips|r, |cff8787EDEdgyboi|r, |cff8787EDRaspütin|r, |cff8787EDDalia|r, |cffC79C6EGoobimus|r, |cffC79C6EKalita|r, |cffC79C6EShinynickels|r, |cffC79C6EHotdogbroth|r, |cffC79C6EQew|r, |cffC79C6EOragan|r, |cffC79C6EGirlslayer|r",
                        ["id"] = 1585797800,
                        ["text"] = "|cff22bb33Blackwing Lair - Flamegor|r",
                        ["bossKill"] = "Flamegor",
                        ["reason"] = "Boss Kill",
                        ["datetime"] = 1585797802,
                        ["time"] = "08:23:22 PM",
                        ["dkpChangeText"] = "|cff22bb3310 DKP|r",
                        ["date"] = "04/01/20",
                        ["dkpChange"] = 10,
                        ["raid"] = "Blackwing Lair",
                        ["serverTime"] = 1585797800,
                        ["officer"] = "|cff8787EDSparklenips|r",
                    },
                    [1585184717] = {
                        ["id"] = 1585184717,
                        ["text"] = "|cff22bb33Blackwing Lair - On Time Bonus|r",
                        ["reason"] = "On Time Bonus",
                        ["names"] = "|cffFF7D0AMorphintyme|r, |cffFF7D0ANeekio|r, |cffABD473Cyskul|r, |cffABD473Athico|r, |cffABD473Ithgar|r, |cff40C7EBJakemeoff|r, |cff40C7EBVeriandra|r, |cff40C7EBNeckbeardo|r, |cff40C7EBStitchess|r, |cff40C7EBXyen|r, |cffF58CBAForerunner|r, |cffF58CBAOdin|r, |cffF58CBAAdvanty|r, |cffF58CBAMaryjohanna|r, |cffFFFFFFBallour|r, |cffFFFFFFPuffhead|r, |cffFFFFFFPhilonious|r, |cffFFF569Mcstaberson|r, |cffFFF569Slipperyjohn|r, |cffFFF569Mozzarella|r, |cffFFF569Aku|r, |cffFFF569Mariku|r, |cff8787EDThenight|r, |cff8787EDEdgyboi|r, |cff8787EDRaspütin|r, |cff8787EDLindo|r, |cff8787EDGartog|r, |cff8787EDDalia|r, |cffC79C6EGirlslayer|r, |cffC79C6EKalita|r, |cffC79C6EQew|r, |cffC79C6EOragan|r, |cffC79C6EMihai|r, |cffC79C6EHotdogbroth|r, |cffC79C6EGoobimus|r",
                        ["datetime"] = 1585184718,
                        ["time"] = "09:05:18 PM",
                        ["dkpChangeText"] = "|cff22bb3310 DKP|r",
                        ["date"] = "03/25/20",
                        ["dkpChange"] = 10,
                        ["raid"] = "Blackwing Lair",
                        ["serverTime"] = 1585184717,
                        ["officer"] = "|cffFF7D0ANeekio|r",
                    },
                    [1585191594] = {
                        ["names"] = "|cffFF7D0ANeekio|r, |cffFF7D0AMorphintyme|r, |cffFF7D0AEmmyy|r, |cffABD473Cyskul|r, |cffABD473Ithgar|r, |cffABD473Athico|r, |cff40C7EBJakemeoff|r, |cff40C7EBFlatulent|r, |cff40C7EBVeriandra|r, |cff40C7EBStitchess|r, |cff40C7EBNeckbeardo|r, |cff40C7EBXyen|r, |cffF58CBAAdvanty|r, |cffF58CBAMaryjohanna|r, |cffF58CBAForerunner|r, |cffF58CBAOdin|r, |cffFFFFFFPriesticuffs|r, |cffFFFFFFTurbohealz|r, |cffFFFFFFPhilonious|r, |cffFFFFFFPuffhead|r, |cffFFFFFFBallour|r, |cffFFF569Slipperyjohn|r, |cffFFF569Mcstaberson|r, |cffFFF569Mozzarella|r, |cffFFF569Sicarrio|r, |cffFFF569Mariku|r, |cffFFF569Aku|r, |cff8787EDRaspütin|r, |cff8787EDDalia|r, |cff8787EDEdgyboi|r, |cff8787EDGartog|r, |cff8787EDLindo|r, |cff8787EDThenight|r, |cffC79C6EKalita|r, |cffC79C6EOragan|r, |cffC79C6EGirlslayer|r, |cffC79C6EGoobimus|r, |cffC79C6EMihai|r, |cffC79C6EQew|r, |cffC79C6EHotdogbroth|r",
                        ["id"] = 1585191594,
                        ["text"] = "|cff22bb33Blackwing Lair - Broodlord Lashlayer|r",
                        ["bossKill"] = "Broodlord Lashlayer",
                        ["reason"] = "Boss Kill",
                        ["datetime"] = 1585191595,
                        ["time"] = "10:59:55 PM",
                        ["dkpChangeText"] = "|cff22bb3310 DKP|r",
                        ["date"] = "03/25/20",
                        ["dkpChange"] = 10,
                        ["raid"] = "Blackwing Lair",
                        ["serverTime"] = 1585191594,
                        ["officer"] = "|cffFF7D0ANeekio|r",
                    },
                    [1585173512] = {
                        ["id"] = 1585173512,
                        ["text"] = "|cff22bb33Blackwing Lair - Completion Bonus|r",
                        ["names"] = "|cffC79C6EGirlslayer|r",
                        ["reason"] = "Completion Bonus",
                        ["datetime"] = 1585173513,
                        ["dkpChangeText"] = "|cff22bb3385 DKP|r",
                        ["time"] = "02:58:33 PM",
                        ["date"] = "03/25/20",
                        ["dkpChange"] = 85,
                        ["raid"] = "Blackwing Lair",
                        ["serverTime"] = 1585173512,
                        ["officer"] = "|cff8787EDSparklenips|r",
                    },
                    [1585200546] = {
                        ["names"] = "|cffFF7D0AInebriated|r, |cffFF7D0AInsub|r, |cffABD473Stellâ|r, |cffABD473Primera|r, |cffABD473Wahcha|r, |cffABD473Ugro|r, |cff40C7EBBugaboom|r, |cff40C7EBZukohere|r, |cff40C7EBChipgizmo|r, |cff40C7EBOxford|r, |cff40C7EBEolith|r, |cff40C7EBSuprarz|r, |cffF58CBAPapasquach|r, |cffF58CBAGneissguy|r, |cffF58CBAOnes|r, |cffF58CBALawduk|r, |cffFFFFFFMongous|r, |cffFFFFFFBubblè|r, |cffFFFFFFEvangelina|r, |cffFFFFFFHoneypot|r, |cffFFFFFFRez|r, |cffFFFFFFFawntine|r, |cffFFF569Mystile|r, |cffFFF569Inigma|r, |cffFFF569Mujinn|r, |cffFFF569Tuggernuts|r, |cff8787EDThepurple|r, |cff8787EDSparklenips|r, |cff8787EDVarix|r, |cff8787EDLittledubs|r, |cff8787EDSoftfondle|r, |cffC79C6EGalagus|r, |cffC79C6EReina|r, |cffC79C6ELuckerdawg|r, |cffC79C6ENightshelf|r, |cffC79C6EJeffry|r, |cffC79C6ERetkin|r, |cffC79C6EShinynickels|r, |cffC79C6ETokentoken|r, |cffC79C6ESnaildaddy|r",
                        ["id"] = 1585200546,
                        ["text"] = "|cff22bb33Blackwing Lair - Ebonroc|r",
                        ["bossKill"] = "Ebonroc",
                        ["reason"] = "Boss Kill",
                        ["datetime"] = 1585200548,
                        ["time"] = "10:29:08 PM",
                        ["dkpChangeText"] = "|cff22bb3310 DKP|r",
                        ["date"] = "03/25/20",
                        ["dkpChange"] = 10,
                        ["raid"] = "Blackwing Lair",
                        ["serverTime"] = 1585200546,
                        ["officer"] = "|cff8787EDSparklenips|r",
                    },
                    [1585202335] = {
                        ["names"] = "|cffFF7D0AInsub|r, |cffFF7D0AInebriated|r, |cffABD473Primera|r, |cffABD473Ugro|r, |cffABD473Stellâ|r, |cffABD473Wahcha|r, |cff40C7EBOxford|r, |cff40C7EBEolith|r, |cff40C7EBSuprarz|r, |cff40C7EBZukohere|r, |cff40C7EBBugaboom|r, |cff40C7EBChipgizmo|r, |cffF58CBALawduk|r, |cffF58CBAOnes|r, |cffF58CBAGneissguy|r, |cffF58CBAPapasquach|r, |cffFFFFFFMongous|r, |cffFFFFFFBubblè|r, |cffFFFFFFEvangelina|r, |cffFFFFFFHoneypot|r, |cffFFFFFFFawntine|r, |cffFFFFFFRez|r, |cffFFF569Inigma|r, |cffFFF569Tuggernuts|r, |cffFFF569Mystile|r, |cffFFF569Mujinn|r, |cff8787EDLittledubs|r, |cff8787EDThepurple|r, |cff8787EDSoftfondle|r, |cff8787EDVarix|r, |cff8787EDSparklenips|r, |cffC79C6ESnaildaddy|r, |cffC79C6ERetkin|r, |cffC79C6ETokentoken|r, |cffC79C6ELuckerdawg|r, |cffC79C6EShinynickels|r, |cffC79C6EGalagus|r, |cffC79C6ENightshelf|r, |cffC79C6EJeffry|r, |cffC79C6EReina|r",
                        ["id"] = 1585202335,
                        ["text"] = "|cff22bb33Blackwing Lair - Nefarian|r",
                        ["bossKill"] = "Nefarian",
                        ["reason"] = "Boss Kill",
                        ["datetime"] = 1585202337,
                        ["time"] = "10:58:57 PM",
                        ["dkpChangeText"] = "|cff22bb3310 DKP|r",
                        ["date"] = "03/25/20",
                        ["dkpChange"] = 10,
                        ["raid"] = "Blackwing Lair",
                        ["serverTime"] = 1585202335,
                        ["officer"] = "|cff8787EDSparklenips|r",
                    },
                    [1585195968] = {
                        ["names"] = "|cffC79C6EReina|r",
                        ["id"] = 1585195968,
                        ["text"] = "|cffE71D36Item Win - |rNone",
                        ["dkpChangeText"] = "|cffE71D36-10 DKP|r",
                        ["reason"] = "Item Win",
                        ["datetime"] = 1585195970,
                        ["item"] = "None",
                        ["time"] = "09:12:50 PM",
                        ["date"] = "03/25/20",
                        ["dkpChange"] = -10,
                        ["raid"] = "Blackwing Lair",
                        ["serverTime"] = 1585195968,
                        ["officer"] = "|cff8787EDSparklenips|r",
                    },
                    [1585800318] = {
                        ["names"] = "|cffABD473Ithgar|r",
                        ["id"] = 1585800318,
                        ["text"] = "|cffE71D36Item Win - |rNone",
                        ["item"] = "None",
                        ["reason"] = "Item Win",
                        ["datetime"] = 1585800320,
                        ["dkpChangeText"] = "|cffE71D36-115 DKP|r",
                        ["time"] = "09:05:20 PM",
                        ["date"] = "04/01/20",
                        ["dkpChange"] = -115,
                        ["raid"] = "Blackwing Lair",
                        ["serverTime"] = 1585800318,
                        ["officer"] = "|cff8787EDSparklenips|r",
                    },
                    [1585173532] = {
                        ["id"] = 1585173532,
                        ["text"] = "|cff22bb33Blackwing Lair - Completion Bonus|r",
                        ["names"] = "|cff40C7EBEolith|r",
                        ["reason"] = "Completion Bonus",
                        ["datetime"] = 1585173533,
                        ["dkpChangeText"] = "|cff22bb33100 DKP|r",
                        ["time"] = "02:58:53 PM",
                        ["date"] = "03/25/20",
                        ["dkpChange"] = 100,
                        ["raid"] = "Blackwing Lair",
                        ["serverTime"] = 1585173532,
                        ["officer"] = "|cff8787EDSparklenips|r",
                    },
                    [1585197251] = {
                        ["reason"] = "Boss Kill",
                        ["id"] = 1585197251,
                        ["text"] = "|cff22bb33Blackwing Lair - Nefarian|r",
                        ["bossKill"] = "Nefarian",
                        ["names"] = "|cffFF7D0ANeekio|r, |cffFF7D0AMorphintyme|r, |cffFF7D0AEmmyy|r, |cffABD473Athico|r, |cffABD473Ithgar|r, |cffABD473Cyskul|r, |cff40C7EBVeriandra|r, |cff40C7EBJakemeoff|r, |cff40C7EBFlatulent|r, |cff40C7EBXyen|r, |cff40C7EBNeckbeardo|r, |cff40C7EBStitchess|r, |cffF58CBAOdin|r, |cffF58CBAAdvanty|r, |cffF58CBAForerunner|r, |cffF58CBAMaryjohanna|r, |cffFFFFFFBallour|r, |cffFFFFFFPriesticuffs|r, |cffFFFFFFPuffhead|r, |cffFFFFFFTurbohealz|r, |cffFFFFFFPhilonious|r, |cffFFF569Mcstaberson|r, |cffFFF569Mozzarella|r, |cffFFF569Slipperyjohn|r, |cffFFF569Aku|r, |cffFFF569Sicarrio|r, |cffFFF569Mariku|r, |cff8787EDEdgyboi|r, |cff8787EDRaspütin|r, |cff8787EDThenight|r, |cff8787EDDalia|r, |cff8787EDLindo|r, |cff8787EDGartog|r, |cffC79C6EKalita|r, |cffC79C6EQew|r, |cffC79C6EHotdogbroth|r, |cffC79C6EMihai|r, |cffC79C6EGirlslayer|r, |cffC79C6EOragan|r, |cffC79C6EGoobimus|r",
                        ["datetime"] = 1585197252,
                        ["dkpChangeText"] = "|cff22bb3310 DKP|r",
                        ["time"] = "12:34:12 AM",
                        ["date"] = "03/26/20",
                        ["dkpChange"] = 10,
                        ["raid"] = "Blackwing Lair",
                        ["serverTime"] = 1585197251,
                        ["officer"] = "|cffFF7D0ANeekio|r",
                    },
                    [1585193169] = {
                        ["id"] = 1585193169,
                        ["text"] = "|cff22bb33Blackwing Lair - On Time Bonus|r",
                        ["names"] = "|cffABD473Ugro|r, |cffC79C6EReina|r",
                        ["reason"] = "On Time Bonus",
                        ["datetime"] = 1585193170,
                        ["time"] = "08:26:10 PM",
                        ["dkpChangeText"] = "|cff22bb3310 DKP|r",
                        ["date"] = "03/25/20",
                        ["dkpChange"] = 10,
                        ["raid"] = "Blackwing Lair",
                        ["serverTime"] = 1585193169,
                        ["officer"] = "|cff8787EDSparklenips|r",
                    },
                    [1585173034] = {
                        ["id"] = 1585173034,
                        ["text"] = "|cff22bb33Blackwing Lair - Completion Bonus|r",
                        ["names"] = "|cffABD473Stellâ|r, |cffC79C6ETokentoken|r, |cffC79C6EGalagus|r",
                        ["reason"] = "Completion Bonus",
                        ["datetime"] = 1585173035,
                        ["dkpChangeText"] = "|cff22bb33100 DKP|r",
                        ["time"] = "02:50:35 PM",
                        ["date"] = "03/25/20",
                        ["dkpChange"] = 100,
                        ["raid"] = "Blackwing Lair",
                        ["serverTime"] = 1585173034,
                        ["officer"] = "|cff8787EDSparklenips|r",
                    },
                    [1585173548] = {
                        ["id"] = 1585173548,
                        ["text"] = "|cff22bb33Blackwing Lair - Completion Bonus|r",
                        ["names"] = "|cff40C7EBNeckbeardo|r",
                        ["reason"] = "Completion Bonus",
                        ["datetime"] = 1585173549,
                        ["dkpChangeText"] = "|cff22bb3385 DKP|r",
                        ["time"] = "02:59:09 PM",
                        ["date"] = "03/25/20",
                        ["dkpChange"] = 85,
                        ["raid"] = "Blackwing Lair",
                        ["serverTime"] = 1585173548,
                        ["officer"] = "|cff8787EDSparklenips|r",
                    },
                    [1585804191] = {
                        ["names"] = "|cffABD473Wahcha|r",
                        ["id"] = 1585804191,
                        ["text"] = "|cffE71D36Item Win - |rNone",
                        ["time"] = "10:09:51 PM",
                        ["reason"] = "Item Win",
                        ["datetime"] = 1585804191,
                        ["dkpChangeText"] = "|cffE71D36-68 DKP|r",
                        ["item"] = "None",
                        ["date"] = "04/01/20",
                        ["dkpChange"] = -68,
                        ["raid"] = "Blackwing Lair",
                        ["serverTime"] = 1585804191,
                        ["officer"] = "|cffC79C6ESnaildaddy|r",
                    },
                    [1585807486] = {
                        ["names"] = "|cffFF7D0AInebriated|r",
                        ["id"] = 1585807486,
                        ["text"] = "|cff22bb33Item Win - |rNone",
                        ["time"] = "11:04:46 PM",
                        ["reason"] = "Item Win",
                        ["datetime"] = 1585807486,
                        ["dkpChangeText"] = "|cff22bb334 DKP|r",
                        ["item"] = "None",
                        ["date"] = "04/01/20",
                        ["dkpChange"] = 4,
                        ["raid"] = "Blackwing Lair",
                        ["serverTime"] = 1585807486,
                        ["officer"] = "|cffC79C6ESnaildaddy|r",
                    },
                    [1585802139] = {
                        ["names"] = "|cffFF7D0ABlanebites|r, |cffFF7D0AInebriated|r, |cffFF7D0AKalijah|r, |cffFF7D0AInsub|r, |cffABD473Primera|r, |cffABD473Ugro|r, |cffABD473Wahcha|r, |cffABD473Stellâ|r, |cff40C7EBZukohere|r, |cff40C7EBSuprarz|r, |cff40C7EBBugaboom|r, |cff40C7EBOxford|r, |cff40C7EBChipgizmo|r, |cffF58CBALawduk|r, |cffF58CBAOnes|r, |cffF58CBAGneissguy|r, |cffFFFFFFMongous|r, |cffFFFFFFRez|r, |cffFFFFFFHoneypot|r, |cffFFFFFFFawntine|r, |cffFFFFFFEvangelina|r, |cffFFF569Trickster|r, |cffFFF569Mystile|r, |cffFFF569Aku|r, |cffFFF569Tuggernuts|r, |cffFFF569Inigma|r, |cff8787EDSoftfondle|r, |cff8787EDIhurricanel|r, |cff8787EDThepurple|r, |cff8787EDTalisker|r, |cff8787EDLittledubs|r, |cff8787EDVarix|r, |cffC79C6ERetkin|r, |cffC79C6EGalagus|r, |cffC79C6EReina|r, |cffC79C6ENightshelf|r, |cffC79C6EJeffry|r, |cffC79C6ESnaildaddy|r, |cffC79C6ETokentoken|r, |cffC79C6ELuckerdawg|r",
                        ["id"] = 1585802139,
                        ["text"] = "|cff22bb33Blackwing Lair - Firemaw|r",
                        ["bossKill"] = "Firemaw",
                        ["reason"] = "Boss Kill",
                        ["datetime"] = 1585802139,
                        ["time"] = "09:35:39 PM",
                        ["dkpChangeText"] = "|cff22bb3310 DKP|r",
                        ["date"] = "04/01/20",
                        ["dkpChange"] = 10,
                        ["raid"] = "Blackwing Lair",
                        ["serverTime"] = 1585802139,
                        ["officer"] = "|cffC79C6ESnaildaddy|r",
                    },
                    [1585799898] = {
                        ["names"] = "|cffFF7D0ABlanebites|r, |cffFF7D0AInebriated|r, |cffFF7D0AKalijah|r, |cffFF7D0AInsub|r, |cffABD473Primera|r, |cffABD473Ugro|r, |cffABD473Wahcha|r, |cffABD473Stellâ|r, |cff40C7EBZukohere|r, |cff40C7EBSuprarz|r, |cff40C7EBBugaboom|r, |cff40C7EBOxford|r, |cff40C7EBChipgizmo|r, |cffF58CBAGneissguy|r, |cffF58CBAOnes|r, |cffF58CBALawduk|r, |cffFFFFFFHoneypot|r, |cffFFFFFFRez|r, |cffFFFFFFFawntine|r, |cffFFFFFFEvangelina|r, |cffFFFFFFMongous|r, |cffFFF569Mystile|r, |cffFFF569Inigma|r, |cffFFF569Trickster|r, |cffFFF569Tuggernuts|r, |cffFFF569Aku|r, |cff8787EDLittledubs|r, |cff8787EDIhurricanel|r, |cff8787EDThepurple|r, |cff8787EDVarix|r, |cff8787EDTalisker|r, |cff8787EDSoftfondle|r, |cffC79C6ERetkin|r, |cffC79C6EReina|r, |cffC79C6EGalagus|r, |cffC79C6ETokentoken|r, |cffC79C6EJeffry|r, |cffC79C6ESnaildaddy|r, |cffC79C6ENightshelf|r, |cffC79C6ELuckerdawg|r",
                        ["id"] = 1585799898,
                        ["text"] = "|cff22bb33Blackwing Lair - Vaelastrasz the Corrupt|r",
                        ["bossKill"] = "Vaelastrasz the Corrupt",
                        ["reason"] = "Boss Kill",
                        ["datetime"] = 1585799898,
                        ["time"] = "08:58:18 PM",
                        ["dkpChangeText"] = "|cff22bb3310 DKP|r",
                        ["date"] = "04/01/20",
                        ["dkpChange"] = 10,
                        ["raid"] = "Blackwing Lair",
                        ["serverTime"] = 1585799898,
                        ["officer"] = "|cffC79C6ESnaildaddy|r",
                    },
                    [1585201618] = {
                        ["names"] = "|cffC79C6ENightshelf|r",
                        ["id"] = 1585201618,
                        ["text"] = "|cffE71D36Item Win - |rNone",
                        ["dkpChangeText"] = "|cffE71D36-5 DKP|r",
                        ["reason"] = "Item Win",
                        ["datetime"] = 1585201620,
                        ["item"] = "None",
                        ["time"] = "10:47:00 PM",
                        ["date"] = "03/25/20",
                        ["dkpChange"] = -5,
                        ["raid"] = "Blackwing Lair",
                        ["serverTime"] = 1585201618,
                        ["officer"] = "|cff8787EDSparklenips|r",
                    },
                    [1585173321] = {
                        ["id"] = 1585173321,
                        ["text"] = "|cff22bb33Blackwing Lair - Completion Bonus|r",
                        ["names"] = "|cff8787EDGartog|r, |cff8787EDDalia|r, |cff8787EDEdgyboi|r",
                        ["reason"] = "Completion Bonus",
                        ["datetime"] = 1585173322,
                        ["dkpChangeText"] = "|cff22bb33100 DKP|r",
                        ["time"] = "02:55:22 PM",
                        ["date"] = "03/25/20",
                        ["dkpChange"] = 100,
                        ["raid"] = "Blackwing Lair",
                        ["serverTime"] = 1585173321,
                        ["officer"] = "|cff8787EDSparklenips|r",
                    },
                    [1585202463] = {
                        ["names"] = "|cffFFF569Tuggernuts|r",
                        ["id"] = 1585202463,
                        ["text"] = "|cffE71D36Item Win - |rNone",
                        ["dkpChangeText"] = "|cffE71D36-90 DKP|r",
                        ["reason"] = "Item Win",
                        ["datetime"] = 1585202465,
                        ["item"] = "None",
                        ["time"] = "11:01:05 PM",
                        ["date"] = "03/25/20",
                        ["dkpChange"] = -90,
                        ["raid"] = "Blackwing Lair",
                        ["serverTime"] = 1585202463,
                        ["officer"] = "|cff8787EDSparklenips|r",
                    },
                    [1585797565] = {
                        ["names"] = "|cffC79C6EGirlslayer|r",
                        ["id"] = 1585797565,
                        ["text"] = "|cffE71D36Item Win - |rNone",
                        ["item"] = "None",
                        ["reason"] = "Item Win",
                        ["datetime"] = 1585797567,
                        ["dkpChangeText"] = "|cffE71D36-123 DKP|r",
                        ["time"] = "08:19:27 PM",
                        ["date"] = "04/01/20",
                        ["dkpChange"] = -123,
                        ["raid"] = "Blackwing Lair",
                        ["serverTime"] = 1585797565,
                        ["officer"] = "|cff8787EDSparklenips|r",
                    },
                    [1585195514] = {
                        ["names"] = "|cffF58CBAMaryjohanna|r",
                        ["id"] = 1585195514,
                        ["text"] = "|cffE71D36Item Win - |r|cffa335ee|Hitem:19357::::::::60:::::::|h[Herald of Woe]|h|r",
                        ["dkpChangeText"] = "|cffE71D36-11 DKP|r",
                        ["reason"] = "Item Win",
                        ["datetime"] = 1585195515,
                        ["time"] = "12:05:15 AM",
                        ["item"] = "|cffa335ee|Hitem:19357::::::::60:::::::|h[Herald of Woe]|h|r",
                        ["date"] = "03/26/20",
                        ["dkpChange"] = -11,
                        ["raid"] = "Blackwing Lair",
                        ["serverTime"] = 1585195514,
                        ["officer"] = "|cffFF7D0ANeekio|r",
                    },
                    [1585187103] = {
                        ["reason"] = "Boss Kill",
                        ["id"] = 1585187103,
                        ["text"] = "|cff22bb33Blackwing Lair - Razorgore the Untamed|r",
                        ["bossKill"] = "Razorgore the Untamed",
                        ["names"] = "|cffFF7D0AMorphintyme|r, |cffFF7D0AEmmyy|r, |cffFF7D0ANeekio|r, |cffABD473Cyskul|r, |cffABD473Athico|r, |cffABD473Ithgar|r, |cff40C7EBVeriandra|r, |cff40C7EBJakemeoff|r, |cff40C7EBNeckbeardo|r, |cff40C7EBFlatulent|r, |cff40C7EBStitchess|r, |cff40C7EBXyen|r, |cffF58CBAAdvanty|r, |cffF58CBAOdin|r, |cffF58CBAMaryjohanna|r, |cffF58CBAForerunner|r, |cffFFFFFFPhilonious|r, |cffFFFFFFBallour|r, |cffFFFFFFPriesticuffs|r, |cffFFFFFFPuffhead|r, |cffFFFFFFTurbohealz|r, |cffFFF569Slipperyjohn|r, |cffFFF569Mariku|r, |cffFFF569Mcstaberson|r, |cffFFF569Aku|r, |cffFFF569Sicarrio|r, |cffFFF569Mozzarella|r, |cff8787EDEdgyboi|r, |cff8787EDThenight|r, |cff8787EDLindo|r, |cff8787EDDalia|r, |cff8787EDRaspütin|r, |cff8787EDGartog|r, |cffC79C6EKalita|r, |cffC79C6EGirlslayer|r, |cffC79C6EQew|r, |cffC79C6EMihai|r, |cffC79C6EHotdogbroth|r, |cffC79C6EOragan|r, |cffC79C6EGoobimus|r",
                        ["datetime"] = 1585187104,
                        ["dkpChangeText"] = "|cff22bb3310 DKP|r",
                        ["time"] = "09:45:04 PM",
                        ["date"] = "03/25/20",
                        ["dkpChange"] = 10,
                        ["raid"] = "Blackwing Lair",
                        ["serverTime"] = 1585187103,
                        ["officer"] = "|cffFF7D0ANeekio|r",
                    },
                    [1585198582] = {
                        ["names"] = "|cffABD473Stellâ|r",
                        ["id"] = 1585198582,
                        ["text"] = "|cffE71D36Item Win - |rNone",
                        ["dkpChangeText"] = "|cffE71D36-10 DKP|r",
                        ["reason"] = "Item Win",
                        ["datetime"] = 1585198584,
                        ["item"] = "None",
                        ["time"] = "09:56:24 PM",
                        ["date"] = "03/25/20",
                        ["dkpChange"] = -10,
                        ["raid"] = "Blackwing Lair",
                        ["serverTime"] = 1585198582,
                        ["officer"] = "|cff8787EDSparklenips|r",
                    },
                    [1585173067] = {
                        ["id"] = 1585173067,
                        ["text"] = "|cff22bb33Blackwing Lair - Completion Bonus|r",
                        ["names"] = "|cffC79C6ERetkin|r, |cffC79C6ESnaildaddy|r, |cffC79C6EGoobimus|r",
                        ["reason"] = "Completion Bonus",
                        ["datetime"] = 1585173068,
                        ["dkpChangeText"] = "|cff22bb33100 DKP|r",
                        ["time"] = "02:51:08 PM",
                        ["date"] = "03/25/20",
                        ["dkpChange"] = 100,
                        ["raid"] = "Blackwing Lair",
                        ["serverTime"] = 1585173067,
                        ["officer"] = "|cff8787EDSparklenips|r",
                    },
                    [1585792238] = {
                        ["names"] = "|cffC79C6EQew|r",
                        ["id"] = 1585792238,
                        ["text"] = "|cffE71D36Item Win - |rNone",
                        ["item"] = "None",
                        ["reason"] = "Item Win",
                        ["datetime"] = 1585792240,
                        ["dkpChangeText"] = "|cffE71D36-11 DKP|r",
                        ["time"] = "06:50:40 PM",
                        ["date"] = "04/01/20",
                        ["dkpChange"] = -11,
                        ["raid"] = "Blackwing Lair",
                        ["serverTime"] = 1585792238,
                        ["officer"] = "|cff8787EDSparklenips|r",
                    },
                    [1585693334] = {
                        ["id"] = 1585693334,
                        ["text"] = "|cff22bb33Molten Core - On Time Bonus|r",
                        ["names"] = "|cff40C7EBUrkh|r",
                        ["reason"] = "On Time Bonus",
                        ["datetime"] = 1585693335,
                        ["time"] = "03:22:15 PM",
                        ["dkpChangeText"] = "|cff22bb3310 DKP|r",
                        ["date"] = "03/31/20",
                        ["dkpChange"] = 10,
                        ["raid"] = "Molten Core",
                        ["serverTime"] = 1585693334,
                        ["officer"] = "|cff8787EDSparklenips|r",
                    },
                    [1585173098] = {
                        ["id"] = 1585173098,
                        ["text"] = "|cff22bb33Blackwing Lair - Completion Bonus|r",
                        ["names"] = "|cffFFFFFFRez|r",
                        ["reason"] = "Completion Bonus",
                        ["datetime"] = 1585173099,
                        ["dkpChangeText"] = "|cff22bb3385 DKP|r",
                        ["time"] = "02:51:39 PM",
                        ["date"] = "03/25/20",
                        ["dkpChange"] = 85,
                        ["raid"] = "Blackwing Lair",
                        ["serverTime"] = 1585173098,
                        ["officer"] = "|cff8787EDSparklenips|r",
                    },
                    [1585173364] = {
                        ["id"] = 1585173364,
                        ["text"] = "|cff22bb33Blackwing Lair - Completion Bonus|r",
                        ["names"] = "|cffFF7D0AJunglemain|r",
                        ["reason"] = "Completion Bonus",
                        ["datetime"] = 1585173365,
                        ["dkpChangeText"] = "|cff22bb3372 DKP|r",
                        ["time"] = "02:56:05 PM",
                        ["date"] = "03/25/20",
                        ["dkpChange"] = 72,
                        ["raid"] = "Blackwing Lair",
                        ["serverTime"] = 1585173364,
                        ["officer"] = "|cff8787EDSparklenips|r",
                    },
                    [1585797601] = {
                        ["id"] = 1585797601,
                        ["text"] = "|cffE71D36Other - Correction|r",
                        ["reason"] = "Other",
                        ["names"] = "|cffF58CBALawduk|r",
                        ["datetime"] = 1585797602,
                        ["time"] = "11:20:02 PM",
                        ["dkpChangeText"] = "|cffE71D36-60 DKP|r",
                        ["date"] = "04/01/20",
                        ["dkpChange"] = -60,
                        ["raid"] = "Blackwing Lair",
                        ["serverTime"] = 1585797601,
                        ["officer"] = "|cffFF7D0ANeekio|r",
                    },
                    [1585802223] = {
                        ["names"] = "|cffFF7D0AKalijah|r",
                        ["id"] = 1585802223,
                        ["text"] = "|cffE71D36Item Win - |rNone",
                        ["time"] = "09:37:03 PM",
                        ["reason"] = "Item Win",
                        ["datetime"] = 1585802223,
                        ["dkpChangeText"] = "|cffE71D36-5 DKP|r",
                        ["item"] = "None",
                        ["date"] = "04/01/20",
                        ["dkpChange"] = -5,
                        ["raid"] = "Blackwing Lair",
                        ["serverTime"] = 1585802223,
                        ["officer"] = "|cffC79C6ESnaildaddy|r",
                    },
                    [1585173373] = {
                        ["id"] = 1585173373,
                        ["text"] = "|cff22bb33Blackwing Lair - Completion Bonus|r",
                        ["names"] = "|cffFF7D0ANeekio|r",
                        ["reason"] = "Completion Bonus",
                        ["datetime"] = 1585173374,
                        ["dkpChangeText"] = "|cff22bb33100 DKP|r",
                        ["time"] = "02:56:14 PM",
                        ["date"] = "03/25/20",
                        ["dkpChange"] = 100,
                        ["raid"] = "Blackwing Lair",
                        ["serverTime"] = 1585173373,
                        ["officer"] = "|cff8787EDSparklenips|r",
                    },
                    [1585800418] = {
                        ["names"] = "|cffFFFFFFPuffhead|r",
                        ["id"] = 1585800418,
                        ["text"] = "|cffE71D36Item Win - |rNone",
                        ["item"] = "None",
                        ["reason"] = "Item Win",
                        ["datetime"] = 1585800420,
                        ["dkpChangeText"] = "|cffE71D36-150 DKP|r",
                        ["time"] = "09:07:00 PM",
                        ["date"] = "04/01/20",
                        ["dkpChange"] = -150,
                        ["raid"] = "Blackwing Lair",
                        ["serverTime"] = 1585800418,
                        ["officer"] = "|cff8787EDSparklenips|r",
                    },
                    [1585805028] = {
                        ["names"] = "|cffABD473Stellâ|r",
                        ["id"] = 1585805028,
                        ["text"] = "|cffE71D36Item Win - |rNone",
                        ["time"] = "10:23:48 PM",
                        ["reason"] = "Item Win",
                        ["datetime"] = 1585805028,
                        ["dkpChangeText"] = "|cffE71D36-75 DKP|r",
                        ["item"] = "None",
                        ["date"] = "04/01/20",
                        ["dkpChange"] = -75,
                        ["raid"] = "Blackwing Lair",
                        ["serverTime"] = 1585805028,
                        ["officer"] = "|cffC79C6ESnaildaddy|r",
                    },
                    [1585795087] = {
                        ["names"] = "|cffFF7D0AEmmyy|r, |cffFF7D0AMorphintyme|r, |cffFF7D0ANeekio|r, |cffABD473Athico|r, |cffABD473Bádtothebow|r, |cffABD473Ithgar|r, |cffABD473Bigbootyhoho|r, |cffABD473Cyskul|r, |cff40C7EBEolith|r, |cff40C7EBJakemeoff|r, |cff40C7EBVeriandra|r, |cff40C7EBFlatulent|r, |cff40C7EBStitchess|r, |cff40C7EBNeckbeardo|r, |cff40C7EBXyen|r, |cffF58CBAPapasquach|r, |cffF58CBAAdvanty|r, |cffF58CBAOdin|r, |cffF58CBAMaryjohanna|r, |cffFFFFFFBallour|r, |cffFFFFFFPuffhead|r, |cffFFFFFFPriesticuffs|r, |cffFFFFFFTurbohealz|r, |cffFFFFFFPhilonious|r, |cffFFF569Mariku|r, |cffFFF569Mcstaberson|r, |cffFFF569Sicarrio|r, |cffFFF569Slipperyjohn|r, |cff8787EDEdgyboi|r, |cff8787EDRaspütin|r, |cff8787EDSparklenips|r, |cff8787EDLindo|r, |cff8787EDDalia|r, |cffC79C6EGoobimus|r, |cffC79C6EKalita|r, |cffC79C6EHotdogbroth|r, |cffC79C6EShinynickels|r, |cffC79C6EOragan|r, |cffC79C6EQew|r, |cffC79C6EGirlslayer|r",
                        ["id"] = 1585795087,
                        ["text"] = "|cff22bb33Blackwing Lair - Firemaw|r",
                        ["bossKill"] = "Firemaw",
                        ["reason"] = "Boss Kill",
                        ["datetime"] = 1585795089,
                        ["time"] = "07:38:09 PM",
                        ["dkpChangeText"] = "|cff22bb3310 DKP|r",
                        ["date"] = "04/01/20",
                        ["dkpChange"] = 10,
                        ["raid"] = "Blackwing Lair",
                        ["serverTime"] = 1585795087,
                        ["officer"] = "|cff8787EDSparklenips|r",
                    },
                    [1585804287] = {
                        ["names"] = "|cffFF7D0ABlanebites|r",
                        ["id"] = 1585804287,
                        ["text"] = "|cffE71D36Item Win - |rNone",
                        ["time"] = "10:11:27 PM",
                        ["reason"] = "Item Win",
                        ["datetime"] = 1585804287,
                        ["dkpChangeText"] = "|cffE71D36-51 DKP|r",
                        ["item"] = "None",
                        ["date"] = "04/01/20",
                        ["dkpChange"] = -51,
                        ["raid"] = "Blackwing Lair",
                        ["serverTime"] = 1585804287,
                        ["officer"] = "|cffC79C6ESnaildaddy|r",
                    },
                    [1585187159] = {
                        ["reason"] = "Item Win",
                        ["id"] = 1585187159,
                        ["text"] = "|cffE71D36Item Win - |r|cffa335ee|Hitem:16911::::::::60:::::::|h[Bloodfang Bracers]|h|r",
                        ["item"] = "|cffa335ee|Hitem:16911::::::::60:::::::|h[Bloodfang Bracers]|h|r",
                        ["names"] = "|cffFFF569Mariku|r",
                        ["datetime"] = 1585187160,
                        ["dkpChangeText"] = "|cffE71D36-60 DKP|r",
                        ["time"] = "09:46:00 PM",
                        ["date"] = "03/25/20",
                        ["dkpChange"] = -60,
                        ["raid"] = "Blackwing Lair",
                        ["serverTime"] = 1585187159,
                        ["officer"] = "|cffFF7D0ANeekio|r",
                    },
                    [1585197363] = {
                        ["reason"] = "Item Win",
                        ["id"] = 1585197363,
                        ["text"] = "|cffE71D36Item Win - |r|cffa335ee|Hitem:16966::::::::60:::::::|h[Breastplate of Wrath]|h|r",
                        ["time"] = "12:36:04 AM",
                        ["names"] = "|cffC79C6EOragan|r",
                        ["datetime"] = 1585197364,
                        ["item"] = "|cffa335ee|Hitem:16966::::::::60:::::::|h[Breastplate of Wrath]|h|r",
                        ["dkpChangeText"] = "|cffE71D36-54 DKP|r",
                        ["date"] = "03/26/20",
                        ["dkpChange"] = -54,
                        ["raid"] = "Blackwing Lair",
                        ["serverTime"] = 1585197363,
                        ["officer"] = "|cffFF7D0ANeekio|r",
                    },
                    [1585689772] = {
                        ["id"] = 1585689772,
                        ["text"] = "|cff22bb33Molten Core - On Time Bonus|r",
                        ["reason"] = "On Time Bonus",
                        ["names"] = "|cff40C7EBUrkh|r",
                        ["datetime"] = 1585689773,
                        ["dkpChangeText"] = "|cff22bb3310 DKP|r",
                        ["time"] = "02:22:53 PM",
                        ["date"] = "03/31/20",
                        ["dkpChange"] = 10,
                        ["raid"] = "Molten Core",
                        ["serverTime"] = 1585689772,
                        ["officer"] = "|cff8787EDSparklenips|r",
                    },
                    [1585195347] = {
                        ["names"] = "|cffFF7D0ANeekio|r, |cffFF7D0AEmmyy|r, |cffFF7D0AMorphintyme|r, |cffABD473Ithgar|r, |cffABD473Cyskul|r, |cffABD473Athico|r, |cff40C7EBNeckbeardo|r, |cff40C7EBVeriandra|r, |cff40C7EBFlatulent|r, |cff40C7EBXyen|r, |cff40C7EBStitchess|r, |cff40C7EBJakemeoff|r, |cffF58CBAOdin|r, |cffF58CBAAdvanty|r, |cffF58CBAMaryjohanna|r, |cffF58CBAForerunner|r, |cffFFFFFFPhilonious|r, |cffFFFFFFTurbohealz|r, |cffFFFFFFBallour|r, |cffFFFFFFPuffhead|r, |cffFFFFFFPriesticuffs|r, |cffFFF569Sicarrio|r, |cffFFF569Aku|r, |cffFFF569Slipperyjohn|r, |cffFFF569Mariku|r, |cffFFF569Mcstaberson|r, |cffFFF569Mozzarella|r, |cff8787EDLindo|r, |cff8787EDThenight|r, |cff8787EDEdgyboi|r, |cff8787EDRaspütin|r, |cff8787EDDalia|r, |cff8787EDGartog|r, |cffC79C6EOragan|r, |cffC79C6EKalita|r, |cffC79C6EGirlslayer|r, |cffC79C6EHotdogbroth|r, |cffC79C6EGoobimus|r, |cffC79C6EQew|r, |cffC79C6EMihai|r",
                        ["id"] = 1585195347,
                        ["text"] = "|cff22bb33Blackwing Lair - Flamegor|r",
                        ["bossKill"] = "Flamegor",
                        ["reason"] = "Boss Kill",
                        ["datetime"] = 1585195348,
                        ["time"] = "12:02:28 AM",
                        ["dkpChangeText"] = "|cff22bb3310 DKP|r",
                        ["date"] = "03/26/20",
                        ["dkpChange"] = 10,
                        ["raid"] = "Blackwing Lair",
                        ["serverTime"] = 1585195347,
                        ["officer"] = "|cffFF7D0ANeekio|r",
                    },
                    [1585804020] = {
                        ["names"] = "|cffF58CBADolamroth|r",
                        ["id"] = 1585804020,
                        ["text"] = "|cff22bb33Blackwing Lair - Ebonroc|r",
                        ["bossKill"] = "Ebonroc",
                        ["reason"] = "Boss Kill",
                        ["datetime"] = 1585804020,
                        ["time"] = "10:07:00 PM",
                        ["dkpChangeText"] = "|cff22bb3310 DKP|r",
                        ["date"] = "04/01/20",
                        ["dkpChange"] = 10,
                        ["raid"] = "Blackwing Lair",
                        ["serverTime"] = 1585804020,
                        ["officer"] = "|cffC79C6ESnaildaddy|r",
                    },
                    [1585798183] = {
                        ["id"] = 1585798183,
                        ["text"] = "|cffE71D36Blackwing Lair - On Time Bonus|r",
                        ["reason"] = "On Time Bonus",
                        ["names"] = "|cffFFF569Mystile|r",
                        ["datetime"] = 1585798183,
                        ["dkpChangeText"] = "|cffE71D36-10 DKP|r",
                        ["time"] = "08:29:43 PM",
                        ["date"] = "04/01/20",
                        ["dkpChange"] = -10,
                        ["raid"] = "Blackwing Lair",
                        ["serverTime"] = 1585798183,
                        ["officer"] = "|cffC79C6ESnaildaddy|r",
                    },
                    [1585173417] = {
                        ["id"] = 1585173417,
                        ["text"] = "|cff22bb33Blackwing Lair - Completion Bonus|r",
                        ["names"] = "|cffABD473Bádtothebow|r",
                        ["reason"] = "Completion Bonus",
                        ["datetime"] = 1585173418,
                        ["dkpChangeText"] = "|cff22bb3390 DKP|r",
                        ["time"] = "02:56:58 PM",
                        ["date"] = "03/25/20",
                        ["dkpChange"] = 90,
                        ["raid"] = "Blackwing Lair",
                        ["serverTime"] = 1585173417,
                        ["officer"] = "|cff8787EDSparklenips|r",
                    },
                    [1585188462] = {
                        ["id"] = 1585188462,
                        ["text"] = "|cff22bb33Other - Correction|r",
                        ["reason"] = "Other",
                        ["names"] = "|cffABD473Ithgar|r",
                        ["datetime"] = 1585188463,
                        ["time"] = "10:07:43 PM",
                        ["dkpChangeText"] = "|cff22bb3310 DKP|r",
                        ["date"] = "03/25/20",
                        ["dkpChange"] = 10,
                        ["raid"] = "Blackwing Lair",
                        ["serverTime"] = 1585188462,
                        ["officer"] = "|cffFF7D0ANeekio|r",
                    },
                    [1585173424] = {
                        ["id"] = 1585173424,
                        ["text"] = "|cff22bb33Blackwing Lair - Completion Bonus|r",
                        ["names"] = "|cffABD473Cyskul|r",
                        ["reason"] = "Completion Bonus",
                        ["datetime"] = 1585173425,
                        ["dkpChangeText"] = "|cff22bb3392 DKP|r",
                        ["time"] = "02:57:05 PM",
                        ["date"] = "03/25/20",
                        ["dkpChange"] = 92,
                        ["raid"] = "Blackwing Lair",
                        ["serverTime"] = 1585173424,
                        ["officer"] = "|cff8787EDSparklenips|r",
                    },
                    [1585197680] = {
                        ["reason"] = "Item Win",
                        ["id"] = 1585197680,
                        ["text"] = "|cffE71D36Item Win - |r|cffa335ee|Hitem:19003::::::::60:::::::|h[Head of Nefarian]|h|r",
                        ["time"] = "12:41:21 AM",
                        ["names"] = "|cffFFF569Mcstaberson|r",
                        ["datetime"] = 1585197681,
                        ["item"] = "|cffa335ee|Hitem:19003::::::::60:::::::|h[Head of Nefarian]|h|r",
                        ["dkpChangeText"] = "|cffE71D36-100 DKP|r",
                        ["date"] = "03/26/20",
                        ["dkpChange"] = -100,
                        ["raid"] = "Blackwing Lair",
                        ["serverTime"] = 1585197680,
                        ["officer"] = "|cffFF7D0ANeekio|r",
                    },
                    [1585195920] = {
                        ["names"] = "|cffFF7D0AInsub|r, |cffFF7D0AInebriated|r, |cffABD473Primera|r, |cffABD473Ugro|r, |cffABD473Stellâ|r, |cffABD473Wahcha|r, |cff40C7EBOxford|r, |cff40C7EBEolith|r, |cff40C7EBSuprarz|r, |cff40C7EBZukohere|r, |cff40C7EBBugaboom|r, |cff40C7EBChipgizmo|r, |cffF58CBAPapasquach|r, |cffF58CBAGneissguy|r, |cffF58CBAOnes|r, |cffF58CBALawduk|r, |cffFFFFFFMongous|r, |cffFFFFFFBubblè|r, |cffFFFFFFEvangelina|r, |cffFFFFFFHoneypot|r, |cffFFFFFFFawntine|r, |cffFFFFFFRez|r, |cffFFF569Inigma|r, |cffFFF569Tuggernuts|r, |cffFFF569Mystile|r, |cffFFF569Mujinn|r, |cff8787EDLittledubs|r, |cff8787EDThepurple|r, |cff8787EDSoftfondle|r, |cff8787EDVarix|r, |cff8787EDSparklenips|r, |cffC79C6ESnaildaddy|r, |cffC79C6ERetkin|r, |cffC79C6ETokentoken|r, |cffC79C6ELuckerdawg|r, |cffC79C6EShinynickels|r, |cffC79C6EGalagus|r, |cffC79C6ENightshelf|r, |cffC79C6EJeffry|r, |cffC79C6EReina|r",
                        ["id"] = 1585195920,
                        ["text"] = "|cff22bb33Blackwing Lair - Vaelastrasz the Corrupt|r",
                        ["bossKill"] = "Vaelastrasz the Corrupt",
                        ["reason"] = "Boss Kill",
                        ["datetime"] = 1585195921,
                        ["time"] = "09:12:01 PM",
                        ["dkpChangeText"] = "|cff22bb3310 DKP|r",
                        ["date"] = "03/25/20",
                        ["dkpChange"] = 10,
                        ["raid"] = "Blackwing Lair",
                        ["serverTime"] = 1585195920,
                        ["officer"] = "|cff8787EDSparklenips|r",
                    },
                    [1585173178] = {
                        ["id"] = 1585173178,
                        ["text"] = "|cff22bb33Blackwing Lair - Completion Bonus|r",
                        ["names"] = "|cffF58CBALawduk|r",
                        ["reason"] = "Completion Bonus",
                        ["datetime"] = 1585173179,
                        ["dkpChangeText"] = "|cff22bb33100 DKP|r",
                        ["time"] = "02:52:59 PM",
                        ["date"] = "03/25/20",
                        ["dkpChange"] = 100,
                        ["raid"] = "Blackwing Lair",
                        ["serverTime"] = 1585173178,
                        ["officer"] = "|cff8787EDSparklenips|r",
                    },
                    [1585173692] = {
                        ["id"] = 1585173692,
                        ["text"] = "|cff22bb33Blackwing Lair - Completion Bonus|r",
                        ["names"] = "|cffFFF569Slipperyjohn|r",
                        ["reason"] = "Completion Bonus",
                        ["datetime"] = 1585173693,
                        ["dkpChangeText"] = "|cff22bb3355 DKP|r",
                        ["time"] = "03:01:33 PM",
                        ["date"] = "03/25/20",
                        ["dkpChange"] = 55,
                        ["raid"] = "Blackwing Lair",
                        ["serverTime"] = 1585173692,
                        ["officer"] = "|cff8787EDSparklenips|r",
                    },
                    [1585173186] = {
                        ["id"] = 1585173186,
                        ["text"] = "|cff22bb33Blackwing Lair - Completion Bonus|r",
                        ["names"] = "|cffF58CBAOnes|r",
                        ["reason"] = "Completion Bonus",
                        ["datetime"] = 1585173187,
                        ["dkpChangeText"] = "|cff22bb3392 DKP|r",
                        ["time"] = "02:53:07 PM",
                        ["date"] = "03/25/20",
                        ["dkpChange"] = 92,
                        ["raid"] = "Blackwing Lair",
                        ["serverTime"] = 1585173186,
                        ["officer"] = "|cff8787EDSparklenips|r",
                    },
                    [1585195630] = {
                        ["names"] = "|cffC79C6EShinynickels|r",
                        ["id"] = 1585195630,
                        ["text"] = "|cffE71D36Item Win - |rNone",
                        ["dkpChangeText"] = "|cffE71D36-8 DKP|r",
                        ["reason"] = "Item Win",
                        ["datetime"] = 1585195631,
                        ["item"] = "None",
                        ["time"] = "09:07:11 PM",
                        ["date"] = "03/25/20",
                        ["dkpChange"] = -8,
                        ["raid"] = "Blackwing Lair",
                        ["serverTime"] = 1585195630,
                        ["officer"] = "|cff8787EDSparklenips|r",
                    },
                    [1585173194] = {
                        ["id"] = 1585173194,
                        ["text"] = "|cff22bb33Blackwing Lair - Completion Bonus|r",
                        ["names"] = "|cffF58CBAForerunner|r",
                        ["reason"] = "Completion Bonus",
                        ["datetime"] = 1585173195,
                        ["dkpChangeText"] = "|cff22bb3350 DKP|r",
                        ["time"] = "02:53:15 PM",
                        ["date"] = "03/25/20",
                        ["dkpChange"] = 50,
                        ["raid"] = "Blackwing Lair",
                        ["serverTime"] = 1585173194,
                        ["officer"] = "|cff8787EDSparklenips|r",
                    },
                    [1585172943] = {
                        ["id"] = 1585172943,
                        ["text"] = "|cff22bb33Blackwing Lair - Completion Bonus|r",
                        ["names"] = "|cff8787EDThepurple|r",
                        ["reason"] = "Completion Bonus",
                        ["datetime"] = 1585172944,
                        ["dkpChangeText"] = "|cff22bb3365 DKP|r",
                        ["time"] = "02:49:04 PM",
                        ["date"] = "03/25/20",
                        ["dkpChange"] = 65,
                        ["raid"] = "Blackwing Lair",
                        ["serverTime"] = 1585172943,
                        ["officer"] = "|cff8787EDSparklenips|r",
                    },
                    [1585192008] = {
                        ["id"] = 1585192008,
                        ["text"] = "|cff22bb33Blackwing Lair - On Time Bonus|r",
                        ["names"] = "|cffFF7D0AInsub|r, |cffFF7D0AInebriated|r, |cffABD473Wahcha|r, |cffABD473Primera|r, |cffABD473Stellâ|r, |cff40C7EBEolith|r, |cff40C7EBSuprarz|r, |cff40C7EBOxford|r, |cff40C7EBBugaboom|r, |cff40C7EBZukohere|r, |cff40C7EBChipgizmo|r, |cffF58CBAPapasquach|r, |cffF58CBAOnes|r, |cffF58CBALawduk|r, |cffF58CBAGneissguy|r, |cffFFFFFFBubblè|r, |cffFFFFFFMongous|r, |cffFFFFFFEvangelina|r, |cffFFFFFFHoneypot|r, |cffFFFFFFFawntine|r, |cffFFFFFFRez|r, |cffFFF569Mystile|r, |cffFFF569Inigma|r, |cffFFF569Mujinn|r, |cffFFF569Tuggernuts|r, |cff8787EDLittledubs|r, |cff8787EDSoftfondle|r, |cff8787EDThepurple|r, |cff8787EDVarix|r, |cff8787EDSparklenips|r, |cffC79C6ENightshelf|r, |cffC79C6ELuckerdawg|r, |cffC79C6EShinynickels|r, |cffC79C6EJeffry|r, |cffC79C6ETokentoken|r, |cffC79C6ERetkin|r, |cffC79C6ESnaildaddy|r",
                        ["reason"] = "On Time Bonus",
                        ["datetime"] = 1585192009,
                        ["time"] = "08:06:49 PM",
                        ["dkpChangeText"] = "|cff22bb3310 DKP|r",
                        ["date"] = "03/25/20",
                        ["dkpChange"] = 10,
                        ["raid"] = "Blackwing Lair",
                        ["serverTime"] = 1585192008,
                        ["officer"] = "|cff8787EDSparklenips|r",
                    },
                    [1585173465] = {
                        ["id"] = 1585173465,
                        ["text"] = "|cff22bb33Blackwing Lair - Completion Bonus|r",
                        ["names"] = "|cffFFFFFFTurbohealz|r",
                        ["reason"] = "Completion Bonus",
                        ["datetime"] = 1585173466,
                        ["dkpChangeText"] = "|cff22bb3375 DKP|r",
                        ["time"] = "02:57:46 PM",
                        ["date"] = "03/25/20",
                        ["dkpChange"] = 75,
                        ["raid"] = "Blackwing Lair",
                        ["serverTime"] = 1585173465,
                        ["officer"] = "|cff8787EDSparklenips|r",
                    },
                    [1585804004] = {
                        ["names"] = "|cffFF7D0ABlanebites|r, |cffFF7D0AInebriated|r, |cffFF7D0AKalijah|r, |cffFF7D0AInsub|r, |cffABD473Primera|r, |cffABD473Ugro|r, |cffABD473Wahcha|r, |cffABD473Stellâ|r, |cff40C7EBZukohere|r, |cff40C7EBSuprarz|r, |cff40C7EBBugaboom|r, |cff40C7EBOxford|r, |cff40C7EBChipgizmo|r, |cffF58CBALawduk|r, |cffF58CBAOnes|r, |cffF58CBAGneissguy|r, |cffFFFFFFMongous|r, |cffFFFFFFRez|r, |cffFFFFFFHoneypot|r, |cffFFFFFFFawntine|r, |cffFFFFFFEvangelina|r, |cffFFF569Trickster|r, |cffFFF569Mystile|r, |cffFFF569Aku|r, |cffFFF569Tuggernuts|r, |cffFFF569Inigma|r, |cff8787EDSoftfondle|r, |cff8787EDIhurricanel|r, |cff8787EDThepurple|r, |cff8787EDTalisker|r, |cff8787EDLittledubs|r, |cff8787EDVarix|r, |cffC79C6ERetkin|r, |cffC79C6EGalagus|r, |cffC79C6EReina|r, |cffC79C6ENightshelf|r, |cffC79C6EJeffry|r, |cffC79C6ESnaildaddy|r, |cffC79C6ETokentoken|r, |cffC79C6ELuckerdawg|r",
                        ["id"] = 1585804004,
                        ["text"] = "|cff22bb33Blackwing Lair - Ebonroc|r",
                        ["bossKill"] = "Ebonroc",
                        ["reason"] = "Boss Kill",
                        ["datetime"] = 1585804004,
                        ["time"] = "10:06:44 PM",
                        ["dkpChangeText"] = "|cff22bb3310 DKP|r",
                        ["date"] = "04/01/20",
                        ["dkpChange"] = 10,
                        ["raid"] = "Blackwing Lair",
                        ["serverTime"] = 1585804004,
                        ["officer"] = "|cffC79C6ESnaildaddy|r",
                    },
                    [1585172434] = {
                        ["id"] = 1585172434,
                        ["text"] = "|cff22bb33Molten Core - Completion Bonus|r",
                        ["names"] = "|cffFFF569Pantheonbank|r",
                        ["reason"] = "Completion Bonus",
                        ["datetime"] = 1585172435,
                        ["dkpChangeText"] = "|cff22bb3310 DKP|r",
                        ["time"] = "02:40:35 PM",
                        ["date"] = "03/25/20",
                        ["dkpChange"] = 10,
                        ["raid"] = "Molten Core",
                        ["serverTime"] = 1585172434,
                        ["officer"] = "|cff8787EDSparklenips|r",
                    },
                    [1585172963] = {
                        ["id"] = 1585172963,
                        ["text"] = "|cff22bb33Blackwing Lair - Completion Bonus|r",
                        ["names"] = "|cff8787EDThenight|r",
                        ["reason"] = "Completion Bonus",
                        ["datetime"] = 1585172964,
                        ["dkpChangeText"] = "|cff22bb33100 DKP|r",
                        ["time"] = "02:49:24 PM",
                        ["date"] = "03/25/20",
                        ["dkpChange"] = 100,
                        ["raid"] = "Blackwing Lair",
                        ["serverTime"] = 1585172963,
                        ["officer"] = "|cff8787EDSparklenips|r",
                    },
                    [1585797824] = {
                        ["names"] = "|cff40C7EBEolith|r",
                        ["id"] = 1585797824,
                        ["text"] = "|cffE71D36Item Win - |rNone",
                        ["item"] = "None",
                        ["reason"] = "Item Win",
                        ["datetime"] = 1585797826,
                        ["dkpChangeText"] = "|cffE71D36-93 DKP|r",
                        ["time"] = "08:23:46 PM",
                        ["date"] = "04/01/20",
                        ["dkpChange"] = -93,
                        ["raid"] = "Blackwing Lair",
                        ["serverTime"] = 1585797824,
                        ["officer"] = "|cff8787EDSparklenips|r",
                    },
                    [1585200766] = {
                        ["names"] = "|cff40C7EBSuprarz|r",
                        ["id"] = 1585200766,
                        ["text"] = "|cff22bb33Blackwing Lair - Flamegor|r",
                        ["bossKill"] = "Flamegor",
                        ["reason"] = "Boss Kill",
                        ["datetime"] = 1585200768,
                        ["time"] = "10:32:48 PM",
                        ["dkpChangeText"] = "|cff22bb3310 DKP|r",
                        ["date"] = "03/25/20",
                        ["dkpChange"] = 10,
                        ["raid"] = "Blackwing Lair",
                        ["serverTime"] = 1585200766,
                        ["officer"] = "|cff8787EDSparklenips|r",
                    },
                    [1585172975] = {
                        ["id"] = 1585172975,
                        ["text"] = "|cff22bb33Blackwing Lair - Completion Bonus|r",
                        ["names"] = "|cff8787EDIhurricanel|r",
                        ["reason"] = "Completion Bonus",
                        ["datetime"] = 1585172976,
                        ["dkpChangeText"] = "|cff22bb33100 DKP|r",
                        ["time"] = "02:49:36 PM",
                        ["date"] = "03/25/20",
                        ["dkpChange"] = 100,
                        ["raid"] = "Blackwing Lair",
                        ["serverTime"] = 1585172975,
                        ["officer"] = "|cff8787EDSparklenips|r",
                    },
                    [1585173493] = {
                        ["id"] = 1585173493,
                        ["text"] = "|cff22bb33Blackwing Lair - Completion Bonus|r",
                        ["names"] = "|cffC79C6EMihai|r",
                        ["reason"] = "Completion Bonus",
                        ["datetime"] = 1585173494,
                        ["dkpChangeText"] = "|cff22bb3398 DKP|r",
                        ["time"] = "02:58:14 PM",
                        ["date"] = "03/25/20",
                        ["dkpChange"] = 98,
                        ["raid"] = "Blackwing Lair",
                        ["serverTime"] = 1585173493,
                        ["officer"] = "|cff8787EDSparklenips|r",
                    },
                    [1585188797] = {
                        ["reason"] = "Item Win",
                        ["id"] = 1585188797,
                        ["text"] = "|cffE71D36Item Win - |r|cffa335ee|Hitem:16818::::::::60:::::::|h[Netherwind Belt]|h|r",
                        ["item"] = "|cffa335ee|Hitem:16818::::::::60:::::::|h[Netherwind Belt]|h|r",
                        ["names"] = "|cff40C7EBXyen|r",
                        ["datetime"] = 1585188798,
                        ["dkpChangeText"] = "|cffE71D36-3 DKP|r",
                        ["time"] = "10:13:18 PM",
                        ["date"] = "03/25/20",
                        ["dkpChange"] = -3,
                        ["raid"] = "Blackwing Lair",
                        ["serverTime"] = 1585188797,
                        ["officer"] = "|cffFF7D0ANeekio|r",
                    },
                    [1585200845] = {
                        ["names"] = "|cffC79C6ENightshelf|r",
                        ["id"] = 1585200845,
                        ["text"] = "|cffE71D36Item Win - |rNone",
                        ["dkpChangeText"] = "|cffE71D36-33 DKP|r",
                        ["reason"] = "Item Win",
                        ["datetime"] = 1585200847,
                        ["item"] = "None",
                        ["time"] = "10:34:07 PM",
                        ["date"] = "03/25/20",
                        ["dkpChange"] = -33,
                        ["raid"] = "Blackwing Lair",
                        ["serverTime"] = 1585200845,
                        ["officer"] = "|cff8787EDSparklenips|r",
                    },
                    [1585172991] = {
                        ["id"] = 1585172991,
                        ["text"] = "|cff22bb33Blackwing Lair - Completion Bonus|r",
                        ["names"] = "|cffFF7D0AInebriated|r, |cffFF7D0AInsub|r",
                        ["reason"] = "Completion Bonus",
                        ["datetime"] = 1585172992,
                        ["dkpChangeText"] = "|cff22bb33100 DKP|r",
                        ["time"] = "02:49:52 PM",
                        ["date"] = "03/25/20",
                        ["dkpChange"] = 100,
                        ["raid"] = "Blackwing Lair",
                        ["serverTime"] = 1585172991,
                        ["officer"] = "|cff8787EDSparklenips|r",
                    },
                    [1585791664] = {
                        ["names"] = "|cffFF7D0AEmmyy|r, |cffFF7D0ANeekio|r, |cffFF7D0AMorphintyme|r, |cffABD473Bádtothebow|r, |cffABD473Athico|r, |cffABD473Ithgar|r, |cffABD473Cyskul|r, |cffABD473Bigbootyhoho|r, |cff40C7EBEolith|r, |cff40C7EBJakemeoff|r, |cff40C7EBVeriandra|r, |cff40C7EBFlatulent|r, |cff40C7EBStitchess|r, |cff40C7EBNeckbeardo|r, |cff40C7EBXyen|r, |cffF58CBAPapasquach|r, |cffF58CBAAdvanty|r, |cffF58CBAOdin|r, |cffF58CBAMaryjohanna|r, |cffFFFFFFBallour|r, |cffFFFFFFPuffhead|r, |cffFFFFFFPriesticuffs|r, |cffFFFFFFTurbohealz|r, |cffFFFFFFPhilonious|r, |cffFFF569Slipperyjohn|r, |cffFFF569Mcstaberson|r, |cffFFF569Sicarrio|r, |cffFFF569Mariku|r, |cff8787EDEdgyboi|r, |cff8787EDLindo|r, |cff8787EDRaspütin|r, |cff8787EDSparklenips|r, |cff8787EDDalia|r, |cffC79C6EGoobimus|r, |cffC79C6EKalita|r, |cffC79C6EGirlslayer|r, |cffC79C6EOragan|r, |cffC79C6EQew|r, |cffC79C6EHotdogbroth|r, |cffC79C6EShinynickels|r",
                        ["id"] = 1585791664,
                        ["text"] = "|cff22bb33Blackwing Lair - Razorgore the Untamed|r",
                        ["bossKill"] = "Razorgore the Untamed",
                        ["reason"] = "Boss Kill",
                        ["datetime"] = 1585791666,
                        ["time"] = "06:41:06 PM",
                        ["dkpChangeText"] = "|cff22bb3310 DKP|r",
                        ["date"] = "04/01/20",
                        ["dkpChange"] = 10,
                        ["raid"] = "Blackwing Lair",
                        ["serverTime"] = 1585791664,
                        ["officer"] = "|cff8787EDSparklenips|r",
                    },
                    [1585689372] = {
                        ["reason"] = "Boss Kill",
                        ["id"] = 1585689372,
                        ["text"] = "|cff22bb33Onyxia's Lair - Onyxia|r",
                        ["bossKill"] = "Onyxia",
                        ["names"] = "|cff40C7EBUrkh|r",
                        ["datetime"] = 1585689373,
                        ["dkpChangeText"] = "|cff22bb3310 DKP|r",
                        ["time"] = "02:16:13 PM",
                        ["date"] = "03/31/20",
                        ["dkpChange"] = 10,
                        ["raid"] = "Onyxia's Lair",
                        ["serverTime"] = 1585689372,
                        ["officer"] = "|cff8787EDSparklenips|r",
                    },
                    [1585200798] = {
                        ["names"] = "|cffABD473Stellâ|r",
                        ["id"] = 1585200798,
                        ["text"] = "|cffE71D36Item Win - |rNone",
                        ["dkpChangeText"] = "|cffE71D36-11 DKP|r",
                        ["reason"] = "Item Win",
                        ["datetime"] = 1585200800,
                        ["item"] = "None",
                        ["time"] = "10:33:20 PM",
                        ["date"] = "03/25/20",
                        ["dkpChange"] = -11,
                        ["raid"] = "Blackwing Lair",
                        ["serverTime"] = 1585200798,
                        ["officer"] = "|cff8787EDSparklenips|r",
                    },
                    [1585791781] = {
                        ["names"] = "|cff40C7EBXyen|r",
                        ["id"] = 1585791781,
                        ["text"] = "|cffE71D36Item Win - |rNone",
                        ["item"] = "None",
                        ["reason"] = "Item Win",
                        ["datetime"] = 1585791783,
                        ["dkpChangeText"] = "|cffE71D36-12 DKP|r",
                        ["time"] = "06:43:03 PM",
                        ["date"] = "04/01/20",
                        ["dkpChange"] = -12,
                        ["raid"] = "Blackwing Lair",
                        ["serverTime"] = 1585791781,
                        ["officer"] = "|cff8787EDSparklenips|r",
                    },
                    [1585173011] = {
                        ["id"] = 1585173011,
                        ["text"] = "|cff22bb33Blackwing Lair - Completion Bonus|r",
                        ["names"] = "|cffABD473Bigbootyhoho|r",
                        ["reason"] = "Completion Bonus",
                        ["datetime"] = 1585173012,
                        ["dkpChangeText"] = "|cff22bb3360 DKP|r",
                        ["time"] = "02:50:12 PM",
                        ["date"] = "03/25/20",
                        ["dkpChange"] = 60,
                        ["raid"] = "Blackwing Lair",
                        ["serverTime"] = 1585173011,
                        ["officer"] = "|cff8787EDSparklenips|r",
                    },
                    [1585173274] = {
                        ["id"] = 1585173274,
                        ["text"] = "|cff22bb33Blackwing Lair - Completion Bonus|r",
                        ["names"] = "|cffFFF569Sicarrio|r",
                        ["reason"] = "Completion Bonus",
                        ["datetime"] = 1585173275,
                        ["dkpChangeText"] = "|cff22bb3350 DKP|r",
                        ["time"] = "02:54:35 PM",
                        ["date"] = "03/25/20",
                        ["dkpChange"] = 50,
                        ["raid"] = "Blackwing Lair",
                        ["serverTime"] = 1585173274,
                        ["officer"] = "|cff8787EDSparklenips|r",
                    },
                    [1585792188] = {
                        ["names"] = "|cffFFFFFFBallour|r",
                        ["id"] = 1585792188,
                        ["text"] = "|cffE71D36Item Win - |rNone",
                        ["item"] = "None",
                        ["reason"] = "Item Win",
                        ["datetime"] = 1585792190,
                        ["dkpChangeText"] = "|cffE71D36-67 DKP|r",
                        ["time"] = "06:49:50 PM",
                        ["date"] = "04/01/20",
                        ["dkpChange"] = -67,
                        ["raid"] = "Blackwing Lair",
                        ["serverTime"] = 1585792188,
                        ["officer"] = "|cff8787EDSparklenips|r",
                    },
                    [1585194720] = {
                        ["names"] = "|cffFF7D0AInebriated|r, |cffFF7D0AInsub|r, |cffABD473Stellâ|r, |cffABD473Wahcha|r, |cffABD473Ugro|r, |cffABD473Primera|r, |cff40C7EBSuprarz|r, |cff40C7EBEolith|r, |cff40C7EBOxford|r, |cff40C7EBZukohere|r, |cff40C7EBBugaboom|r, |cff40C7EBChipgizmo|r, |cffF58CBAOnes|r, |cffF58CBAGneissguy|r, |cffF58CBALawduk|r, |cffF58CBAPapasquach|r, |cffFFFFFFBubblè|r, |cffFFFFFFMongous|r, |cffFFFFFFEvangelina|r, |cffFFFFFFHoneypot|r, |cffFFFFFFFawntine|r, |cffFFFFFFRez|r, |cffFFF569Tuggernuts|r, |cffFFF569Inigma|r, |cffFFF569Mystile|r, |cffFFF569Mujinn|r, |cff8787EDSparklenips|r, |cff8787EDThepurple|r, |cff8787EDSoftfondle|r, |cff8787EDLittledubs|r, |cff8787EDVarix|r, |cffC79C6EShinynickels|r, |cffC79C6EGalagus|r, |cffC79C6ELuckerdawg|r, |cffC79C6ESnaildaddy|r, |cffC79C6EJeffry|r, |cffC79C6ENightshelf|r, |cffC79C6ERetkin|r, |cffC79C6ETokentoken|r, |cffC79C6EReina|r",
                        ["id"] = 1585194720,
                        ["text"] = "|cff22bb33Blackwing Lair - Razorgore the Untamed|r",
                        ["bossKill"] = "Razorgore the Untamed",
                        ["reason"] = "Boss Kill",
                        ["datetime"] = 1585194722,
                        ["time"] = "08:52:02 PM",
                        ["dkpChangeText"] = "|cff22bb3310 DKP|r",
                        ["date"] = "03/25/20",
                        ["dkpChange"] = 10,
                        ["raid"] = "Blackwing Lair",
                        ["serverTime"] = 1585194720,
                        ["officer"] = "|cff8787EDSparklenips|r",
                    },
                    [1585199079] = {
                        ["names"] = "|cffFF7D0AInsub|r, |cffFF7D0AInebriated|r, |cffABD473Primera|r, |cffABD473Wahcha|r, |cffABD473Stellâ|r, |cffABD473Ugro|r, |cff40C7EBSuprarz|r, |cff40C7EBOxford|r, |cff40C7EBEolith|r, |cff40C7EBBugaboom|r, |cff40C7EBZukohere|r, |cff40C7EBChipgizmo|r, |cffF58CBAOnes|r, |cffF58CBAPapasquach|r, |cffF58CBAGneissguy|r, |cffF58CBALawduk|r, |cffFFFFFFMongous|r, |cffFFFFFFRez|r, |cffFFFFFFBubblè|r, |cffFFFFFFEvangelina|r, |cffFFFFFFHoneypot|r, |cffFFFFFFFawntine|r, |cffFFF569Tuggernuts|r, |cffFFF569Inigma|r, |cffFFF569Mujinn|r, |cffFFF569Mystile|r, |cff8787EDSparklenips|r, |cff8787EDLittledubs|r, |cff8787EDThepurple|r, |cff8787EDSoftfondle|r, |cff8787EDVarix|r, |cffC79C6ESnaildaddy|r, |cffC79C6EJeffry|r, |cffC79C6EReina|r, |cffC79C6EGalagus|r, |cffC79C6EShinynickels|r, |cffC79C6ERetkin|r, |cffC79C6ETokentoken|r, |cffC79C6ENightshelf|r, |cffC79C6ELuckerdawg|r",
                        ["id"] = 1585199079,
                        ["text"] = "|cff22bb33Blackwing Lair - Firemaw|r",
                        ["bossKill"] = "Firemaw",
                        ["reason"] = "Boss Kill",
                        ["datetime"] = 1585199081,
                        ["time"] = "10:04:41 PM",
                        ["dkpChangeText"] = "|cff22bb3310 DKP|r",
                        ["date"] = "03/25/20",
                        ["dkpChange"] = 10,
                        ["raid"] = "Blackwing Lair",
                        ["serverTime"] = 1585199079,
                        ["officer"] = "|cff8787EDSparklenips|r",
                    },
                    [1585173541] = {
                        ["id"] = 1585173541,
                        ["text"] = "|cff22bb33Blackwing Lair - Completion Bonus|r",
                        ["names"] = "|cff40C7EBFlatulent|r",
                        ["reason"] = "Completion Bonus",
                        ["datetime"] = 1585173542,
                        ["dkpChangeText"] = "|cff22bb3355 DKP|r",
                        ["time"] = "02:59:02 PM",
                        ["date"] = "03/25/20",
                        ["dkpChange"] = 55,
                        ["raid"] = "Blackwing Lair",
                        ["serverTime"] = 1585173541,
                        ["officer"] = "|cff8787EDSparklenips|r",
                    },
                    [1585798801] = {
                        ["reason"] = "Item Win",
                        ["id"] = 1585798801,
                        ["text"] = "|cffE71D36Item Win - |rNone",
                        ["dkpChangeText"] = "|cffE71D36-18 DKP|r",
                        ["names"] = "|cff40C7EBChipgizmo|r",
                        ["datetime"] = 1585798801,
                        ["item"] = "None",
                        ["time"] = "08:40:01 PM",
                        ["date"] = "04/01/20",
                        ["dkpChange"] = -18,
                        ["raid"] = "Blackwing Lair",
                        ["serverTime"] = 1585798801,
                        ["officer"] = "|cffC79C6ESnaildaddy|r",
                    },
                    [1585198539] = {
                        ["names"] = "|cffABD473Wahcha|r",
                        ["id"] = 1585198539,
                        ["text"] = "|cffE71D36Item Win - |rNone",
                        ["dkpChangeText"] = "|cffE71D36-70 DKP|r",
                        ["reason"] = "Item Win",
                        ["datetime"] = 1585198541,
                        ["item"] = "None",
                        ["time"] = "09:55:41 PM",
                        ["date"] = "03/25/20",
                        ["dkpChange"] = -70,
                        ["raid"] = "Blackwing Lair",
                        ["serverTime"] = 1585198539,
                        ["officer"] = "|cff8787EDSparklenips|r",
                    },
                    [1585793691] = {
                        ["names"] = "|cffFFFFFFTurbohealz|r",
                        ["id"] = 1585793691,
                        ["text"] = "|cffE71D36Item Win - |rNone",
                        ["item"] = "None",
                        ["reason"] = "Item Win",
                        ["datetime"] = 1585793693,
                        ["dkpChangeText"] = "|cffE71D36-103 DKP|r",
                        ["time"] = "07:14:53 PM",
                        ["date"] = "04/01/20",
                        ["dkpChange"] = -103,
                        ["raid"] = "Blackwing Lair",
                        ["serverTime"] = 1585793691,
                        ["officer"] = "|cff8787EDSparklenips|r",
                    },
                    [1585693212] = {
                        ["id"] = 1585693212,
                        ["text"] = "|cff22bb33Molten Core - On Time Bonus|r",
                        ["names"] = "|cff40C7EBUrkh|r",
                        ["reason"] = "On Time Bonus",
                        ["datetime"] = 1585693213,
                        ["time"] = "03:20:13 PM",
                        ["dkpChangeText"] = "|cff22bb3310 DKP|r",
                        ["date"] = "03/31/20",
                        ["dkpChange"] = 10,
                        ["raid"] = "Molten Core",
                        ["serverTime"] = 1585693212,
                        ["officer"] = "|cff8787EDSparklenips|r",
                    },
                    [1585806220] = {
                        ["names"] = "|cffFFF569Aku|r",
                        ["id"] = 1585806220,
                        ["text"] = "|cffE71D36Item Win - |rNone",
                        ["time"] = "10:43:40 PM",
                        ["reason"] = "Item Win",
                        ["datetime"] = 1585806220,
                        ["dkpChangeText"] = "|cffE71D36-145 DKP|r",
                        ["item"] = "None",
                        ["date"] = "04/01/20",
                        ["dkpChange"] = -145,
                        ["raid"] = "Blackwing Lair",
                        ["serverTime"] = 1585806220,
                        ["officer"] = "|cffC79C6ESnaildaddy|r",
                    },
                    [1585191670] = {
                        ["names"] = "|cffFFF569Mariku|r",
                        ["id"] = 1585191670,
                        ["text"] = "|cffE71D36Item Win - |r|cffa335ee|Hitem:16906::::::::60:::::::|h[Bloodfang Boots]|h|r",
                        ["dkpChangeText"] = "|cffE71D36-8 DKP|r",
                        ["reason"] = "Item Win",
                        ["datetime"] = 1585191671,
                        ["time"] = "11:01:11 PM",
                        ["item"] = "|cffa335ee|Hitem:16906::::::::60:::::::|h[Bloodfang Boots]|h|r",
                        ["date"] = "03/25/20",
                        ["dkpChange"] = -8,
                        ["raid"] = "Blackwing Lair",
                        ["serverTime"] = 1585191670,
                        ["officer"] = "|cffFF7D0ANeekio|r",
                    },
                    [1585197330] = {
                        ["reason"] = "Item Win",
                        ["id"] = 1585197330,
                        ["text"] = "|cffE71D36Item Win - |r|cffa335ee|Hitem:16923::::::::60:::::::|h[Robes of Transcendence]|h|r",
                        ["time"] = "12:35:31 AM",
                        ["names"] = "|cffFFFFFFBallour|r",
                        ["datetime"] = 1585197331,
                        ["item"] = "|cffa335ee|Hitem:16923::::::::60:::::::|h[Robes of Transcendence]|h|r",
                        ["dkpChangeText"] = "|cffE71D36-94 DKP|r",
                        ["date"] = "03/26/20",
                        ["dkpChange"] = -94,
                        ["raid"] = "Blackwing Lair",
                        ["serverTime"] = 1585197330,
                        ["officer"] = "|cffFF7D0ANeekio|r",
                    },
                    [1585693193] = {
                        ["id"] = 1585693193,
                        ["text"] = "|cff22bb33Molten Core - On Time Bonus|r",
                        ["names"] = "|cff40C7EBUrkh|r",
                        ["reason"] = "On Time Bonus",
                        ["datetime"] = 1585693194,
                        ["time"] = "03:19:54 PM",
                        ["dkpChangeText"] = "|cff22bb3310 DKP|r",
                        ["date"] = "03/31/20",
                        ["dkpChange"] = 10,
                        ["raid"] = "Molten Core",
                        ["serverTime"] = 1585693193,
                        ["officer"] = "|cff8787EDSparklenips|r",
                    },
                    [1585693467] = {
                        ["id"] = 1585693467,
                        ["text"] = "|cff22bb33Molten Core - On Time Bonus|r",
                        ["names"] = "|cff40C7EBUrkh|r",
                        ["reason"] = "On Time Bonus",
                        ["datetime"] = 1585693468,
                        ["time"] = "03:24:28 PM",
                        ["dkpChangeText"] = "|cff22bb3310 DKP|r",
                        ["date"] = "03/31/20",
                        ["dkpChange"] = 10,
                        ["raid"] = "Molten Core",
                        ["serverTime"] = 1585693467,
                        ["officer"] = "|cff8787EDSparklenips|r",
                    },
                },
            },
        }
}

-- Check banks latest officer note. If the value is higher than your value, then you can assume you're out of date.
-- If you are out of date, look to see what officers are online, and request a push from their database, if their last
-- edit is equal to the banks last edit or greater than your own. If not, continue until you find one.
--

function Import:AcceptData(reqData)
    print('Import received from addon version ', reqData.addon_version)

    if reqData.full then -- THIS IS A FULL OVERWRITE
        Import:AcceptFullDatabase(reqData)
        PDKP:Print('Full database overwrite in progress')
    else -- THIS IS A MERGE
        PDKP:Print('Merging data...')
        local reqDKP = reqData.dkpDB;
        local reqGuild = reqData.guildDB;

--        local reqNumOfMembers, reqMembers = reqGuild.numOfMembers, reqGuild.members;
        local reqLastEdit, reqHistory = reqDKP.lastEdit, reqDKP.history;
        local reqAll, reqDeleted = reqHistory.all, reqHistory.deleted



        local members = Guild.members;
        local history = DKP.dkpDB.history;
        local allHistory = history.all;
        local deleted = history.deleted;
        local lastEdit = DKP.dkpDB.lastEdit;

        local function updateEntry(entry)
            if entry['deleted'] then
                Util:Debug("Processing as a delete")
                return DKP:DeleteEntry(entry, true)
            elseif entry['edited'] then
                Util:Debug("Processing as an edit")
            else
                Util:Debug('Processing an addition')
                local raid = entry['raid']
                local entryKey = entry['id']

                for _, memberName in pairs(entry['members']) do
                    local member = members[memberName];

                    if member ~= nil then
                        local isInDeleted, isInHistory = member:CheckForEntryHistory(entry)

                        if isInDeleted == false and isInHistory == false then
                            local dkp = member:GetDKP(raid, 'all')
                            if dkp then
                                if dkp.entries == nil then member.dkp[raid].entries = {} end
                                table.insert(dkp.entries, entryKey)
                                dkp.previousTotal = dkp.total;
                                dkp.total = dkp.total + entry['dkpChange']
                                if dkp.total < 0 then dkp.total = 0 end;

                                if member.bName then -- update the player visually.
                                    local dkpText = _G[member.bName .. '_col3']
                                    if dkpText:IsVisible() then
                                        dkpText:SetText(dkp.total)
                                    end
                                end
                                member:Save() -- Update the database locally.
                            else
                                Util:ThrowError('Sync Error: Could not find dkp for '.. member.name)
                            end
                        else
                            if isInDeleted then
                                Util:Debug('This entry was recently deleted, skipping.')
                            elseif isInHistory then
                                Util:Debug('This entry already exists, skipping.')
                            end
                        end
                    end
                end
                if entryKey > lastEdit then DKP.dkpDB.lastEdit = entryKey; end
            end
        end
        if #reqHistory == 1 and (reqAll == nil and reqDeleted == nil) then -- This is a single entry update.
            local entry = DKP:FixEntryMembers(reqHistory[1])
            updateEntry(entry)
        elseif reqAll and reqDeleted and (#reqAll > 1 or #reqDeleted > 1) then -- we have the [deleted] and [all] tables in this table.
            for key, entry in pairs(reqAll) do
                Util:Debug('Processing entry...' .. key)
                if key ~= 1 and entry ~= nil then
                    entry = DKP:FixEntryMembers(entry)
                    updateEntry(entry)
                end
            end
            for _, entryID in pairs(reqDeleted) do
                Util:Debug('Processing deleted entry: ' .. entryID)
                local entry = reqAll[entryID]
                if entry ~= nil then
                    entry['deleted'] = true
                    updateEntry(entry)
                end
            end
        end
    end

   if GUI.pdkp_frame and GUI.pdkp_frame:IsVisible() then
       GUI:pdkp_dkp_table_sort('dkpTotal')
       DKP:ChangeDKPSheets(DKP.dkpDB.currentDB, true)
       pdkp_dkp_scrollbar_Update()
       pdkp_dkp_table_filter()
       GUI:pdkp_dkp_table_sort('dkpTotal')

       GUI.pushFrame:Hide()
   end

    PDKP:Print("The DKP push has completed successfully")
end

function Import:AcceptFullDatabase(data)
    local guildData = data.guildDB
    local dkpData = data.dkpDB

    Guild.db.members = guildData.members

    DKP.dkpDB.lastEdit = dkpData.lastEdit;
    DKP.dkpDB.history = dkpData.history;
    DKP.dkpDB.members = dkpData.members;
end

function Import:GetHistoryKeys()
    local pushHistory = { ['all'] = {}, ['deleted'] = {}, }
    for entryKey, entry in pairs(DKP.dkpDB.history) do
        pushHistory['all'][entryKey] = true
    end
    for key, entryKey in pairs(DKP.dkpDB.deleted) do
        pushHistory['deleted'][entryKey] = true
    end
    return pushHistory
end

function Import:RequestData(officer)
    Util:Debug("Requesting data from " .. officer['formattedName'] .. ' this may take a few minutes...')
    local historyKeys = Import:GetHistoryKeys()

    -- TODO Hook this function up!

    Comms:SendCommsMessage('pdkpPushRequest', PDKP:Serialize(''), 'WHISPER', officer['name'], 'BULK')
end

function Import:TestDataImport()
    Import:AcceptData(pdkpTestDatabase)
end

